@@ -41,7 +41,9 @@ extern char e_range[];		/* #-1 OUT OF RANGE */
  */
 
 extern int parse_boolean(char const *str);
-extern dbref parse_dbref(const char *s);
+extern dbref parse_dbref(char const *str);
+extern dbref qparse_dbref(char const *str);
+extern dbref parse_objid(char const *str);
 
 #define parse_integer(str) strtol(str, NULL, 10)
 #define parse_uinteger(str) strtoul(str, NULL, 10)
@@ -62,11 +64,13 @@ char *unparse_dbref(dbref num);
 char *unparse_integer(int num);
 char *unparse_uinteger(unsigned int num);
 char *unparse_number(NVAL num);
+char *unparse_types(int type);
 
 /* The following routines all take strings as arguments, and return
  * true iff the string is a valid representation of the appropriate type.
  */
 int is_dbref(char const *str);
+int is_objid(char const *str);
 
 int is_integer(char const *str);
 int is_uinteger(char const *str);
@@ -74,6 +78,9 @@ int is_boolean(char const *str);
 
 /* Split a sep-delimited string into individual elements */
 extern int list2arr(char *r[], int max, char *list, char sep);
+/* The reverse */
+extern void arr2list(char *r[], int max, char *list, char **lp, char *sep);
+
 
 
 
@@ -82,6 +89,7 @@ extern int list2arr(char *r[], int max, char *list, char sep);
 typedef struct fun FUN;
 #define HAVE_FUN_DEFINED
 #endif
+/** Common declaration for softcode function implementations */
 #define FUNCTION(fun_name) \
   /* ARGSUSED */ /* try to keep lint happy */ \
   void fun_name (FUN *fun, char *buff, char **bp, int nargs, char *args[], \
@@ -170,6 +178,7 @@ int process_expression(char *buff, char **bp, char const **str,
 #define PE_FUNCTION_MANDATORY   0x00000040
 #define PE_LITERAL              0x00000100
 #define PE_DOLLAR               0x00000200
+#define PE_DEBUG                0x00000400
 
 #define PE_DEFAULT (PE_COMPRESS_SPACES | PE_STRIP_BRACES | \
                     PE_EVALUATE | PE_FUNCTION_CHECK)
