@@ -1,4 +1,10 @@
-/* atr_tab.c */
+/**
+ * \file atr_tab.c
+ *
+ * \brief The table of standard attributes and code to manipulate it.
+ *
+ *
+ */
 
 #include "config.h"
 
@@ -17,15 +23,18 @@
 #include "parse.h"
 #include "confmagic.h"
 
-typedef struct atr_alias ATRALIAS;
 
-struct atr_alias {
-  const char *alias;
-  const char *realname;
-};
+/** An alias for an attribute.
+ */
+typedef struct atr_alias {
+  const char *alias;		/**< The alias. */
+  const char *realname;		/**< The attribute's canonical name. */
+} ATRALIAS;
 
+/** Prefix table for standard attribute names */
 PTAB ptab_attrib;
 
+/** Attribute flags */
 PRIV attr_privs[] = {
   {"no_command", '$', AF_NOPROG, AF_NOPROG},
   {"no_inherit", 'i', AF_PRIVATE, AF_PRIVATE},
@@ -39,7 +48,14 @@ PRIV attr_privs[] = {
   {"case", 'C', AF_CASE, AF_CASE},
   {"locked", '+', AF_LOCKED, AF_LOCKED},
   {"safe", 'S', AF_SAFE, AF_SAFE},
+  {"internal", '\0', AF_INTERNAL, AF_INTERNAL},
   {"prefixmatch", '\0', AF_PREFIXMATCH, AF_PREFIXMATCH},
+  {"veiled", 'V', AF_VEILED, AF_VEILED},
+  {"debug", 'b', AF_DEBUG, AF_DEBUG},
+  {"public", 'p', AF_PUBLIC, AF_PUBLIC},
+  {"nearby", 'n', AF_NEARBY, AF_NEARBY},
+  {"noname", 'N', AF_NONAME, AF_NONAME},
+  {"nospace", 's', AF_NOSPACE, AF_NOSPACE},
   {NULL, '\0', 0, 0}
 };
 
@@ -51,37 +67,39 @@ PRIV attr_privs[] = {
 static ATTR *aname_find_exact(const char *name);
 void init_aname_table(void);
 
+/** Attribute table lookup by name or alias.
+ * given an attribute name, look it up in the complete attribute table
+ * (real names plus aliases), and return the appropriate real attribute.
+ */
 ATTR *
 aname_hash_lookup(const char *name)
 {
-  /* given an attribute name, look it up in the complete attribute table
-   * (real names plus aliases), and return the appropriate real attribute.
-   */
   ATTR *ap;
   /* Exact matches always work */
   if ((ap = (ATTR *) ptab_find_exact(&ptab_attrib, name)))
     return ap;
   /* Prefix matches work if the attribute is AF_PREFIXMATCH */
-  if ((ap = (ATTR *) ptab_find(&ptab_attrib, name)) &&
-      (AL_FLAGS(ap) & AF_PREFIXMATCH))
+  if ((ap = (ATTR *) ptab_find(&ptab_attrib, name)) && AF_Prefixmatch(ap))
     return ap;
   return NULL;
 }
 
+/** Build the basic attribute table.
+ */
 void
 init_aname_table(void)
 {
   ATTR *ap;
 
   ptab_init(&ptab_attrib);
-
-  /* build the basic attr table */
   ptab_start_inserts(&ptab_attrib);
   for (ap = attr; ap->name; ap++)
     ptab_insert(&ptab_attrib, ap->name, ap);
   ptab_end_inserts(&ptab_attrib);
 }
 
+/** Associate a new alias with an existing attribute.
+ */
 int
 alias_attribute(const char *atr, const char *alias)
 {
@@ -112,13 +130,21 @@ aname_find_exact(const char *name)
 }
 
 
-/* Add an attribute to the hash table, given its name and permissions */
+/** Add new standard attributes, or change permissions on them.
+ * \verbatim
+ * Given the name and permission string for an attribute, add it to
+ * the attribute table (or modify the permissions if it's already
+ * there). Permissions may be changed retroactively, which modifies
+ * permissions on any copies of that attribute set on objects in the
+ * database. This is the top-level code for @attribute/access.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name the attribute name.
+ * \param perms a string of attribute permissions, space-separated.
+ * \param retroactive if true, apply the permissions retroactively.
+ */
 void
-do_attribute_access(player, name, perms, retroactive)
-    dbref player;
-    char *name;
-    char *perms;
-    int retroactive;
+do_attribute_access(dbref player, char *name, char *perms, int retroactive)
 {
   ATTR *ap, *ap2;
   int flags = 0;
@@ -137,9 +163,9 @@ do_attribute_access(player, name, perms, retroactive)
   }
   upcasestr(name);
   /* Is this attribute already in the table? */
-  ap = ptab_find_exact(&ptab_attrib, name);
+  ap = (ATTR *) ptab_find_exact(&ptab_attrib, name);
   if (ap) {
-    if (ap->flags & AF_INTERNAL) {
+    if (AF_Internal(ap)) {
       /* Don't muck with internal attributes */
       notify(player, T("That attribute's permissions can not be changed."));
       return;
@@ -154,7 +180,7 @@ do_attribute_access(player, name, perms, retroactive)
       return;
     }
     AL_NAME(ap) = strdup(name);
-    AL_STR(ap) = NULL;
+    ap->data = NULL_CHUNK_REFERENCE;
   }
   AL_FLAGS(ap) = flags;
   AL_CREATOR(ap) = player;
@@ -184,10 +210,15 @@ do_attribute_access(player, name, perms, retroactive)
 }
 
 
+/** Delete an attribute from the attribute table.
+ * \verbatim
+ * Top-level function for @attrib/delete.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name the name of the attribute to delete.
+ */
 void
-do_attribute_delete(player, name)
-    dbref player;
-    char *name;
+do_attribute_delete(dbref player, char *name)
 {
   ATTR *ap;
 
@@ -209,11 +240,16 @@ do_attribute_delete(player, name)
   return;
 }
 
+/** Rename an attribute in the attribute table.
+ * \verbatim
+ * Top-level function for @attrib/rename.
+ * \endverbatim
+ * \param player the enactor.
+ * \param old the name of the attribute to rename.
+ * \param newname the new name (surprise!)
+ */
 void
-do_attribute_rename(player, old, newname)
-    dbref player;
-    char *old;
-    char *newname;
+do_attribute_rename(dbref player, char *old, char *newname)
 {
   ATTR *ap;
   if (!old || !*old || !newname || !*newname) {
@@ -250,11 +286,15 @@ do_attribute_rename(player, old, newname)
   return;
 }
 
-/* Info on an attribute in the table */
+/** Display information on an attribute from the table.
+ * \verbatim
+ * Top-level function for @attribute.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name the name of the attribute.
+ */
 void
-do_attribute_info(player, name)
-    dbref player;
-    char *name;
+do_attribute_info(dbref player, char *name)
 {
   ATTR *ap;
   if (!name || !*name) {
@@ -275,17 +315,26 @@ do_attribute_info(player, name)
   return;
 }
 
+/** Display a list of standard attributes.
+ * \verbatim
+ * Top-level function for @list/attribs.
+ * \endverbatim
+ * \param player the enactor.
+ * \param lc if true, display the list in lowercase; otherwise uppercase.
+ */
 void
-do_list_attribs(player, lc)
-    dbref player;
-    int lc;
+do_list_attribs(dbref player, int lc)
 {
   char *b = list_attribs();
   notify_format(player, "Attribs: %s", lc ? strlower(b) : b);
 }
 
+/** Return a list of standard attributes.
+ * This functions returns the list of standard attributes, separated by
+ * spaces, in a statically allocated buffer.
+ */
 char *
-list_attribs()
+list_attribs(void)
 {
   ATTR *ap;
   const char *ptrs[BUFFER_LEN / 2];
@@ -307,3 +356,18 @@ list_attribs()
   *bp = '\0';
   return buff;
 }
+
+/** Attr things to be done after the config file is loaded but before
+ * objects are restarted.
+ */
+void
+attr_init_postconfig(void)
+{
+  ATTR *a;
+  /* read_remote_desc affects AF_NEARBY flag on DESCRIBE attribute */
+  a = aname_hash_lookup("DESCRIBE");
+  if (READ_REMOTE_DESC)
+    a->flags &= ~AF_NEARBY;
+  else
+    a->flags |= AF_NEARBY;
+}
