@@ -1,25 +1,27 @@
-
-/* cmds.c
- * Actual definitions of the commands
- * Parsing is handled in command.c
+/**
+ * \file cmds.c
+ *
+ * \brief Definitions of commands.
+ * This file is a set of functions that defines commands. The parsing
+ * of commands is elsewhere (command.c), as are the implementations
+ * of most of the commands (throughout the source.)
+ *
  */
+
 #include "copyrite.h"
 #include "config.h"
 
 #include <string.h>
 
 #include "conf.h"
+#include "externs.h"
 #include "dbdefs.h"
 #include "mushdb.h"
-#include "externs.h"
 #include "match.h"
 #include "game.h"
 #include "attrib.h"
 #include "extmail.h"
 #include "malias.h"
-#ifdef CHAT_SYSTEM
-#include "extchat.h"
-#endif
 #include "getpgsiz.h"
 #include "parse.h"
 #include "access.h"
@@ -35,9 +37,9 @@
 void do_poor(dbref player, char *arg1);
 void do_list_memstats(dbref player);
 
-#define DOL_MAP 1
-#define DOL_NOTIFY 2
-#define DOL_DELIM 4
+#define DOL_MAP 1		/**< dolist/map bitflag */
+#define DOL_NOTIFY 2		/**< dolist/notify bitflag */
+#define DOL_DELIM 4		/**< dolist/delim bitflag */
 
 void do_dolist(dbref player, char *list, char *command,
 	       dbref cause, unsigned int flags);
@@ -46,24 +48,19 @@ void do_writelog(dbref player, char *str, int ltype);
 void do_readcache(dbref player);
 void do_scan(dbref player, char *command, int flag);
 void do_uptime(dbref player, int mortal);
-extern int config_set(const char *opt, char *val, int source);
+extern int config_set(const char *opt, char *val, int source, int restrictions);
 
-/* From command.c */
+/** Is there a right-hand side of the equal sign? From command.c */
 extern int rhs_present;
-void do_hook(dbref player, char *command, char *obj, char *attrname,
-	     int hook_type);
 
 COMMAND (cmd_allhalt) {
   do_allhalt(player);
 }
 
-#ifdef QUOTA
 COMMAND (cmd_allquota) {
   do_allquota(player, arg_left, SW_ISSET(sw, SWITCH_QUIET));
 }
 
-#endif
-
 COMMAND (cmd_atrlock) {
   do_atrlock(player, arg_left, arg_right);
 }
@@ -86,78 +83,31 @@ COMMAND (cmd_atrchown) {
 
 COMMAND (cmd_boot) {
   if (SW_ISSET(sw, SWITCH_PORT))
-    do_boot(player, arg_left, 1);
+    do_boot(player, arg_left, BOOT_DESC);
   else if (SW_ISSET(sw, SWITCH_ME))
-    do_boot(player, (char *) NULL, 2);
+    do_boot(player, (char *) NULL, BOOT_SELF);
+  else if (SW_ISSET(sw, SWITCH_SILENT))
+    do_boot(player, arg_left, BOOT_SILENT);
   else
-    do_boot(player, arg_left, 0);
+    do_boot(player, arg_left, BOOT_NAME);
 }
 
-extern int break_called;
 COMMAND (cmd_break) {
-  if (parse_boolean(arg_left))
-    break_called = 1;
+  if (parse_boolean(arg_left)) {
+    global_eval_context.break_called = 1;
+    if (arg_right && *arg_right)
+      strcpy(global_eval_context.break_replace, arg_right);
+  }
 }
 
-#ifdef CHAT_SYSTEM
-COMMAND (cmd_cemit) {
-  do_cemit(player, arg_left, arg_right, SW_ISSET(sw, SWITCH_NOISY));
+COMMAND (cmd_assert) {
+  if (!parse_boolean(arg_left)) {
+    global_eval_context.break_called = 1;
+    if (arg_right && *arg_right)
+      strcpy(global_eval_context.break_replace, arg_right);
+  }
 }
 
-COMMAND (cmd_channel) {
-  if (switches)
-    do_channel(player, arg_left, arg_right, switches);
-  else if (SW_ISSET(sw, SWITCH_LIST))
-    do_channel_list(player, arg_left);
-
-  else if (SW_ISSET(sw, SWITCH_ADD))
-    do_chan_admin(player, arg_left, arg_right, 0);
-  else if (SW_ISSET(sw, SWITCH_DELETE))
-    do_chan_admin(player, arg_left, arg_right, 1);
-  else if (SW_ISSET(sw, SWITCH_NAME))
-    do_chan_admin(player, arg_left, arg_right, 2);
-  else if (SW_ISSET(sw, SWITCH_RENAME))
-    do_chan_admin(player, arg_left, arg_right, 2);
-  else if (SW_ISSET(sw, SWITCH_PRIVS))
-    do_chan_admin(player, arg_left, arg_right, 3);
-  else if (SW_ISSET(sw, SWITCH_QUIET))
-    do_chan_admin(player, arg_left, arg_right, 4);
-  else if (SW_ISSET(sw, SWITCH_NOISY))
-    do_chan_admin(player, arg_left, "n", 4);
-  else if (SW_ISSET(sw, SWITCH_DECOMPILE))
-    do_chan_decompile(player, arg_left, SW_ISSET(sw, SWITCH_BRIEF));
-  else if (SW_ISSET(sw, SWITCH_DESCRIBE))
-    do_chan_desc(player, arg_left, arg_right);
-  else if (SW_ISSET(sw, SWITCH_TITLE))
-    do_chan_title(player, arg_left, arg_right);
-  else if (SW_ISSET(sw, SWITCH_CHOWN))
-    do_chan_chown(player, arg_left, arg_right);
-  else if (SW_ISSET(sw, SWITCH_WIPE))
-    do_chan_wipe(player, arg_left);
-  else if (SW_ISSET(sw, SWITCH_MUTE))
-    do_chan_user_flags(player, arg_left, arg_right, 0, 0);
-  else if (SW_ISSET(sw, SWITCH_UNMUTE))
-    do_chan_user_flags(player, arg_left, "n", 0, 0);
-  else if (SW_ISSET(sw, SWITCH_HIDE))
-    do_chan_user_flags(player, arg_left, arg_right, 1, 0);
-  else if (SW_ISSET(sw, SWITCH_UNHIDE))
-    do_chan_user_flags(player, arg_left, "n", 1, 0);
-  else if (SW_ISSET(sw, SWITCH_GAG))
-    do_chan_user_flags(player, arg_left, arg_right, 2, 0);
-  else if (SW_ISSET(sw, SWITCH_UNGAG))
-    do_chan_user_flags(player, arg_left, "n", 2, 0);
-  else if (SW_ISSET(sw, SWITCH_WHAT))
-    do_chan_what(player, arg_left);
-  else
-    do_channel(player, arg_left, NULL, arg_right);
-}
-
-COMMAND (cmd_chat) {
-  do_chat_by_name(player, arg_left, arg_right, 1);
-}
-
-#endif
-
 COMMAND (cmd_chownall) {
   do_chownall(player, arg_left, arg_right, SW_ISSET(sw, SWITCH_PRESERVE));
 }
@@ -191,6 +141,10 @@ COMMAND (cmd_config) {
     do_list_attribs(player, lc);
   else if (SW_ISSET(sw, SWITCH_LIST))
     do_config_list(player, arg_left, lc);
+  else if (SW_ISSET(sw, SWITCH_FLAGS))
+    do_list_flags("FLAG", player, arg_left, lc, T("Flags"));
+  else if (SW_ISSET(sw, SWITCH_POWERS))
+    do_list_flags("POWER", player, arg_left, lc, T("Powers"));
   else if (SW_ISSET(sw, SWITCH_SET)) {
     if (!Wizard(player)) {
       notify(player, T("You can't remake the world in your image."));
@@ -200,11 +154,8 @@ COMMAND (cmd_config) {
       notify(player, T("What did you want to set?"));
       return;
     }
-    if (!arg_right || !*arg_right) {
-      notify(player, T("What did you want to set that to?"));
-      return;
-    }
-    if (!config_set(arg_left, arg_right, 1))
+    if (!config_set(arg_left, arg_right, 1, 0)
+	&& !config_set(arg_left, arg_right, 1, 1))
       notify(player, T("Couldn't set that option"));
     else
       notify(player, T("Option set."));
@@ -217,7 +168,7 @@ COMMAND (cmd_cpattr) {
 }
 
 COMMAND (cmd_create) {
-  do_create(player, arg_left, atol(arg_right));
+  do_create(player, arg_left, parse_integer(arg_right));
 }
 
 COMMAND (cmd_clone) {
@@ -227,38 +178,34 @@ COMMAND (cmd_clone) {
     do_clone(player, arg_left, arg_right, SWITCH_NONE);
 }
 
-#ifdef CHAT_SYSTEM
-COMMAND (cmd_clock) {
-  if (SW_ISSET(sw, SWITCH_JOIN))
-    do_chan_lock(player, arg_left, arg_right, CL_JOIN);
-  else if (SW_ISSET(sw, SWITCH_SPEAK))
-    do_chan_lock(player, arg_left, arg_right, CL_SPEAK);
-  else if (SW_ISSET(sw, SWITCH_MOD))
-    do_chan_lock(player, arg_left, arg_right, CL_MOD);
-  else if (SW_ISSET(sw, SWITCH_SEE))
-    do_chan_lock(player, arg_left, arg_right, CL_SEE);
-  else if (SW_ISSET(sw, SWITCH_HIDE))
-    do_chan_lock(player, arg_left, arg_right, CL_HIDE);
-  else
-    notify(player, T("You must specify a type of lock"));
-}
-#endif
-
 COMMAND (cmd_dbck) {
   do_dbck(player);
 }
 
 COMMAND (cmd_decompile) {
-  if (SW_ISSET(sw, SWITCH_DB))
-    do_decompile(player, arg_left, 1, SW_ISSET(sw, SWITCH_SKIPDEFAULTS));
-  else if (SW_ISSET(sw, SWITCH_TF))
-    do_decompile(player, arg_left, 2, SW_ISSET(sw, SWITCH_SKIPDEFAULTS));
+  char prefix[BUFFER_LEN];
+  int sd = SW_ISSET(sw, SWITCH_SKIPDEFAULTS);
+  *prefix = '\0';
+  if (SW_ISSET(sw, SWITCH_TF)) {
+    /* @dec/tf overrides @dec/prefix */
+    ATTR *a;
+    if (((a = atr_get_noparent(player, "TFPREFIX")) != NULL) &&
+	AL_STR(a) && *AL_STR(a)) {
+      strcpy(prefix, atr_value(a));
+    } else {
+      strcpy(prefix, "FugueEdit > ");
+    }
+  } else if (SW_ISSET(sw, SWITCH_PREFIX)) {
+    strcpy(prefix, arg_right);
+  }
+  if (SW_ISSET(sw, SWITCH_DB) || SW_ISSET(sw, SWITCH_TF))
+    do_decompile(player, arg_left, prefix, DEC_DB, sd);
   else if (SW_ISSET(sw, SWITCH_FLAGS))
-    do_decompile(player, arg_left, 3, SW_ISSET(sw, SWITCH_SKIPDEFAULTS));
+    do_decompile(player, arg_left, prefix, DEC_FLAG, sd);
   else if (SW_ISSET(sw, SWITCH_ATTRIBS))
-    do_decompile(player, arg_left, 4, SW_ISSET(sw, SWITCH_SKIPDEFAULTS));
+    do_decompile(player, arg_left, prefix, DEC_ATTR, sd);
   else
-    do_decompile(player, arg_left, 0, SW_ISSET(sw, SWITCH_SKIPDEFAULTS));
+    do_decompile(player, arg_left, prefix, DEC_NORMAL, sd);
 }
 
 COMMAND (cmd_teach) {
@@ -295,15 +242,18 @@ COMMAND (cmd_dolist) {
 
 COMMAND (cmd_dump) {
   if (SW_ISSET(sw, SWITCH_PARANOID))
-    do_dump(player, arg_left, 1);
+    do_dump(player, arg_left, DUMP_PARANOID);
   else if (SW_ISSET(sw, SWITCH_DEBUG))
-    do_dump(player, arg_left, 2);
+    do_dump(player, arg_left, DUMP_DEBUG);
   else
-    do_dump(player, arg_left, 0);
+    do_dump(player, arg_left, DUMP_NORMAL);
 }
 
 COMMAND (cmd_edit) {
-  do_gedit(player, arg_left, args_right);
+  if (SW_ISSET(sw, SWITCH_FIRST))
+    do_gedit(player, arg_left, args_right, EDIT_FIRST);
+  else
+    do_gedit(player, arg_left, args_right, EDIT_ALL);
 }
 
 COMMAND (cmd_elock) {
@@ -311,10 +261,14 @@ COMMAND (cmd_elock) {
 }
 
 COMMAND (cmd_emit) {
+  int spflags = !strcmp(cmd->name, "@NSEMIT") ? PEMIT_SPOOF : 0;
+  SPOOF(player, cause, sw);
+
   if (SW_ISSET(sw, SWITCH_ROOM))
-    do_lemit(player, arg_left, SW_ISSET(sw, SWITCH_SILENT));
+    do_lemit(player, arg_left,
+	     (SW_ISSET(sw, SWITCH_SILENT) * PEMIT_SILENT) | spflags);
   else
-    do_emit(player, arg_left);
+    do_emit(player, arg_left, spflags);
 }
 
 COMMAND (cmd_enable) {
@@ -323,15 +277,15 @@ COMMAND (cmd_enable) {
 
 COMMAND (cmd_entrances) {
   if (SW_ISSET(sw, SWITCH_EXITS))
-    do_entrances(player, arg_left, args_right, 1);
+    do_entrances(player, arg_left, args_right, ENT_EXITS);
   else if (SW_ISSET(sw, SWITCH_THINGS))
-    do_entrances(player, arg_left, args_right, 2);
+    do_entrances(player, arg_left, args_right, ENT_THINGS);
   else if (SW_ISSET(sw, SWITCH_PLAYERS))
-    do_entrances(player, arg_left, args_right, 3);
+    do_entrances(player, arg_left, args_right, ENT_PLAYERS);
   else if (SW_ISSET(sw, SWITCH_ROOMS))
-    do_entrances(player, arg_left, args_right, 4);
+    do_entrances(player, arg_left, args_right, ENT_ROOMS);
   else
-    do_entrances(player, arg_left, args_right, 0);
+    do_entrances(player, arg_left, args_right, ENT_ALL);
 }
 
 COMMAND (cmd_eunlock) {
@@ -346,6 +300,29 @@ COMMAND (cmd_firstexit) {
   do_firstexit(player, arg_left);
 }
 
+COMMAND (cmd_flag) {
+  if (SW_ISSET(sw, SWITCH_LIST))
+    do_list_flags("FLAG", player, arg_left, 0, T("Flags"));
+  else if (SW_ISSET(sw, SWITCH_ADD))
+    do_flag_add("FLAG", player, arg_left, args_right);
+  else if (SW_ISSET(sw, SWITCH_DELETE))
+    do_flag_delete("FLAG", player, arg_left);
+  else if (SW_ISSET(sw, SWITCH_ALIAS))
+    do_flag_alias("FLAG", player, arg_left, args_right[1]);
+  else if (SW_ISSET(sw, SWITCH_RESTRICT))
+    do_flag_restrict("FLAG", player, arg_left, args_right);
+  else if (SW_ISSET(sw, SWITCH_DISABLE))
+    do_flag_disable("FLAG", player, arg_left);
+  else if (SW_ISSET(sw, SWITCH_ENABLE))
+    do_flag_enable("FLAG", player, arg_left);
+  else if (SW_ISSET(sw, SWITCH_LETTER))
+    do_flag_letter("FLAG", player, arg_left, args_right[1]);
+  else if (SW_ISSET(sw, SWITCH_TYPE))
+    do_flag_type("FLAG", player, arg_left, args_right[1]);
+  else
+    do_flag_info("FLAG", player, arg_left);
+}
+
 COMMAND (cmd_force) {
   do_force(player, arg_left, arg_right);
 }
@@ -389,9 +366,6 @@ COMMAND (cmd_function) {
   }
 }
 
-COMMAND (cmd_gedit) {
-  do_gedit(player, arg_left, args_right);
-}
 
 COMMAND (cmd_grep) {
   do_grep(player, arg_left, arg_right, ((SW_ISSET(sw, SWITCH_IPRINT))
@@ -412,7 +386,7 @@ COMMAND (cmd_hide) {
 }
 
 COMMAND (cmd_hook) {
-  int hook_type = 0;
+  enum hook_type flags;
 
   if (!Wizard(player)) {
     notify(player, T("You need a fishing license to use that hook."));
@@ -420,14 +394,22 @@ COMMAND (cmd_hook) {
   }
 
   if (SW_ISSET(sw, SWITCH_AFTER))
-    hook_type = 2;
+    flags = HOOK_AFTER;
   else if (SW_ISSET(sw, SWITCH_BEFORE))
-    hook_type = 1;
+    flags = HOOK_BEFORE;
+  else if (SW_ISSET(sw, SWITCH_IGNORE))
+    flags = HOOK_IGNORE;
+  else if (SW_ISSET(sw, SWITCH_OVERRIDE))
+    flags = HOOK_OVERRIDE;
   else {
     notify(player, T("You must give a switch for @hook"));
     return;
   }
-  do_hook(player, arg_left, args_right[1], args_right[2], hook_type);
+  do_hook(player, arg_left, args_right[1], args_right[2], flags);
+}
+
+COMMAND (cmd_huh_command) {
+  notify(player, T("Huh?  (Type \"help\" for help.)"));
 }
 
 COMMAND (cmd_kick) {
@@ -435,7 +417,10 @@ COMMAND (cmd_kick) {
 }
 
 COMMAND (cmd_lemit) {
-  do_lemit(player, arg_left, SW_ISSET(sw, SWITCH_SILENT));
+  int spflags = !strcmp(cmd->name, "@NSLEMIT") ? PEMIT_SPOOF : 0;
+  SPOOF(player, cause, sw);
+  do_lemit(player, arg_left,
+	   (SW_ISSET(sw, SWITCH_SILENT) * PEMIT_SILENT) | spflags);
 }
 
 COMMAND (cmd_link) {
@@ -443,20 +428,24 @@ COMMAND (cmd_link) {
 }
 
 COMMAND (cmd_listmotd) {
-  do_motd(player, 3, "");
+  do_motd(player, MOTD_LIST, "");
 }
 
 COMMAND (cmd_list) {
   int lc;
   lc = SW_ISSET(sw, SWITCH_LOWERCASE);
   if (SW_ISSET(sw, SWITCH_MOTD))
-    do_motd(player, 3, "");
+    do_motd(player, MOTD_LIST, "");
   else if (SW_ISSET(sw, SWITCH_FUNCTIONS))
     do_list_functions(player, lc);
   else if (SW_ISSET(sw, SWITCH_COMMANDS))
     do_list_commands(player, lc);
   else if (SW_ISSET(sw, SWITCH_ATTRIBS))
     do_list_attribs(player, lc);
+  else if (SW_ISSET(sw, SWITCH_FLAGS))
+    do_list_flags("FLAG", player, arg_left, lc, T("Flags"));
+  else if (SW_ISSET(sw, SWITCH_POWERS))
+    do_list_flags("POWER", player, arg_left, lc, T("Powers"));
   else
     do_list(player, arg_left, lc);
 }
@@ -504,18 +493,17 @@ COMMAND (cmd_lset) {
   do_lset(player, arg_left, arg_right);
 }
 
-#ifdef USE_MAILER
 COMMAND (cmd_mail) {
   int urgent = SW_ISSET(sw, SWITCH_URGENT);
   int silent = SW_ISSET(sw, SWITCH_SILENT);
   int nosig = SW_ISSET(sw, SWITCH_NOSIG);
   /* First, mail commands that can be used even if you're gagged */
   if (SW_ISSET(sw, SWITCH_STATS))
-    do_mail_stats(player, arg_left, 0);
+    do_mail_stats(player, arg_left, MSTATS_COUNT);
   else if (SW_ISSET(sw, SWITCH_DSTATS))
-    do_mail_stats(player, arg_left, 1);
+    do_mail_stats(player, arg_left, MSTATS_READ);
   else if (SW_ISSET(sw, SWITCH_FSTATS))
-    do_mail_stats(player, arg_left, 2);
+    do_mail_stats(player, arg_left, MSTATS_SIZE);
   else if (SW_ISSET(sw, SWITCH_DEBUG))
     do_mail_debug(player, arg_left, arg_right);
   else if (SW_ISSET(sw, SWITCH_NUKE))
@@ -558,7 +546,6 @@ COMMAND (cmd_mail) {
     do_mail(player, arg_left, arg_right);	/* Does its own gagged check */
 }
 
-#ifdef MAIL_ALIASES
 
 COMMAND (cmd_malias) {
   if (SW_ISSET(sw, SWITCH_LIST))
@@ -594,8 +581,6 @@ COMMAND (cmd_malias) {
   else
     do_malias(player, arg_left, arg_right);
 }
-#endif
-#endif
 
 COMMAND (cmd_map) {
   unsigned int flags = DOL_MAP;
@@ -606,17 +591,17 @@ COMMAND (cmd_map) {
 
 COMMAND (cmd_motd) {
   if (SW_ISSET(sw, SWITCH_CONNECT))
-    do_motd(player, 1, arg_left);
+    do_motd(player, MOTD_MOTD, arg_left);
   else if (SW_ISSET(sw, SWITCH_LIST))
-    do_motd(player, 3, "");
+    do_motd(player, MOTD_LIST, "");
   else if (SW_ISSET(sw, SWITCH_WIZARD))
-    do_motd(player, 2, arg_left);
+    do_motd(player, MOTD_WIZ, arg_left);
   else if (SW_ISSET(sw, SWITCH_DOWN))
-    do_motd(player, 4, arg_left);
+    do_motd(player, MOTD_DOWN, arg_left);
   else if (SW_ISSET(sw, SWITCH_FULL))
-    do_motd(player, 5, arg_left);
+    do_motd(player, MOTD_FULL, arg_left);
   else
-    do_motd(player, 1, arg_left);
+    do_motd(player, MOTD_MOTD, arg_left);
 }
 
 COMMAND (cmd_mvattr) {
@@ -631,28 +616,14 @@ COMMAND (cmd_newpassword) {
   do_newpassword(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_nspemit) {
-  int silent;
-
-  if (SW_ISSET(sw, SWITCH_SILENT))
-    silent = 1;
-  else if (SW_ISSET(sw, SWITCH_NOISY))
-    silent = 0;
-  else
-    silent = SILENT_PEMIT ? 1 : 0;
-
-  if (SW_ISSET(sw, SWITCH_LIST))
-    do_pemit_list(player, arg_left, arg_right, 0);
-  else
-    do_pemit(player, arg_left, arg_right, silent, 0);
-}
-
 COMMAND (cmd_nuke) {
   do_destroy(player, arg_left, 1);
 }
 
 COMMAND (cmd_oemit) {
-  do_oemit_list(player, arg_left, arg_right);
+  int spflags = !strcmp(cmd->name, "@NSOEMIT") ? PEMIT_SPOOF : 0;
+  SPOOF(player, cause, sw);
+  do_oemit_list(player, arg_left, arg_right, spflags);
 }
 
 COMMAND (cmd_open) {
@@ -672,21 +643,24 @@ COMMAND (cmd_pcreate) {
 }
 
 COMMAND (cmd_pemit) {
-  int silent;
+  int flags;
 
+  SPOOF(player, cause, sw);
   if (SW_ISSET(sw, SWITCH_SILENT))
-    silent = 1;
+    flags = PEMIT_SILENT;
   else if (SW_ISSET(sw, SWITCH_NOISY))
-    silent = 0;
+    flags = 0;
   else
-    silent = SILENT_PEMIT ? 1 : 0;
+    flags = SILENT_PEMIT ? PEMIT_SILENT : 0;
 
+  if (!strcmp(cmd->name, "@NSPEMIT"))
+    flags |= PEMIT_SPOOF;
   if (SW_ISSET(sw, SWITCH_LIST))
-    do_pemit_list(player, arg_left, arg_right, 1);
+    do_pemit_list(player, arg_left, arg_right, flags);
   else if (SW_ISSET(sw, SWITCH_CONTENTS))
-    do_remit(player, arg_left, arg_right, !silent);
+    do_remit(player, arg_left, arg_right, flags);
   else
-    do_pemit(player, arg_left, arg_right, silent, 1);
+    do_pemit(player, arg_left, arg_right, flags);
 }
 
 COMMAND (cmd_poll) {
@@ -698,25 +672,43 @@ COMMAND (cmd_poor) {
 }
 
 COMMAND (cmd_power) {
-  do_power(player, arg_left, arg_right);
+  if (SW_ISSET(sw, SWITCH_LIST))
+    do_list_flags("POWER", player, arg_left, 0, T("Powers"));
+  else if (SW_ISSET(sw, SWITCH_ADD))
+    do_flag_add("POWER", player, arg_left, args_right);
+  else if (SW_ISSET(sw, SWITCH_DELETE))
+    do_flag_delete("POWER", player, arg_left);
+  else if (SW_ISSET(sw, SWITCH_ALIAS))
+    do_flag_alias("POWER", player, arg_left, args_right[1]);
+  else if (SW_ISSET(sw, SWITCH_RESTRICT))
+    do_flag_restrict("POWER", player, arg_left, args_right);
+  else if (SW_ISSET(sw, SWITCH_DISABLE))
+    do_flag_disable("POWER", player, arg_left);
+  else if (SW_ISSET(sw, SWITCH_ENABLE))
+    do_flag_enable("POWER", player, arg_left);
+  else if (SW_ISSET(sw, SWITCH_LETTER))
+    do_flag_letter("POWER", player, arg_left, args_right[1]);
+  else if (SW_ISSET(sw, SWITCH_TYPE))
+    do_flag_type("POWER", player, arg_left, args_right[1]);
+  else
+    do_power(player, arg_left, args_right[1]);
 }
 
 COMMAND (cmd_ps) {
   if (SW_ISSET(sw, SWITCH_ALL))
-    do_queue(player, arg_left, 1);
+    do_queue(player, arg_left, QUEUE_ALL);
   else if (SW_ISSET(sw, SWITCH_SUMMARY) || SW_ISSET(sw, SWITCH_COUNT))
-    do_queue(player, arg_left, 2);
+    do_queue(player, arg_left, QUEUE_SUMMARY);
   else if (SW_ISSET(sw, SWITCH_QUICK))
-    do_queue(player, arg_left, 3);
+    do_queue(player, arg_left, QUEUE_QUICK);
   else
-    do_queue(player, arg_left, 0);
+    do_queue(player, arg_left, QUEUE_NORMAL);
 }
 
 COMMAND (cmd_purge) {
   do_purge(player);
 }
 
-#ifdef QUOTA
 COMMAND (cmd_quota) {
   if (SW_ISSET(sw, SWITCH_ALL))
     do_allquota(player, arg_left, (SW_ISSET(sw, SWITCH_QUIET)));
@@ -725,7 +717,6 @@ COMMAND (cmd_quota) {
   else
     do_quota(player, arg_left, "", 0);
 }
-#endif				/* QUOTA */
 
 COMMAND (cmd_readcache) {
   do_readcache(player);
@@ -733,19 +724,22 @@ COMMAND (cmd_readcache) {
 
 COMMAND (cmd_remit) {
   int flags;
+  SPOOF(player, cause, sw);
   if (SW_ISSET(sw, SWITCH_SILENT))
-    flags = 0;
+    flags = PEMIT_SILENT;
   else if (SW_ISSET(sw, SWITCH_NOISY))
-    flags = 1;
+    flags = 0;
   else
-    flags = SILENT_PEMIT ? 0 : 1;
+    flags = SILENT_PEMIT ? PEMIT_SILENT : 0;
   if (SW_ISSET(sw, SWITCH_LIST))
-    flags |= 0x2;
+    flags |= PEMIT_LIST;
+  if (!strcmp(cmd->name, "@NSREMIT"))
+    flags |= PEMIT_SPOOF;
   do_remit(player, arg_left, arg_right, flags);
 }
 
 COMMAND (cmd_rejectmotd) {
-  do_motd(player, 4, arg_left);
+  do_motd(player, MOTD_DOWN, arg_left);
 }
 
 COMMAND (cmd_restart) {
@@ -756,16 +750,9 @@ COMMAND (cmd_restart) {
 }
 
 COMMAND (cmd_rwall) {
-  do_wall(player, arg_left, 1, 1);
+  do_wall(player, arg_left, WALL_RW, SW_ISSET(sw, SWITCH_EMIT));
 }
 
-COMMAND (cmd_rwallpose) {
-  do_wall(player, arg_left, 1, 2);
-}
-
-COMMAND (cmd_rwallemit) {
-  do_wall(player, arg_left, 1, 3);
-}
 
 COMMAND (cmd_scan) {
   if (SW_ISSET(sw, SWITCH_ROOM))
@@ -787,7 +774,7 @@ COMMAND (cmd_search) {
 
 COMMAND (cmd_select) {
   do_switch(player, arg_left, args_right, cause, 1,
-	    (SW_ISSET(sw, SWITCH_NOTIFY)));
+	    SW_ISSET(sw, SWITCH_NOTIFY), SW_ISSET(sw, SWITCH_REGEXP));
 }
 
 COMMAND (cmd_set) {
@@ -795,36 +782,51 @@ COMMAND (cmd_set) {
 }
 
 COMMAND (cmd_shutdown) {
-  int paranoid;
+  enum shutdown_type paranoid;
   if (!Wizard(player)) {
     notify(player, T("You don't have the authority to do that!"));
     return;
   }
-  paranoid = (SW_ISSET(sw, SWITCH_PARANOID)) ? 1 : 0;
+  paranoid = SW_ISSET(sw, SWITCH_PARANOID) ? SHUT_PARANOID : SHUT_NORMAL;
   if (SW_ISSET(sw, SWITCH_REBOOT))
-    do_reboot(player, paranoid);
+    do_reboot(player, paranoid == SHUT_PARANOID);
   else if (SW_ISSET(sw, SWITCH_PANIC))
-    do_shutdown(player, -1);
+    do_shutdown(player, SHUT_PANIC);
   else
     do_shutdown(player, paranoid);
 }
 
 COMMAND (cmd_sitelock) {
   if (SW_ISSET(sw, SWITCH_BAN))
-    do_sitelock(player, arg_left, NULL, NULL, 1);
+    do_sitelock(player, arg_left, NULL, NULL, SITELOCK_BAN);
   else if (SW_ISSET(sw, SWITCH_REGISTER))
-    do_sitelock(player, arg_left, NULL, NULL, 0);
+    do_sitelock(player, arg_left, NULL, NULL, SITELOCK_ADD);
   else if (SW_ISSET(sw, SWITCH_NAME))
-    do_sitelock(player, arg_left, NULL, NULL, 2);
+    do_sitelock_name(player, arg_left);
   else if (SW_ISSET(sw, SWITCH_REMOVE))
-    do_sitelock(player, arg_left, NULL, NULL, 3);
+    do_sitelock(player, arg_left, NULL, NULL, SITELOCK_REMOVE);
+  else if (SW_ISSET(sw, SWITCH_CHECK))
+    do_sitelock(player, arg_left, NULL, NULL, SITELOCK_CHECK);
+  else if (!arg_left || !*arg_left)
+    do_sitelock(player, NULL, NULL, NULL, SITELOCK_LIST);
   else
-    do_sitelock(player, arg_left, args_right[1], args_right[2], 0);
+    do_sitelock(player, arg_left, args_right[1], args_right[2], SITELOCK_ADD);
 }
 
 COMMAND (cmd_stats) {
   if (SW_ISSET(sw, SWITCH_TABLES))
     do_list_memstats(player);
+  else if (SW_ISSET(sw, SWITCH_CHUNKS)) {
+    if (SW_ISSET(sw, SWITCH_REGIONS))
+      chunk_stats(player, CSTATS_REGION);
+    else
+      chunk_stats(player, CSTATS_SUMMARY);
+  } else if (SW_ISSET(sw, SWITCH_REGIONS))
+    chunk_stats(player, CSTATS_REGIONG);
+  else if (SW_ISSET(sw, SWITCH_PAGING))
+    chunk_stats(player, CSTATS_PAGINGG);
+  else if (SW_ISSET(sw, SWITCH_FREESPACE))
+    chunk_stats(player, CSTATS_FREESPACEG);
   else
     do_stats(player, arg_left);
 }
@@ -843,22 +845,21 @@ COMMAND (cmd_sweep) {
 }
 
 COMMAND (cmd_switch) {
-  do_switch(player, arg_left, args_right, cause, (SW_ISSET(sw, SWITCH_FIRST)),
-	    (SW_ISSET(sw, SWITCH_NOTIFY)));
+  do_switch(player, arg_left, args_right, cause, SW_ISSET(sw, SWITCH_FIRST),
+	    SW_ISSET(sw, SWITCH_NOTIFY), SW_ISSET(sw, SWITCH_REGEXP));
 }
 
-#ifdef QUOTA
 COMMAND (cmd_squota) {
   do_quota(player, arg_left, arg_right, 1);
 }
 
-#endif
 
 COMMAND (cmd_teleport) {
   if (rhs_present && !*arg_right)
     notify(player, T("You can't teleport to nothing!"));
   else
-    do_teleport(player, arg_left, arg_right, (SW_ISSET(sw, SWITCH_SILENT)));
+    do_teleport(player, arg_left, arg_right, (SW_ISSET(sw, SWITCH_SILENT)),
+		(SW_ISSET(sw, SWITCH_INSIDE)));
 }
 
 COMMAND (cmd_trigger) {
@@ -905,28 +906,9 @@ COMMAND (cmd_wait) {
 }
 
 COMMAND (cmd_wall) {
-  int target = 2;
-  int type = 1;
-  if (SW_ISSET(sw, SWITCH_WIZARD))
-    target = 0;
-  if (SW_ISSET(sw, SWITCH_ROYALTY))
-    target = 1;
-  if (SW_ISSET(sw, SWITCH_EMIT))
-    type = 3;
-  if (SW_ISSET(sw, SWITCH_POSE))
-    type = 2;
-  do_wall(player, arg_left, target, type);
+  do_wall(player, arg_left, WALL_ALL, SW_ISSET(sw, SWITCH_EMIT));
 }
 
-COMMAND (cmd_wallpose) {
-  do_wall(player, arg_left, 2, 2);
-}
-
-COMMAND (cmd_wallemit) {
-  do_wall(player, arg_left, 2, 3);
-}
-
-#ifdef USE_WARNINGS
 COMMAND (cmd_warnings) {
   do_warnings(player, arg_left, arg_right);
 }
@@ -939,7 +921,6 @@ COMMAND (cmd_wcheck) {
   else
     do_wcheck(player, arg_left);
 }
-#endif
 
 COMMAND (cmd_whereis) {
   do_whereis(player, arg_left);
@@ -950,27 +931,20 @@ COMMAND (cmd_wipe) {
 }
 
 COMMAND (cmd_wizwall) {
-  do_wall(player, arg_left, 0, 1);
+  do_wall(player, arg_left, WALL_WIZ, SW_ISSET(sw, SWITCH_EMIT));
 }
 
 COMMAND (cmd_wizmotd) {
-  do_motd(player, 2, arg_left);
-}
-
-COMMAND (cmd_wizpose) {
-  do_wall(player, arg_left, 0, 2);
-}
-
-COMMAND (cmd_wizemit) {
-  do_wall(player, arg_left, 0, 3);
+  do_motd(player, MOTD_WIZ, arg_left);
 }
 
 COMMAND (cmd_zemit) {
-  do_zemit(player, arg_left, arg_right);
+  int flags = !strcmp(cmd->name, "@NSZEMIT") ? PEMIT_SPOOF : 0;
+  do_zemit(player, arg_left, arg_right, flags);
 }
 
 COMMAND (cmd_brief) {
-  do_examine(player, arg_left, 1);
+  do_examine(player, arg_left, EXAM_BRIEF, 0, 0);
 }
 
 COMMAND (cmd_drop) {
@@ -978,14 +952,20 @@ COMMAND (cmd_drop) {
 }
 
 COMMAND (cmd_examine) {
+  int all = SW_ISSET(sw, SWITCH_ALL);
+  int parent = SW_ISSET(sw, SWITCH_PARENT);
   if (SW_ISSET(sw, SWITCH_BRIEF))
-    do_examine(player, arg_left, 1);
+    do_examine(player, arg_left, EXAM_BRIEF, all, 0);
   else if (SW_ISSET(sw, SWITCH_DEBUG))
     do_debug_examine(player, arg_left);
   else if (SW_ISSET(sw, SWITCH_MORTAL))
-    do_examine(player, arg_left, 2);
+    do_examine(player, arg_left, EXAM_MORTAL, all, parent);
   else
-    do_examine(player, arg_left, 0);
+    do_examine(player, arg_left, EXAM_NORMAL, all, parent);
+}
+
+COMMAND (cmd_empty) {
+  do_empty(player, arg_left);
 }
 
 COMMAND (cmd_enter) {
@@ -1012,6 +992,39 @@ COMMAND (cmd_get) {
   do_get(player, arg_left);
 }
 
+COMMAND (cmd_buy) {
+  char *from = NULL;
+  char *forwhat = NULL;
+  int price = -1;
+
+  upcasestr(arg_left);
+
+  from = strstr(arg_left, " FROM ");
+  forwhat = strstr(arg_left, " FOR ");
+  if (from) {
+    *from = '\0';
+    from += 6;
+  }
+  if (forwhat) {
+    *forwhat = '\0';
+    forwhat += 5;
+  }
+  if (forwhat && !is_strict_integer(forwhat)) {
+    notify(player, T("Buy for WHAT price?"));
+    return;
+  } else if (forwhat) {
+    price = parse_integer(forwhat);
+    if (price < 0) {
+      notify(player, T("You can't buy things by taking money."));
+      return;
+    }
+  }
+
+  if (from)
+    from = trim_space_sep(from, ' ');
+  do_buy(player, arg_left, from, price);
+}
+
 COMMAND (cmd_give) {
   do_give(player, arg_left, arg_right, (SW_ISSET(sw, SWITCH_SILENT)));
 }
@@ -1108,3 +1121,11 @@ COMMAND (command_atrset) {
 COMMAND (cmd_null) {
   return;
 }
+
+COMMAND (cmd_warn_on_missing) {
+  notify_format(Owner(player),
+		T
+		("No command found in code by %s - don't start code with functions."),
+		unparse_dbref(player));
+  return;
+}
