@@ -1,16 +1,23 @@
-/* comp_h.c */
+/**
+ * \file comp_h.c
+ *
+ * \brief Huffman compression routines.
+ *
+ * One of several options for attribute compression. Usually the best.
+ * Talek's rewrite of compress.c, using a Huffman compression routine.
+ * This routine adds some time to the MUSH startup, since it reads a file 
+ * in order to auto-tune the compression at each restart. The SAMPLE_SIZE
+ * define can trade efficiency for speed.
+ * This rewrite was inspired by Javelin's rewrite on a similar vein.
+ * Most of the comments are his. The nasty ones are Talek's.
+ *
+ */
 
 /* Compression routines */
 #include "copyrite.h"
 #include "config.h"
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
-#ifdef I_STDLIB
 #include <stdlib.h>
-#endif
 #include <ctype.h>
 #include "conf.h"
 #include "externs.h"
@@ -21,32 +28,26 @@
 #pragma warning( disable : 4244)	/* NJG: disable warning re conversion */
 #endif
 
-/* Talek's rewrite of compress.c, using a Huffman compression routine. */
-/* This routine adds some time to the MUSH startup, since it reads a file */
-/* in order to auto-tune the compression at each restart. See the */
-/* SAMPLE_SIZE define in options.h for ways to trade efficiency for speed */
-
-/* This rewrite was inspired by Javelin's rewrite on a similar vein. */
-/* Most of the comments are his. The nasty ones are Talek's. */
-
-#define TABLE_SIZE      256	/* allow all characters */
-#define EOS             0	/* use null code for end of string */
-#define CHAR_BITS       8	/* number of bits in char */
-#define CHAR_MASK       255	/* mask for just one char */
-#define CODE_BITS       25	/* max number of bits in code */
+#define TABLE_SIZE      256	/**< allow all characters */
+#define EOS             0	/**< use null code for end of string */
+#define CHAR_BITS       8	/**< number of bits in char */
+#define CHAR_MASK       255	/**< mask for just one char */
+#define CODE_BITS       25	/**< max number of bits in code */
 #ifndef SAMPLE_SIZE
-#define SAMPLE_SIZE     0
+#define SAMPLE_SIZE     0	/**< sample entire database */
 #endif
 
 
-/* The code type */
-typedef unsigned long CType;	/* must be at least CODE_BITS+CHAR_BITS-1 */
-				/* bits long */
+/** Type for a huffman code. It must be at least CODE_BITS+CHAR_BITS-1
+ * bits long.
+ */
+typedef unsigned long CType;
 
-/* A node in the compression tree */
+/** A node in the huffman compression tree. */
 typedef struct cnode {
-  struct cnode *left, *right;
-  unsigned char c;
+  struct cnode *left;		/**< Left child node. */
+  struct cnode *right;		/**< Right child node. */
+  unsigned char c;		/**< character at this node. */
 } CNode;
 
 static CNode *ctop;
@@ -59,15 +60,20 @@ static void build_ctable(CNode *root, CType code, int numbits);
 int init_compress(FILE * f);
 
 
-/* Compress a string: this is pretty easy. For each char in the string,
+/** Huffman-compress a string.
+ * Compress a string: this is pretty easy. For each char in the string,
  * look up its code in ctable and add it to the compressed string we
  * build, keeping careful track of the number of bits we add.
  * Then stick the EOS character at the end.
+ *
  * Important notes:
  *   This function mallocs memory that should be freed by the caller!
  *   The caller is also currently responsible for adding mem checks
  *   Don't use it to compress strings longer than BUFFER_LEN or the
  *     later uncompression will not go well.
+ *
+ * \param s string to be compressed.
+ * \return newly allocated compressed string.
  */
 unsigned char *
 compress(const char *s)
@@ -114,7 +120,8 @@ compress(const char *s)
   return buf;
 }
 
-/* This macro is used for walking the compression tree.
+/** Walk the huffman tree.
+ * This macro is used for walking the compression tree.
  * It is a macro for efficiency.
  */
 #define WALK_TREE(bitpos) \
@@ -136,21 +143,27 @@ do { \
   } \
 } while (0)
 
-/* Uncompression is a snap, too. Go bit by bit, using the
+/** Huffman uncompress a string.
+ * Uncompression is a snap, too. Go bit by bit, using the
  * bits to traverse the binary tree (0=left, 1=right) until reaching
  * a leaf node, which is the uncompressed character.
  * Stop when the leaf node turns out to be EOS.
+ *
+ * To avoid generating memory problems, this function should be
+ * used with something of the format
+ * \verbatim
+ * char tbuf1[BUFFER_LEN];
+ * strcpy(tbuf1, uncompress(a->value));
+ * \endverbatim
+ * if you are using something of type char *buff, use the
+ * safe_uncompress function instead.
+ *
+ * \param s a compressed string.
+ * \return a pointer to a static buffer containing the uncompressed string.
  */
 char *
 uncompress(const unsigned char *s)
 {
-  /* to avoid generating memory problems, this function should be
-   * used with something of the format
-   * char tbuf1[BUFFER_LEN];
-   * strcpy(tbuf1, uncompress(a->value));
-   * if you are using something of type char *buff, use the
-   * safe_uncompress function instead.
-   */
 
   static char buf[BUFFER_LEN];
   const unsigned char *p;
@@ -177,26 +190,30 @@ uncompress(const unsigned char *s)
   }
 }
 
+/** Huffman uncompress a string, allocating memory.
+ * this function should be used when you're doing something like
+ * \verbatim
+ * char *attrib = safe_uncompress(a->value);
+ *
+ * NEVER use it with something like
+ *
+ * char tbuf1[BUFFER_LEN]; 
+ * strcpy(tbuf1, safe_uncompress(a->value));
+ * \endverbatim
+ * or you will create a horrendous memory leak.
+ *
+ * \param s compressed string to uncompress.
+ * \return pointer to newly allocated string containing uncompressed text.
+ */
 char *
-safe_uncompress(s)
-    unsigned char const *s;
+safe_uncompress(unsigned char const *s)
 {
-  /* this function should be used when you're doing something like
-   * char *attrib = safe_uncompress(a->value);
-   * NEVER use it with something like
-   * char tbuf1[BUFFER_LEN]; strcpy(tbuf1, safe_uncompress(a->value));
-   * or you will create a horrendous memory leak.
-   */
-
   return (char *) strdup((char *) uncompress(s));
 }
 
 
 static int
-fix_tree_depth(node, height, zeros)
-    CNode *node;
-    int height;
-    int zeros;
+fix_tree_depth(CNode *node, int height, int zeros)
 {
   int a, b;
   CNode *temp;
@@ -233,8 +250,7 @@ fix_tree_depth(node, height, zeros)
 
 /* Add 1s to the tree, recursively */
 static void
-add_ones(node)
-    CNode *node;
+add_ones(CNode *node)
 {
   int count;
 
@@ -264,10 +280,7 @@ add_ones(node)
 
 /* Build ctable and ltable from the tree, recursively */
 static void
-build_ctable(root, code, numbits)
-    CNode *root;
-    CType code;
-    int numbits;
+build_ctable(CNode *root, CType code, int numbits)
 {
 #ifdef STANDALONE
   int i;
@@ -295,7 +308,8 @@ build_ctable(root, code, numbits)
   }
 }
 
-/* Initialize the compression tree and table in 5 steps:
+/** Initialize huffman compression.
+ * Initialize the compression tree and table in 5 steps:
  * 1. Initialize arrays and things
  * 2. Read indb (up to SAMPLE_SIZE chars, if defined) and count
  *    the frequency of every character
@@ -303,10 +317,10 @@ build_ctable(root, code, numbits)
  *    and upper-case letters
  * 4. Construct an (un)compression tree based on frequencies
  * 5. Construct a compression table by searching the tree
+ * \param f filehandle to read from to build the tree.
  */
 int
-init_compress(f)
-    FILE *f;
+init_compress(FILE * f)
 {
   int total;
   unsigned char c;
@@ -341,12 +355,13 @@ init_compress(f)
 #endif
 
   /* Part 2: count frequencies */
-  total = 0;
-  while (!feof(f) && (!SAMPLE_SIZE || (total++ < SAMPLE_SIZE))) {
-    c = fgetc(f);
-    table[c].freq++;
+  if (f) {
+    total = 0;
+    while (!feof(f) && (!SAMPLE_SIZE || (total++ < SAMPLE_SIZE))) {
+      c = fgetc(f);
+      table[c].freq++;
+    }
   }
-
 #ifdef STANDALONE
   for (indx = 0; indx < TABLE_SIZE; indx++) {
     printf(isprint(indx) ? "Frequency for '%c': %d\n"
@@ -372,7 +387,8 @@ init_compress(f)
   /* The DEL character is returned once for no apparent reason (I think
    * it is returned at EOF), so remove that one count...
    */
-  table[255].freq--;
+  if (table[255].freq)
+    table[255].freq--;
 
   /* Newlines really aren't all that common in the attributes, so
    * chop the value substantially.
