@@ -1,13 +1,18 @@
-/* create.c */
+/**
+ * \file create.c
+ *
+ * \brief Functions for creating objects of all types.
+ *
+ *
+ */
 
-/* Commands that create new objects */
 #include "copyrite.h"
 #include "config.h"
 #include <string.h>
 #include "conf.h"
+#include "externs.h"
 #include "mushdb.h"
 #include "attrib.h"
-#include "externs.h"
 #include "match.h"
 #include "extchat.h"
 #include "log.h"
@@ -24,7 +29,7 @@ static dbref check_var_link(const char *dest_name);
 static dbref clone_object(dbref player, dbref thing, const char *newname,
 			  int preserve);
 
-struct db_stat_info current_state;
+struct db_stat_info current_state; /**< Current stats for database */
 
 /* utility for open and link */
 static dbref
@@ -70,16 +75,18 @@ check_var_link(const char *dest_name)
     return NOTHING;
 }
 
+/** Create an exit.
+ * This function opens an exit and optionally links it.
+ * \param player the enactor.
+ * \param direction the name of the exit.
+ * \param linkto the room to link to, as a string.
+ * \param pseudo a phony location for player if a back exit is needed. This is bpass by do_open() as the source room of the back exit.
+ * \return dbref of the new exit, or NOTHING.
+ */
 dbref
 do_real_open(dbref player, const char *direction, const char *linkto,
 	     dbref pseudo)
-     /* psuedo: a phony location for a player if a back
-      * exit is needed */
 {
-  /* function creates an exit. Note that it returns 0 and NOT -1
-   * if it fails.
-   */
-
   dbref loc =
     (pseudo !=
      NOTHING) ? pseudo : (IsExit(player) ? Source(player) : Location(player));
@@ -114,9 +121,8 @@ do_real_open(dbref player, const char *direction, const char *linkto,
     Owner(new_exit) = Owner(player);
     Zone(new_exit) = Zone(player);
     Source(new_exit) = loc;
-    Flags(new_exit) = TYPE_EXIT;
-    Flags(new_exit) |= options.exit_flags;
-    Toggles(new_exit) |= options.exit_toggles;
+    Type(new_exit) = TYPE_EXIT;
+    Flags(new_exit) = string_to_bits("FLAG", options.exit_flags);
 
     /* link it in */
     PUSH(new_exit, Exits(loc));
@@ -140,14 +146,21 @@ do_real_open(dbref player, const char *direction, const char *linkto,
       }
     }
     current_state.exits++;
-#ifdef LOCAL_DATA
     local_data_create(new_exit);
-#endif
     return new_exit;
   }
   return NOTHING;
 }
 
+/** Open a new exit.
+ * \verbatim
+ * This is the top-level function for @open. It calls do_real_open()
+ * to do the real work of opening both the exit forward and the exit back.
+ * \endverbatim
+ * \param player the enactor.
+ * \param direction name of the exit forward.
+ * \param links 1-based array containing name of destination and optionally name of exit back.
+ */
 void
 do_open(dbref player, const char *direction, char **links)
 {
@@ -158,6 +171,14 @@ do_open(dbref player, const char *direction, char **links)
   }
 }
 
+/** Unlink an exit or room.
+ * \verbatim
+ * This is the top-level function for @unlink, which can unlink an exit
+ * or remove a drop-to from a room.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name name of the object to unlink.
+ */
 void
 do_unlink(dbref player, const char *name)
 {
@@ -195,6 +216,20 @@ do_unlink(dbref player, const char *name)
   }
 }
 
+/** Link an exit, room, player, or thing.
+ * \verbatim
+ * This is the top-level function for @link, which is used to link an
+ * exit to a destination, set a player or thing's home, or set a 
+ * drop-to on a room.
+ *
+ * Linking an exit usually seizes ownership of the exit and costs 1 penny.
+ * 1 penny is also transferred to the former owner.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name the name of the object to link.
+ * \param room_name the name of the link destination.
+ * \param preserve if 1, preserve ownership and zone data on exit relink.
+ */
 void
 do_link(dbref player, const char *name, const char *room_name, int preserve)
 {
@@ -231,9 +266,9 @@ do_link(dbref player, const char *name, const char *room_name, int preserve)
       /* We may link an exit if it's unlinked and we pass the link-lock
        * or if we control it.
        */
-      if (!(controls(player, thing) || ((Location(thing) == NOTHING)
-					&& eval_lock(player, thing,
-						     Link_Lock)))) {
+      if (!(controls(player, thing)
+	    || ((Location(thing) == NOTHING)
+		&& eval_lock(player, thing, Link_Lock)))) {
 	notify(player, T("Permission denied."));
 	return;
       }
@@ -298,7 +333,8 @@ do_link(dbref player, const char *name, const char *room_name, int preserve)
       } else {
 	/* do the link */
 	Home(thing) = room;	/* home */
-	notify(player, T("Home set."));
+	if (!Quiet(player) && !(Quiet(thing) && (Owner(thing) == player)))
+	  notify(player, T("Home set."));
       }
       break;
     case TYPE_ROOM:
@@ -325,10 +361,18 @@ do_link(dbref player, const char *name, const char *room_name, int preserve)
   }
 }
 
-/* use this to create a room */
+/** Create a room.
+ * \verbatim
+ * This is the top-level interface for @dig.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name the name of the room to create.
+ * \param argv array of additional arguments to command (exit forward,exit back)
+ * \param tport if 1, teleport the player to the new room.
+ * \return dbref of new room, or NOTHING.
+ */
 dbref
-do_dig(dbref player, const char *name, char **argv, int tport	/* @tel player to new location? */
-  )
+do_dig(dbref player, const char *name, char **argv, int tport)
 {
   dbref room;
 
@@ -344,9 +388,8 @@ do_dig(dbref player, const char *name, char **argv, int tport	/* @tel player to
     set_name(room, name);
     Owner(room) = Owner(player);
     Zone(room) = Zone(player);
-    Flags(room) = TYPE_ROOM;
-    Flags(room) |= options.room_flags;
-    Toggles(room) |= options.room_toggles;
+    Type(room) = TYPE_ROOM;
+    Flags(room) = string_to_bits("FLAG", options.room_flags);
 
     notify_format(player, T("%s created with room number %d."), name, room);
     if (argv[1] && *argv[1]) {
@@ -358,22 +401,28 @@ do_dig(dbref player, const char *name, char **argv, int tport	/* @tel player to
       do_real_open(player, argv[2], "here", room);
     }
     current_state.rooms++;
-#ifdef LOCAL_DATA
     local_data_create(room);
-#endif
     if (tport) {
       /* We need to use the full command, because we need NO_TEL
        * and Z_TEL checking */
       char roomstr[MAX_COMMAND_LEN];
       sprintf(roomstr, "#%d", room);
-      do_teleport(player, "me", roomstr, 0);	/* if flag, move the player */
+      do_teleport(player, "me", roomstr, 0, 0);	/* if flag, move the player */
     }
     return room;
   }
   return NOTHING;
 }
 
-/* use this to create an object */
+/** Create a thing.
+ * \verbatim
+ * This is the top-level function for @create.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name name of thing to create.
+ * \param cost pennies spent in creation.
+ * \return dbref of new thing, or NOTHING.
+ */
 dbref
 do_create(dbref player, char *name, int cost)
 {
@@ -402,9 +451,8 @@ do_create(dbref player, char *name, int cost)
     Owner(thing) = Owner(player);
     Zone(thing) = Zone(player);
     s_Pennies(thing, cost);
-    Flags(thing) = TYPE_THING;
-    Flags(thing) |= options.thing_flags;
-    Toggles(thing) |= options.thing_toggles;
+    Type(thing) = TYPE_THING;
+    Flags(thing) = string_to_bits("FLAG", options.thing_flags);
 
     /* home is here (if we can link to it) or player's home */
     if ((loc = Location(player)) != NOTHING &&
@@ -423,9 +471,7 @@ do_create(dbref player, char *name, int cost)
     /* and we're done */
     notify_format(player, "Created: Object %s.", unparse_dbref(thing));
     current_state.things++;
-#ifdef LOCAL_DATA
     local_data_create(thing);
-#endif
     return thing;
   }
   return NOTHING;
@@ -441,9 +487,6 @@ clone_object(dbref player, dbref thing, const char *newname, int preserve)
 
   memcpy(REFDB(clone), REFDB(thing), sizeof(struct object));
   Owner(clone) = Owner(player);
-#ifdef CHAT_SYSTEM
-  Chanlist(clone) = NULL;
-#endif
   Name(clone) = NULL;
   if (newname && *newname)
     set_name(clone, newname);
@@ -455,32 +498,23 @@ clone_object(dbref player, dbref thing, const char *newname, int preserve)
   clone_locks(player, thing, clone);
   Zone(clone) = Zone(thing);
   Parent(clone) = Parent(thing);
+  Flags(clone) = clone_flag_bitmask("FLAG", Flags(thing));
   if (!preserve) {
-    Flags(clone) &= ~WIZARD;
-#ifdef ROYALTY_FLAG
-    Flags(clone) &= ~ROYALTY;
-#endif
-#ifdef USE_WARNINGS
+    clear_flag_internal(clone, "WIZARD");
+    clear_flag_internal(clone, "ROYALTY");
     Warnings(clone) = 0;	/* zap warnings */
-#endif
-    Powers(clone) = 0;		/* zap powers */
-  } else if (Wizard(clone) ||
-#ifdef ROYALTY_FLAG
-	     Royalty(clone) ||
-#endif
-#ifdef USE_WARNINGS
-	     Warnings(clone) ||
-#endif
-	     Powers(clone)) {
-    notify(player,
-	   T
-	   ("Warning: @CLONE/PRESERVE on an object with WIZ, ROY, @powers, or @warnings."));
+    Powers(clone) = new_flag_bitmask("POWER");	/* zap powers */
+  } else {
+    Powers(clone) = clone_flag_bitmask("POWER", Powers(thing));
+    if (Wizard(clone) || Royalty(clone) || Warnings(clone) ||
+	!null_flagmask("POWER", Powers(clone)))
+      notify(player,
+	     T
+	     ("Warning: @CLONE/PRESERVE on an object with WIZ, ROY, @powers, or @warnings."));
   }
-#ifdef CREATION_TIMES
   /* We give the clone the same modification time that its
    * other clone has, but update the creation time */
   CreTime(clone) = mudtime;
-#endif
 
   Contents(clone) = Location(clone) = Next(clone) = NOTHING;
 
@@ -488,6 +522,17 @@ clone_object(dbref player, dbref thing, const char *newname, int preserve)
 
 }
 
+/** Clone an object.
+ * \verbatim
+ * This is the top-level function for @clone, which creates a duplicate
+ * of a (non-player) object.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name the name of the object to clone.
+ * \param newname the name to give the duplicate.
+ * \param preserve if 1, preserve ownership and privileges on duplicate.
+ * \return dbref of the duplicate, or NOTHING.
+ */
 dbref
 do_clone(dbref player, char *name, char *newname, int preserve)
 {
@@ -530,12 +575,9 @@ do_clone(dbref player, char *name, char *newname, int preserve)
       else
 	moveto(clone, Location(player));
       current_state.things++;
-#ifdef LOCAL_DATA
-      LocData(clone) = NULL;
       local_data_clone(clone, thing);
-#endif
       real_did_it(player, clone, NULL, NULL, NULL, NULL, "ACLONE", NOTHING,
-		  wenv);
+		  global_eval_context.wenv, 0);
       return clone;
     }
     return NOTHING;
@@ -546,12 +588,9 @@ do_clone(dbref player, char *name, char *newname, int preserve)
       Exits(clone) = NOTHING;
       notify_format(player, T("Cloned: Room #%d."), clone);
       current_state.rooms++;
-#ifdef LOCAL_DATA
-      LocData(clone) = NULL;
       local_data_clone(clone, thing);
-#endif
       real_did_it(player, clone, NULL, NULL, NULL, NULL, "ACLONE", NOTHING,
-		  wenv);
+		  global_eval_context.wenv, 0);
       return clone;
     }
     return NOTHING;
@@ -584,34 +623,23 @@ do_clone(dbref player, char *name, char *newname, int preserve)
       clone_locks(player, thing, clone);
       Zone(clone) = Zone(thing);
       Parent(clone) = Parent(thing);
-      Flags(clone) = Flags(thing);
-      Toggles(clone) = Toggles(thing);
+      Flags(clone) = clone_flag_bitmask("FLAG", Flags(thing));
       if (!preserve) {
-	Flags(clone) &= ~WIZARD;
-	Flags(clone) &= ~ROYALTY;
-#ifdef USE_WARNINGS
+	clear_flag_internal(clone, "WIZARD");
+	clear_flag_internal(clone, "ROYALTY");
 	Warnings(clone) = 0;	/* zap warnings */
-#endif
-	Powers(clone) = 0;	/* zap powers */
+	Powers(clone) = new_flag_bitmask("POWER");	/* zap powers */
       } else {
-#ifdef USE_WARNINGS
 	Warnings(clone) = Warnings(thing);
-#endif
-	Powers(clone) = Powers(thing);
+	Powers(clone) = clone_flag_bitmask("POWER", Powers(thing));
       }
-      if (Wizard(clone) || Royalty(clone) ||
-#ifdef USE_WARNINGS
-	  Warnings(clone) ||
-#endif
-	  Powers(clone))
+      if (Wizard(clone) || Royalty(clone) || Warnings(clone) ||
+	  !null_flagmask("POWER", Powers(clone)))
 	notify(player,
 	       T
-	       ("Warning: @CLONE/PRESERVE on an exit with WIZ, ROY, @Powers, or Warnings."));
+	       ("Warning: @CLONE/PRESERVE on an exit with WIZ, ROY, @powers, or Warnings."));
       notify_format(player, T("Cloned: Exit #%d."), clone);
-#ifdef LOCAL_DATA
-      LocData(clone) = NULL;
       local_data_clone(clone, thing);
-#endif
       return clone;
     }
   }
