@@ -1,11 +1,18 @@
+/**
+ * \file fundb.c
+ *
+ * \brief Dbref-related functions for mushcode.
+ *
+ *
+ */
 
 #include "copyrite.h"
 
 #include "config.h"
 #include <string.h>
 #include "conf.h"
-#include "dbdefs.h"
 #include "externs.h"
+#include "dbdefs.h"
 #include "flags.h"
 
 #include "match.h"
@@ -14,9 +21,6 @@
 #include "game.h"
 #include "mushdb.h"
 #include "privtab.h"
-#ifdef MEM_CHECK
-#include "memcheck.h"
-#endif
 #include "lock.h"
 #include "log.h"
 #include "attrib.h"
@@ -27,27 +31,18 @@
 #pragma warning( disable : 4761)	/* NJG: disable warning re conversion */
 #endif
 
-#ifdef CHAT_SYSTEM
-#include "extchat.h"
-extern CHAN *channels;
-#endif
-
-extern char ccom[BUFFER_LEN];	/* bsd.c */
 extern PRIV attr_privs[];
-static const char *do_get_attrib(dbref executor, dbref thing,
-				 const char *attrib);
-dbref next_con(dbref player, dbref this);
-dbref next_exit(dbref player, dbref this);
 static lock_type get_locktype(char *str);
 extern struct db_stat_info *get_stats(dbref owner);
-static int lattr_helper(dbref player, dbref thing, char const *pattern,
-			ATTR *atr, void *args);
+static int lattr_helper(dbref player, dbref thing, dbref parent,
+			char const *pattern, ATTR *atr, void *args);
 static dbref
  dbwalk(char *buff, char **bp, dbref executor, dbref enactor,
-	int type, dbref loc, dbref after, int skipdark);
+	int type, dbref loc, dbref after, int skipdark,
+	int start, int count, int *retcount);
 
 
-static const char *
+const char *
 do_get_attrib(dbref executor, dbref thing, const char *attrib)
 {
   ATTR *a;
@@ -56,7 +51,7 @@ do_get_attrib(dbref executor, dbref thing, const char *attrib)
   a = atr_get(thing, strupper(attrib));
   if (a) {
     if (Can_Read_Attr(executor, thing, a)) {
-      if (strlen(value = uncompress(a->value)) < BUFFER_LEN)
+      if (strlen(value = atr_value(a)) < BUFFER_LEN)
 	return value;
       else
 	return T("#-1 ATTRIBUTE LENGTH TOO LONG");
@@ -74,33 +69,71 @@ do_get_attrib(dbref executor, dbref thing, const char *attrib)
   return "";
 }
 
+/** Structure containing arguments for lattr_helper */
+struct lh_args {
+  int first;		/**< Is this is the first attribute, or later? */
+  int nattr;
+  int start;		/**< Where do we start counting? */
+  int count;		/**< How many do we count? */
+  char *buff;		/**< Buffer to store output */
+  char **bp;		/**< Pointer to address of insertion point in buff */
+};
+
 /* this function produces a space-separated list of attributes that are
  * on an object.
  */
-struct lh_args {
-  int first;
-  char *buff;
-  char **bp;
-};
-
 /* ARGSUSED */
 static int
-lattr_helper(player, thing, pattern, atr, args)
-    dbref player __attribute__ ((__unused__));
-    dbref thing __attribute__ ((__unused__));
-    char const *pattern __attribute__ ((__unused__));
-    ATTR *atr;
-    void *args;
+lattr_helper(dbref player __attribute__ ((__unused__)),
+	     dbref thing __attribute__ ((__unused__)),
+	     dbref parent __attribute__ ((__unused__)),
+	     char const *pattern __attribute__ ((__unused__)),
+	     ATTR *atr, void *args)
 {
   struct lh_args *lh = args;
-  if (lh->first)
-    lh->first = 0;
-  else
-    safe_chr(' ', lh->buff, lh->bp);
-  safe_str(AL_NAME(atr), lh->buff, lh->bp);
+  lh->nattr++;
+  if (lh->count < 1
+      || (lh->nattr >= lh->start && lh->nattr < (lh->count + lh->start))) {
+    if (lh->first)
+      lh->first = 0;
+    else
+      safe_chr(' ', lh->buff, lh->bp);
+    safe_str(AL_NAME(atr), lh->buff, lh->bp);
+    return 1;
+  }
   return 1;
 }
 
+/* ARGSUSED */
+FUNCTION(fun_nattr)
+{
+  dbref thing;
+  int doparent;
+  static const char *matchall = "**";	/* count atrees by default too */
+  char *pattern;
+
+  pattern = strchr(args[0], '/');
+  if (pattern)
+    *pattern++ = '\0';
+  else
+    pattern = (char *) matchall;	/* match anything */
+
+  thing = match_thing(executor, args[0]);
+  if (!GoodObject(thing)) {
+    safe_str(T(e_notvis), buff, bp);
+    return;
+  }
+
+  doparent = strchr(called_as, 'P') ? 1 : 0;
+
+  if (!doparent && pattern == matchall && Can_Examine(executor, thing)) {
+    safe_integer(AttrCount(thing), buff, bp);
+  } else {
+    safe_integer(atr_pattern_count(executor, thing, pattern, doparent,
+				   !Can_Examine(executor, thing)), buff, bp);
+  }
+}
+
 /* ARGSUSED */
 FUNCTION(fun_lattr)
 {
@@ -108,6 +141,23 @@ FUNCTION(fun_lattr)
   char *pattern;
   struct lh_args lh;
 
+  lh.nattr = lh.start = lh.count = 0;
+
+  if (*called_as == 'X') {
+    if (!is_strict_integer(args[1]) || !is_strict_integer(args[2])) {
+      safe_str(T(e_int), buff, bp);
+      return;
+    }
+
+    lh.start = parse_integer(args[1]);
+    lh.count = parse_integer(args[2]);
+
+    if (lh.start < 1 || lh.count < 1) {
+      safe_str(T("#-1 ARGUMENT OUT OF RANGE"), buff, bp);
+      return;
+    }
+  }
+
   pattern = strchr(args[0], '/');
   if (pattern)
     *pattern++ = '\0';
@@ -119,30 +169,17 @@ FUNCTION(fun_lattr)
     safe_str(T(e_notvis), buff, bp);
     return;
   }
-  if (!Can_Examine(executor, thing)) {
-    safe_str(T(e_perm), buff, bp);
-    return;
-  }
   lh.first = 1;
   lh.buff = buff;
   lh.bp = bp;
-  (void) atr_iter_get(executor, thing, pattern, lattr_helper, &lh);
-}
-
-/* ARGSUSED */
-FUNCTION(fun_nattr)
-{
-  dbref thing;
-  thing = match_thing(executor, args[0]);
-  if (!GoodObject(thing)) {
-    safe_str(T(e_notvis), buff, bp);
-    return;
+  if (strchr(called_as, 'P')) {
+    (void) atr_iter_get_parent(executor, thing, pattern,
+			       !Can_Examine(executor, thing), lattr_helper,
+			       &lh);
+  } else {
+    (void) atr_iter_get(executor, thing, pattern,
+			!Can_Examine(executor, thing), lattr_helper, &lh);
   }
-  if (!Can_Examine(executor, thing)) {
-    safe_str(T(e_perm), buff, bp);
-    return;
-  }
-  safe_integer(AttrCount(thing), buff, bp);
 }
 
 /* ARGSUSED */
@@ -229,7 +266,7 @@ FUNCTION(fun_default)
   parse_attrib(executor, mstr, &thing, &attrib);
   if (GoodObject(thing) && attrib && Can_Read_Attr(executor, thing, attrib)) {
     /* Ok, we've got it */
-    dp = safe_uncompress(attrib->value);
+    dp = safe_atr_value(attrib);
     safe_str(dp, buff, bp);
     free(dp);
     return;
@@ -258,14 +295,12 @@ FUNCTION(fun_eval)
   }
   a = atr_get(thing, upcasestr(args[1]));
   if (a && Can_Read_Attr(executor, thing, a)) {
-    if (!CanEval(executor, thing)) {
+    if (!CanEvalAttr(executor, thing, a)) {
       safe_str(T(e_perm), buff, bp);
       return;
     }
-    tp = tbuf = safe_uncompress(a->value);
-#ifdef MEM_CHECK
+    tp = tbuf = safe_atr_value(a);
     add_check("fun_eval.attr_value");
-#endif
     process_expression(buff, bp, &tp, thing, executor, executor,
 		       PE_DEFAULT, PT_DEFAULT, pe_info);
     mush_free((Malloc_t) tbuf, "fun_eval.attr_value");
@@ -300,14 +335,12 @@ FUNCTION(fun_get_eval)
   }
   a = atr_get(thing, upcasestr(s));
   if (a && Can_Read_Attr(executor, thing, a)) {
-    if (!CanEval(executor, thing)) {
+    if (!CanEvalAttr(executor, thing, a)) {
       safe_str(T(e_perm), buff, bp);
       return;
     }
-    tp = tbuf = safe_uncompress(a->value);
-#ifdef MEM_CHECK
+    tp = tbuf = safe_atr_value(a);
     add_check("fun_eval.attr_value");
-#endif
     process_expression(buff, bp, &tp, thing, executor, executor,
 		       PE_DEFAULT, PT_DEFAULT, pe_info);
     mush_free((Malloc_t) tbuf, "fun_eval.attr_value");
@@ -338,15 +371,13 @@ FUNCTION(fun_edefault)
   *dp = '\0';
   parse_attrib(executor, mstr, &thing, &attrib);
   if (GoodObject(thing) && attrib && Can_Read_Attr(executor, thing, attrib)) {
-    if (!CanEval(executor, thing)) {
+    if (!CanEvalAttr(executor, thing, attrib)) {
       safe_str(T(e_perm), buff, bp);
       return;
     }
     /* Ok, we've got it */
-    sp = sbuf = safe_uncompress(attrib->value);
-#ifdef MEM_CHECK
+    sp = sbuf = safe_atr_value(attrib);
     add_check("fun_edefault.attr_value");
-#endif
     process_expression(buff, bp, &sp, thing, executor, executor,
 		       PE_DEFAULT, PT_DEFAULT, pe_info);
     mush_free((Malloc_t) sbuf, "fun_edefault.attr_value");
@@ -378,8 +409,8 @@ FUNCTION(fun_v)
     case '7':
     case '8':
     case '9':
-      if (wenv[c - '0'])
-	safe_str(wenv[c - '0'], buff, bp);
+      if (global_eval_context.wenv[c - '0'])
+	safe_str(global_eval_context.wenv[c - '0'], buff, bp);
       return;
     case '#':
       /* enactor dbref */
@@ -406,7 +437,7 @@ FUNCTION(fun_v)
       return;
     case 'c':
     case 'C':
-      safe_str(ccom, buff, bp);
+      safe_str(global_eval_context.ccom, buff, bp);
       return;
     }
   }
@@ -419,7 +450,10 @@ FUNCTION(fun_flags)
   dbref thing;
   char *p;
   ATTR *a;
-
+  if (nargs == 0) {
+    safe_str(list_all_flags("FLAG", NULL, executor, 0x1), buff, bp);
+    return;
+  }
   if ((p = strchr(args[0], '/')))
     *p++ = '\0';
   thing = match_thing(executor, args[0]);
@@ -435,12 +469,45 @@ FUNCTION(fun_flags)
       return;
     }
     safe_str(privs_to_letters(attr_privs, AL_FLAGS(a)), buff, bp);
+    if (atr_sub_branch(a))
+      safe_chr('`', buff, bp);
   } else {
     /* Object flags, visible to all */
     safe_str(unparse_flags(thing, executor), buff, bp);
   }
 }
 
+/* ARGSUSED */
+FUNCTION(fun_lflags)
+{
+  dbref thing;
+  char *p;
+  ATTR *a;
+  if (nargs == 0) {
+    safe_str(list_all_flags("FLAG", NULL, executor, 0x2), buff, bp);
+    return;
+  }
+  if ((p = strchr(args[0], '/')))
+    *p++ = '\0';
+  thing = match_thing(executor, args[0]);
+  if (!GoodObject(thing)) {
+    safe_str(T(e_notvis), buff, bp);
+    return;
+  }
+  if (p) {
+    /* Attribute flags, you must be able to read the attribute */
+    a = atr_get_noparent(thing, upcasestr(p));
+    if (!a || !Can_Read_Attr(executor, thing, a)) {
+      safe_str("#-1", buff, bp);
+      return;
+    }
+    safe_str(privs_to_string(attr_privs, AL_FLAGS(a)), buff, bp);
+  } else {
+    /* Object flags, visible to all */
+    safe_str(bits_to_string("FLAG", Flags(thing), executor, thing), buff, bp);
+  }
+}
+
 #ifdef WIN32
 #pragma warning( disable : 4761)	/* Disable bogus conversion warning */
 #endif
@@ -448,25 +515,13 @@ FUNCTION(fun_flags)
 FUNCTION(fun_haspower)
 {
   dbref it;
-  object_flag_type pwr;
 
   it = match_thing(executor, args[0]);
   if (!GoodObject(it)) {
     safe_str(T(e_notvis), buff, bp);
     return;
   }
-  if (HASPOWER_RESTRICTED)
-    if (!Can_Examine(executor, it)) {
-      notify(executor,
-	     T("We could let you see that, but then we'd have to kill you."));
-      safe_str("#-1", buff, bp);
-      return;
-    }
-  pwr = find_power(args[1]);
-  if (pwr == -1)
-    safe_str(T("#-1 NO SUCH POWER"), buff, bp);
-  else
-    safe_boolean(Powers(it) & pwr, buff, bp);
+  safe_boolean(sees_flag("POWER", executor, it, args[1]), buff, bp);
 }
 
 /* ARGSUSED */
@@ -474,19 +529,23 @@ FUNCTION(fun_powers)
 {
   dbref it;
 
+  if (nargs == 2) {
+    if (!command_check_byname(executor, "@power") || fun->flags & FN_NOSIDEFX) {
+      safe_str(T(e_perm), buff, bp);
+      return;
+    }
+    if (FUNCTION_SIDE_EFFECTS)
+      do_power(executor, args[0], args[1]);
+    else
+      safe_str(T(e_disabled), buff, bp);
+    return;
+  }
   it = match_thing(executor, args[0]);
   if (!GoodObject(it)) {
     safe_str(T(e_notvis), buff, bp);
     return;
   }
-  if (HASPOWER_RESTRICTED)
-    if (!Can_Examine(executor, it)) {
-      notify(executor,
-	     T("We could let you see that, but then we'd have to kill you."));
-      safe_str("#-1", buff, bp);
-      return;
-    }
-  safe_str(power_description(it), buff, bp);
+  safe_str(power_description(executor, it), buff, bp);
 }
 
 #ifdef WIN32
@@ -523,9 +582,9 @@ FUNCTION(fun_rnum)
 }
 
 /*
- * fun_lcon, fun_lexits, fun_con, fun_exit, fun_next, and next_exit were all
- * re-coded by d'mike, 7/12/91.  next_con was added at this time as well.
- *
+ * fun_lcon, fun_lexits, fun_con, fun_exit, fun_next were all
+ * re-coded by d'mike, 7/12/91.
+  *
  * The function behavior was changed by Amberyl, to remove what she saw
  * as a security problem, since mortals could get the contents of rooms
  * they didn't control, thus (if they were willing to go through the trouble)
@@ -555,14 +614,18 @@ FUNCTION(fun_rnum)
  */
 static dbref
 dbwalk(char *buff, char **bp, dbref executor, dbref enactor,
-       int type, dbref loc, dbref after, int skipdark)
+       int type, dbref loc, dbref after, int skipdark,
+       int start, int count, int *retcount)
 {
   dbref result;
   int first;
+  int nthing;
   dbref thing;
-  int validloc, validthing;
+  int validloc;
   dbref startdb;
 
+  nthing = 0;
+
   if (!GoodObject(loc)) {
     if (buff)
       safe_str("#-1", buff, bp);
@@ -582,12 +645,21 @@ dbwalk(char *buff, char **bp, dbref executor, dbref enactor,
        (enactor == loc))) {
     first = 1;
     DOLIST_VISIBLE(thing, startdb, executor) {
-      if (type == TYPE_PLAYER)
-	validthing = skipdark ? (IsPlayer(thing) && !Dark(thing)
-				 && Connected(thing)) : IsPlayer(thing);
-      else
-	validthing = skipdark ? (!Dark(thing)) : 1;
-      if (validthing) {
+      /* Skip if:
+       * - We're not checking this type
+       * - We can't interact with this thing
+       * - We're only listing visual objects, and it's dark
+       * - It's a player, not connected, and skipdark is true
+       *   Funkily, lvcon() shows unconnected players, so we
+       *   use type == TYPE_PLAYER for this check. :-/.
+       */
+      if (!(Typeof(thing) & type) ||
+	  !can_interact(thing, executor, INTERACT_SEE) ||
+	  (skipdark && Dark(thing)) ||
+	  ((type == TYPE_PLAYER) && skipdark && !Connected(thing)))
+	continue;
+      nthing += 1;
+      if (count < 1 || (nthing >= start && nthing < start + count)) {
 	if (buff) {
 	  if (first)
 	    first = 0;
@@ -595,13 +667,15 @@ dbwalk(char *buff, char **bp, dbref executor, dbref enactor,
 	    safe_chr(' ', buff, bp);
 	  safe_dbref(thing, buff, bp);
 	}
-	if (result == NOTHING) {
-	  if (after == NOTHING)
-	    result = thing;
-	  if (after == thing)
-	    after = NOTHING;
-	}
       }
+      if (result == NOTHING) {
+	if (after == NOTHING)
+	  result = thing;
+	if (after == thing)
+	  after = NOTHING;
+      }
+      if (retcount)
+	*retcount = nthing;
     }
   } else if (buff)
     safe_str("#-1", buff, bp);
@@ -609,61 +683,87 @@ dbwalk(char *buff, char **bp, dbref executor, dbref enactor,
 }
 
 /* ARGSUSED */
-FUNCTION(fun_lcon)
+FUNCTION(fun_dbwalker)
 {
   dbref loc = match_thing(executor, args[0]);
-  dbwalk(buff, bp, executor, enactor, TYPE_THING, loc, NOTHING, 0);
-}
+  int start, count;
+  int vis = 0;
+  int type = 0;
+  int result = 0;
+  const char *ptr = called_as;
+  char *buffptr = buff;
+  char **bptr = bp;
 
-/* ARGSUSED */
-FUNCTION(fun_lvcon)
-{
-  dbref loc = match_thing(executor, args[0]);
-  dbwalk(buff, bp, executor, enactor, TYPE_THING, loc, NOTHING, 1);
+  start = count = 0;
+  buffptr = buff;
+  bptr = bp;
+
+  switch (*(ptr++)) {
+  case 'X':
+    if (!is_strict_integer(args[1]) || !is_strict_integer(args[2])) {
+      safe_str(T(e_int), buff, bp);
+      return;
+    }
+    start = parse_integer(args[1]);
+    count = parse_integer(args[2]);
+    if (start < 1 || count < 1) {
+      safe_str(T("#-1 ARGUMENT OUT OF RANGE"), buff, bp);
+      return;
+    }
+    break;
+  case 'N':
+    buffptr = NULL;
+    bptr = NULL;
+    break;
+  }
+
+  if (*ptr == 'V') {
+    vis = 1;
+    ptr++;
+  }
+
+  switch (*ptr) {
+  case 'C':			/* con */
+    type = TYPE_THING | TYPE_PLAYER;
+    break;
+  case 'T':			/* things */
+    type = TYPE_THING;
+    break;
+  case 'P':			/* players */
+    type = TYPE_PLAYER;
+    break;
+  case 'E':			/* exits */
+    type = TYPE_EXIT;
+    break;
+  default:
+    /* This should never be reached ... */
+    type = TYPE_THING | TYPE_PLAYER;
+  }
+
+  dbwalk(buffptr, bptr, executor, enactor, type, loc, NOTHING,
+	 vis, start, count, &result);
+
+  if (!buffptr) {
+    safe_integer(result, buff, bp);
+  }
 }
 
 /* ARGSUSED */
 FUNCTION(fun_con)
 {
   dbref loc = match_thing(executor, args[0]);
-  safe_dbref(dbwalk(NULL, NULL, executor, enactor, TYPE_THING, loc, NOTHING, 0),
-	     buff, bp);
-}
-
-/* ARGSUSED */
-FUNCTION(fun_lexits)
-{
-  dbref loc = match_thing(executor, args[0]);
-  dbwalk(buff, bp, executor, enactor, TYPE_EXIT, loc, NOTHING, 0);
-}
-
-/* ARGSUSED */
-FUNCTION(fun_lvexits)
-{
-  dbref loc = match_thing(executor, args[0]);
-  dbwalk(buff, bp, executor, enactor, TYPE_EXIT, loc, NOTHING, 1);
+  safe_dbref(dbwalk
+	     (NULL, NULL, executor, enactor, TYPE_THING, loc, NOTHING, 0, 0, 0,
+	      NULL), buff, bp);
 }
 
 /* ARGSUSED */
 FUNCTION(fun_exit)
 {
   dbref loc = match_thing(executor, args[0]);
-  safe_dbref(dbwalk(NULL, NULL, executor, enactor, TYPE_EXIT, loc, NOTHING, 0),
-	     buff, bp);
-}
-
-/* ARGSUSED */
-FUNCTION(fun_lplayers)
-{
-  dbref loc = match_thing(executor, args[0]);
-  dbwalk(buff, bp, executor, enactor, TYPE_PLAYER, loc, NOTHING, 0);
-}
-
-/* ARGSUSED */
-FUNCTION(fun_lvplayers)
-{
-  dbref loc = match_thing(executor, args[0]);
-  dbwalk(buff, bp, executor, enactor, TYPE_PLAYER, loc, NOTHING, 1);
+  safe_dbref(dbwalk
+	     (NULL, NULL, executor, enactor, TYPE_EXIT, loc, NOTHING, 0, 0, 0,
+	      NULL), buff, bp);
 }
 
 /* ARGSUSED */
@@ -675,14 +775,14 @@ FUNCTION(fun_next)
     switch (Typeof(it)) {
     case TYPE_EXIT:
       safe_dbref(dbwalk
-		 (NULL, NULL, executor, enactor, TYPE_EXIT, Source(it), it, 0),
-		 buff, bp);
+		 (NULL, NULL, executor, enactor, TYPE_EXIT, Source(it), it, 0,
+		  0, 0, NULL), buff, bp);
       break;
     case TYPE_THING:
     case TYPE_PLAYER:
       safe_dbref(dbwalk
 		 (NULL, NULL, executor, enactor, TYPE_THING, Location(it), it,
-		  0), buff, bp);
+		  0, 0, 0, NULL), buff, bp);
       break;
     default:
       safe_str("#-1", buff, bp);
@@ -942,7 +1042,7 @@ FUNCTION(fun_hasflag)
     if (!GoodObject(thing))
       safe_str(T(e_notvis), buff, bp);
     else
-      safe_boolean(sees_flag(executor, thing, args[1]), buff, bp);
+      safe_boolean(sees_flag("FLAG", executor, thing, args[1]), buff, bp);
   }
 }
 
@@ -971,6 +1071,10 @@ FUNCTION(fun_hastype)
   case 'T':
     safe_boolean(IsThing(it), buff, bp);
     break;
+  case 'g':
+  case 'G':
+    safe_boolean(IsGarbage(it), buff, bp);
+    break;
   default:
     safe_str(T("#-1 NO SUCH TYPE"), buff, bp);
     break;
@@ -981,27 +1085,144 @@ FUNCTION(fun_hastype)
 FUNCTION(fun_orflags)
 {
   dbref it = match_thing(executor, args[0]);
-  safe_boolean(flaglist_check(executor, it, args[1], 0), buff, bp);
+  if (!strcmp(called_as, "ORPOWERS"))
+    safe_boolean(flaglist_check("POWER", executor, it, args[1], 0), buff, bp);
+  else
+    safe_boolean(flaglist_check("FLAG", executor, it, args[1], 0), buff, bp);
 }
 
 /* ARGSUSED */
 FUNCTION(fun_andflags)
 {
   dbref it = match_thing(executor, args[0]);
-  safe_boolean(flaglist_check(executor, it, args[1], 1), buff, bp);
+  if (!strcmp(called_as, "ANDPOWERS"))
+    safe_boolean(flaglist_check("POWER", executor, it, args[1], 1), buff, bp);
+  else
+    safe_boolean(flaglist_check("FLAG", executor, it, args[1], 1), buff, bp);
+}
+
+/* ARGSUSED */
+FUNCTION(fun_orlflags)
+{
+  dbref it = match_thing(executor, args[0]);
+  if (!strcmp(called_as, "ORLPOWERS"))
+    safe_boolean(flaglist_check_long("POWER", executor, it, args[1], 0), buff,
+		 bp);
+  else
+    safe_boolean(flaglist_check_long("FLAG", executor, it, args[1], 0), buff,
+		 bp);
+}
+
+/* ARGSUSED */
+FUNCTION(fun_andlflags)
+{
+  dbref it = match_thing(executor, args[0]);
+  if (!strcmp(called_as, "ANDLPOWERS"))
+    safe_boolean(flaglist_check_long("POWER", executor, it, args[1], 1), buff,
+		 bp);
+  else
+    safe_boolean(flaglist_check_long("FLAG", executor, it, args[1], 1), buff,
+		 bp);
 }
 
 static lock_type
-get_locktype(str)
-    char *str;
+get_locktype(char *str)
 {
   /* figure out a lock type */
 
   if (!str || !*str)
     return Basic_Lock;
+  if (!strncasecmp(str, "USER:", 5))
+    str += 5;
   return upcasestr(str);
 }
 
+/* ARGSUSED */
+FUNCTION(fun_locks)
+{
+  dbref thing = match_thing(executor, args[0]);
+  lock_list *ll;
+  const lock_list *p;
+  int first = 1;
+
+  if (!GoodObject(thing)) {
+    safe_str(T(e_notvis), buff, bp);
+    return;
+  }
+
+  for (ll = Locks(thing); ll; ll = ll->next) {
+    p = get_lockproto(L_TYPE(ll));
+    if (!first) {
+      safe_chr(' ', buff, bp);
+    }
+    first = 0;
+    if (!p)
+      safe_str("USER:", buff, bp);
+    safe_str(L_TYPE(ll), buff, bp);
+  }
+}
+
+/* ARGSUSED */
+FUNCTION(fun_lockflags)
+{
+  dbref it;
+  char *p;
+  int fullname = 0;
+  lock_list *ll;
+  lock_type ltype;
+
+  if (called_as[1] == 'L')	/* LLOCKFLAGS */
+    fullname = 1;
+
+  if (nargs == 0) {
+    if (fullname)
+      list_lock_flags_long(buff, bp);
+    else
+      list_lock_flags(buff, bp);
+    return;
+  }
+
+  if ((p = strchr(args[0], '/')))
+    *(p++) = '\0';
+
+  it = match_thing(executor, args[0]);
+  if (!GoodObject(it)) {
+    safe_str(T(e_notvis), buff, bp);
+    return;
+  }
+  ltype = get_locktype(p);
+
+  if (GoodObject(it) && (ltype != NULL)
+      && Can_Read_Lock(executor, it, ltype)) {
+    ll = getlockstruct(it, ltype);
+    if (ll) {
+      if (fullname)
+	safe_str(lock_flags_long(ll), buff, bp);
+      else
+	safe_str(lock_flags(ll), buff, bp);
+      return;
+    } else {
+      safe_str("#-1 NO SUCH LOCK", buff, bp);
+      return;
+    }
+  }
+  safe_str("#-1 NO SUCH LOCK", buff, bp);
+}
+
+/* ARGSUSED */
+FUNCTION(fun_lset)
+{
+  if (!FUNCTION_SIDE_EFFECTS) {
+    safe_str(T(e_disabled), buff, bp);
+    return;
+  }
+  if (!command_check_byname(executor, "@lset") || fun->flags & FN_NOSIDEFX) {
+    safe_str(T(e_perm), buff, bp);
+    return;
+  }
+  do_lset(executor, args[0], args[1]);
+}
+
 /* ARGSUSED */
 FUNCTION(fun_lock)
 {
@@ -1020,12 +1241,12 @@ FUNCTION(fun_lock)
       safe_str(T(e_perm), buff, bp);
       return;
     }
-#ifdef FUNCTION_SIDE_EFFECTS
-    do_lock(executor, args[0], args[1], ltype);
-#else
-    safe_str(T(e_disabled), buff, bp);
-#endif
-    return;
+    if (FUNCTION_SIDE_EFFECTS)
+      do_lock(executor, args[0], args[1], ltype);
+    else {
+      safe_str(T(e_disabled), buff, bp);
+      return;
+    }
   }
   if (GoodObject(it) && (ltype != NULL)
       && Can_Read_Lock(executor, it, ltype)) {
@@ -1054,7 +1275,10 @@ FUNCTION(fun_elock)
     safe_str("#-1", buff, bp);
     return;
   }
-  safe_boolean(eval_lock(victim, it, ltype), buff, bp);
+  if (Can_Locate(executor, victim))
+    safe_boolean(eval_lock(victim, it, ltype), buff, bp);
+  else
+    safe_str("#-1", buff, bp);
   return;
 }
 
@@ -1085,22 +1309,28 @@ FUNCTION(fun_loc)
     safe_str("#-1", buff, bp);
 }
 
-#ifdef CREATION_TIMES
+/* ARGSUSED */
+FUNCTION(fun_objid)
+{
+  dbref it = match_thing(executor, args[0]);
+
+  if (GoodObject(it)) {
+    safe_dbref(it, buff, bp);
+    safe_chr(':', buff, bp);
+    safe_integer(CreTime(it), buff, bp);
+  } else
+    safe_str(T(e_notvis), buff, bp);
+}
+
 /* ARGSUSED */
 FUNCTION(fun_ctime)
 {
   dbref it = match_thing(executor, args[0]);
-  char *s;
 
-  if (GoodObject(it)) {
-    s = (char *) ctime(&CreTime(it));
-    s[strlen(s) - 1] = '\0';
-    if (s[8] == ' ')
-      s[8] = '0';
-    safe_str(s, buff, bp);
-    return;
-  }
-  safe_str(T(e_notvis), buff, bp);
+  if (GoodObject(it))
+    safe_str(show_time(CreTime(it), 0), buff, bp);
+  else
+    safe_str(T(e_notvis), buff, bp);
 }
 
 /* ARGSUSED */
@@ -1111,15 +1341,9 @@ FUNCTION(fun_mtime)
     safe_str(T(e_notvis), buff, bp);
   else if (!Can_Examine(executor, it) || IsPlayer(it))
     safe_str(T(e_perm), buff, bp);
-  else {
-    char *s = (char *) ctime(&ModTime(it));
-    s[strlen(s) - 1] = '\0';
-    if (s[8] == ' ')
-      s[8] = '0';
-    safe_str(s, buff, bp);
-  }
+  else
+    safe_str(show_time(ModTime(it), 0), buff, bp);
 }
-#endif
 
 /* ARGSUSED */
 FUNCTION(fun_where)
@@ -1189,12 +1413,12 @@ FUNCTION(fun_zone)
       safe_str(T(e_perm), buff, bp);
       return;
     }
-#ifdef FUNCTION_SIDE_EFFECTS
-    (void) do_chzone(executor, args[0], args[1], 1);
-#else
-    safe_str(T(e_disabled), buff, bp);
-    return;
-#endif
+    if (FUNCTION_SIDE_EFFECTS)
+      (void) do_chzone(executor, args[0], args[1], 1);
+    else {
+      safe_str(T(e_disabled), buff, bp);
+      return;
+    }
   }
   it = match_thing(executor, args[0]);
   if (!GoodObject(it))
@@ -1215,12 +1439,12 @@ FUNCTION(fun_parent)
       safe_str(T(e_perm), buff, bp);
       return;
     }
-#ifdef FUNCTION_SIDE_EFFECTS
-    do_parent(executor, args[0], args[1]);
-#else
-    safe_str(T(e_disabled), buff, bp);
-    return;
-#endif
+    if (FUNCTION_SIDE_EFFECTS)
+      do_parent(executor, args[0], args[1]);
+    else {
+      safe_str(T(e_disabled), buff, bp);
+      return;
+    }
   }
   it = match_thing(executor, args[0]);
   if (!GoodObject(it))
@@ -1299,7 +1523,7 @@ FUNCTION(fun_money)
    * if its owner is no_pay. Softcode can check money(owner(XX)) if 
    * they want to allow objects to pay like their owners.
    */
-  if (Hasprivs(it) || (Powers(it) & NO_PAY))
+  if (has_power_by_name(it, "NO_PAY", NOTYPE))
     safe_integer(MAX_PENNIES, buff, bp);
   else
     safe_integer(Pennies(it), buff, bp);
@@ -1340,12 +1564,11 @@ FUNCTION(fun_name)
       safe_str(T(e_perm), buff, bp);
       return;
     }
-#ifdef FUNCTION_SIDE_EFFECTS
-    do_name(executor, args[0], args[1]);
-#else
-    safe_str(T(e_disabled), buff, bp);
+    if (FUNCTION_SIDE_EFFECTS)
+      do_name(executor, args[0], args[1]);
+    else
+      safe_str(T(e_disabled), buff, bp);
     return;
-#endif
   }
   it = match_thing(executor, args[0]);
   if (GoodObject(it))
@@ -1364,6 +1587,16 @@ FUNCTION(fun_fullname)
     safe_str(T(e_notvis), buff, bp);
 }
 
+/* ARGSUSED */
+FUNCTION(fun_accname)
+{
+  dbref it = match_thing(executor, args[0]);
+  if (GoodObject(it))
+    safe_str(accented_name(it), buff, bp);
+  else
+    safe_str(T(e_notvis), buff, bp);
+}
+
 /* ARGSUSED */
 FUNCTION(fun_iname)
 {
@@ -1395,7 +1628,7 @@ FUNCTION(fun_pmatch)
   if (*args[0] == '*')
     target = lookup_player(args[0] + 1);
   else if (*args[0] == NUMBER_TOKEN) {
-    target = parse_dbref(args[0]);
+    target = parse_objid(args[0]);
     if (!(GoodObject(target) && IsPlayer(target))) {
       notify(executor, T("No match."));
       safe_str("#-1", buff, bp);
@@ -1430,8 +1663,8 @@ FUNCTION(fun_pmatch)
 FUNCTION(fun_locate)
 {
   dbref looker;
-  object_flag_type pref_type;
-  dbref item;
+  int pref_type;
+  dbref item, loc;
   char *p;
   int keys = 0;
   int ambig_ok = 0;
@@ -1441,12 +1674,10 @@ FUNCTION(fun_locate)
   /* find out what we're matching in relation to */
   looker = match_thing(executor, args[0]);
   if (!GoodObject(looker)) {
-    notify(executor, T("I don't see that here."));
     safe_str("#-1", buff, bp);
     return;
   }
   if (!See_All(executor) && !controls(executor, looker)) {
-    notify(executor, T("Permission denied."));
     safe_str("#-1", buff, bp);
     return;
   }
@@ -1503,6 +1734,9 @@ FUNCTION(fun_locate)
     case 'n':
       match_flags |= MAT_NEIGHBOR;
       break;
+    case 'y':
+      match_flags |= MAT_PMATCH;
+      break;
     case 'p':
       match_flags |= MAT_PLAYER;
       break;
@@ -1527,18 +1761,38 @@ FUNCTION(fun_locate)
   else
     item = last_match_result(looker, args[1], pref_type, match_flags);
 
-  if (item == NOTHING)
-    notify(executor, T("Nothing found."));
-  else if (item == AMBIGUOUS)
-    notify(executor, T("More than one match found."));
-  if (GoodObject(item) && (force_type && pref_type != NOTYPE
-			   && !(Typeof(item) == pref_type)))
-    safe_dbref(NOTHING, buff, bp);
-  else
+  if (!GoodObject(item)) {
     safe_dbref(item, buff, bp);
+    return;
+  }
+
+  if (force_type && pref_type != NOTYPE && !(Typeof(item) == pref_type)) {
+    safe_dbref(NOTHING, buff, bp);
+    return;
+  }
+
+  /* To locate it, you must either be able to examine its location
+   * or be able to see the item.
+   */
+  loc = Location(item);
+  if (GoodObject(loc)) {
+    if (Can_Examine(executor, loc))
+      safe_dbref(item, buff, bp);
+    else if (can_interact(item, executor, INTERACT_SEE)
+	     && (!DarkLegal(item) || (Dark(loc) && Light(item))))
+      safe_dbref(item, buff, bp);
+    else
+      safe_dbref(NOTHING, buff, bp);
+  } else {
+    if (can_interact(item, executor, INTERACT_SEE)
+	&& (See_All(executor) || !DarkLegal(item)))
+      safe_dbref(item, buff, bp);
+    else
+      safe_dbref(NOTHING, buff, bp);
+  }
+  return;
 }
 
-#ifdef FUNCTION_SIDE_EFFECTS
 
 /* --------------------------------------------------------------------------
  * Creation functions: CREATE, PCREATE, OPEN, DIG
@@ -1549,6 +1803,11 @@ FUNCTION(fun_create)
 {
   int cost;
 
+  if (!FUNCTION_SIDE_EFFECTS) {
+    safe_str(T(e_disabled), buff, bp);
+    return;
+  }
+
   if (!command_check_byname(executor, "@create") || fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
@@ -1563,6 +1822,10 @@ FUNCTION(fun_create)
 /* ARGSUSED */
 FUNCTION(fun_pcreate)
 {
+  if (!FUNCTION_SIDE_EFFECTS) {
+    safe_str(T(e_disabled), buff, bp);
+    return;
+  }
   if (!command_check_byname(executor, "@pcreate") || fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
@@ -1573,6 +1836,10 @@ FUNCTION(fun_pcreate)
 /* ARGSUSED */
 FUNCTION(fun_open)
 {
+  if (!FUNCTION_SIDE_EFFECTS) {
+    safe_str(T(e_disabled), buff, bp);
+    return;
+  }
   if (!command_check_byname(executor, "@open") || fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
@@ -1583,6 +1850,10 @@ FUNCTION(fun_open)
 /* ARGSUSED */
 FUNCTION(fun_dig)
 {
+  if (!FUNCTION_SIDE_EFFECTS) {
+    safe_str(T(e_disabled), buff, bp);
+    return;
+  }
   if (!command_check_byname(executor, "@dig") || fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
@@ -1593,6 +1864,10 @@ FUNCTION(fun_dig)
 /* ARGSUSED */
 FUNCTION(fun_clone)
 {
+  if (!FUNCTION_SIDE_EFFECTS) {
+    safe_str(T(e_disabled), buff, bp);
+    return;
+  }
   if (!command_check_byname(executor, "@clone") || fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
@@ -1608,6 +1883,10 @@ FUNCTION(fun_clone)
 /* ARGSUSED */
 FUNCTION(fun_link)
 {
+  if (!FUNCTION_SIDE_EFFECTS) {
+    safe_str(T(e_disabled), buff, bp);
+    return;
+  }
   if (!command_check_byname(executor, "@link") || fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
@@ -1618,6 +1897,10 @@ FUNCTION(fun_link)
 /* ARGSUSED */
 FUNCTION(fun_set)
 {
+  if (!FUNCTION_SIDE_EFFECTS) {
+    safe_str(T(e_disabled), buff, bp);
+    return;
+  }
   if (!command_check_byname(executor, "@set") || fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
@@ -1628,6 +1911,10 @@ FUNCTION(fun_set)
 /* ARGSUSED */
 FUNCTION(fun_wipe)
 {
+  if (!FUNCTION_SIDE_EFFECTS) {
+    safe_str(T(e_disabled), buff, bp);
+    return;
+  }
   if (!command_check_byname(executor, "@wipe") || fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
@@ -1643,72 +1930,24 @@ FUNCTION(fun_wipe)
 /* ARGSUSED */
 FUNCTION(fun_tel)
 {
+  int silent = 0;
+  int inside = 0;
+  if (!FUNCTION_SIDE_EFFECTS) {
+    safe_str(T(e_disabled), buff, bp);
+    return;
+  }
   if (!command_check_byname(executor, "@tel") || fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
   }
-  do_teleport(executor, args[0], args[1], 0);
+  if (nargs > 2)
+    silent = parse_boolean(args[2]);
+  if (nargs > 3)
+    silent = parse_boolean(args[3]);
+  do_teleport(executor, args[0], args[1], silent, inside);
 }
 
 
-#else
-
-/* ARGSUSED */
-FUNCTION(fun_create)
-{
-  safe_str(T(e_disabled), buff, bp);
-}
-
-/* ARGSUSED */
-FUNCTION(fun_pcreate)
-{
-  safe_str(T(e_disabled), buff, bp);
-}
-
-/* ARGSUSED */
-FUNCTION(fun_open)
-{
-  safe_str(T(e_disabled), buff, bp);
-}
-
-/* ARGSUSED */
-FUNCTION(fun_dig)
-{
-  safe_str(T(e_disabled), buff, bp);
-}
-
-/* ARGSUSED */
-FUNCTION(fun_clone)
-{
-  safe_str(T(e_disabled), buff, bp);
-}
-
-/* ARGSUSED */
-FUNCTION(fun_link)
-{
-  safe_str(T(e_disabled), buff, bp);
-}
-
-/* ARGSUSED */
-FUNCTION(fun_set)
-{
-  safe_str(T(e_disabled), buff, bp);
-}
-
-/* ARGSUSED */
-FUNCTION(fun_wipe)
-{
-  safe_str(T(e_disabled), buff, bp);
-}
-
-/* ARGSUSED */
-FUNCTION(fun_tel)
-{
-  safe_str(T(e_disabled), buff, bp);
-}
-
-#endif				/* FUNCTION_SIDE_EFFECTS */
-
 /* ARGSUSED */
 FUNCTION(fun_isdbref)
 {
@@ -1736,9 +1975,7 @@ FUNCTION(fun_grep)
   }
   tp = grep_util(executor, it, args[1], args[2], arglens[2],
 		 strcmp(called_as, "GREP"));
-#ifdef MEM_CHECK
   add_check("fun_grep.attr_list");
-#endif
   safe_str(tp, buff, bp);
   mush_free((Malloc_t) tp, "fun_grep.attr_list");
 }
@@ -1762,7 +1999,7 @@ FUNCTION(fun_lstats)
     }
   }
   if (!Search_All(executor)) {
-    if (who != ANY_OWNER && who != executor) {
+    if (who != ANY_OWNER && !controls(executor, who)) {
       safe_str(T(e_perm), buff, bp);
       return;
     }
@@ -1792,16 +2029,16 @@ FUNCTION(fun_atrlock)
     status = 1;
 
   if (status == 1) {
-#ifdef FUNCTION_SIDE_EFFECTS
-    if (!command_check_byname(executor, "@atrlock") || fun->flags & FN_NOSIDEFX) {
-      safe_str(T(e_perm), buff, bp);
+    if (FUNCTION_SIDE_EFFECTS) {
+      if (!command_check_byname(executor, "@atrlock")
+	  || fun->flags & FN_NOSIDEFX) {
+	safe_str(T(e_perm), buff, bp);
+	return;
+      }
+      do_atrlock(executor, args[0], args[1]);
       return;
-    }
-    do_atrlock(executor, args[0], args[1]);
-    return;
-#else
-    safe_str(T(e_disabled), buff, bp);
-#endif
+    } else
+      safe_str(T(e_disabled), buff, bp);
     return;
   }
 
@@ -1824,138 +2061,11 @@ FUNCTION(fun_atrlock)
 
   ptr = atr_get_noparent(thing, strupper(p));
   if (ptr && Can_Read_Attr(executor, thing, ptr))
-    safe_boolean(AL_FLAGS(ptr) & AF_LOCKED, buff, bp);
+    safe_boolean(AF_Locked(ptr), buff, bp);
   else
     safe_str("#-1", buff, bp);
 }
 
-#ifdef CHAT_SYSTEM
-FUNCTION(fun_channels)
-{
-  dbref it;
-  char sep = ' ';
-  CHAN *c;
-  CHANLIST *cl;
-  CHANUSER *u;
-  int can_ex;
-
-  /* There are these possibilities:
-   *  no args - just a list of all channels
-   *   2 args - object, delimiter
-   *   1 arg - either object or delimiter. If it's longer than 1 char,
-   *           we treat it as an object.
-   * You can see an object's channels if you can examine it.
-   * Otherwise you can see only channels that you share with
-   * it where it's not hidden.
-   */
-  if (nargs >= 1) {
-    /* Given an argument, return list of channels it's on */
-    it = match_result(executor, args[0], NOTYPE, MAT_EVERYTHING);
-    if (GoodObject(it)) {
-      int first = 1;
-      if (!delim_check(buff, bp, nargs, args, 2, &sep))
-	return;
-      can_ex = Can_Examine(executor, it);
-      for (cl = Chanlist(it); cl; cl = cl->next) {
-	if (can_ex || ((u = onchannel(it, cl->chan)) &&!Chanuser_Hide(u)
-		       && onchannel(executor, cl->chan))) {
-	  if (!first)
-	    safe_chr(sep, buff, bp);
-	  safe_str(ChanName(cl->chan), buff, bp);
-	  first = 0;
-	}
-      }
-      return;
-    } else {
-      /* args[0] didn't match. Maybe it's a delimiter? */
-      if (arglens[0] > 1) {
-	if (it == NOTHING)
-	  notify(executor, T("I can't see that here."));
-	else if (it == AMBIGUOUS)
-	  notify(executor, T("I don't know which thing you mean."));
-	return;
-      } else if (!delim_check(buff, bp, nargs, args, 1, &sep))
-	return;
-    }
-  }
-  /* No arguments (except maybe delimiter) - return list of all channels */
-  for (c = channels; c; c = c->next) {
-    if (Chan_Can_See(c, executor)) {
-      if (c != channels)
-	safe_chr(sep, buff, bp);
-      safe_str(ChanName(c), buff, bp);
-    }
-  }
-  return;
-}
-
-FUNCTION(fun_clock)
-{
-  CHAN *c = NULL;
-  char *p = NULL;
-  struct boolexp *lock_ptr = NULL;
-  int which_lock = 0;
-
-  if ((p = strchr(args[0], '/'))) {
-    *p++ = '\0';
-  } else {
-    p = (char *) "JOIN";
-  }
-
-  switch (find_channel(args[0], &c, executor)) {
-  case CMATCH_NONE:
-    safe_str(T("#-1 NO SUCH CHANNEL"), buff, bp);
-    return;
-  case CMATCH_AMBIG:
-    safe_str("#-2 AMBIGUOUS CHANNEL MATCH", buff, bp);
-    return;
-  }
-
-  if (!strcasecmp(p, "JOIN")) {
-    which_lock = CL_JOIN;
-    lock_ptr = ChanJoinLock(c);
-  } else if (!strcasecmp(p, "SPEAK")) {
-    which_lock = CL_SPEAK;
-    lock_ptr = ChanSpeakLock(c);
-  } else if (!strcasecmp(p, "MOD")) {
-    which_lock = CL_MOD;
-    lock_ptr = ChanModLock(c);
-  } else if (!strcasecmp(p, "SEE")) {
-    which_lock = CL_SEE;
-    lock_ptr = ChanSeeLock(c);
-  } else if (!strcasecmp(p, "HIDE")) {
-    which_lock = CL_HIDE;
-    lock_ptr = ChanHideLock(c);
-  } else {
-    safe_str(T("#-1 NO SUCH LOCK TYPE"), buff, bp);
-    return;
-  }
-
-  if (nargs == 2) {
-#ifdef FUNCTION_SIDE_EFFECTS
-    if (!command_check_byname(executor, "@clock")) {
-      safe_str(T(e_perm), buff, bp);
-      return;
-    }
-    do_chan_lock(executor, args[0], args[1], which_lock);
-    return;
-#else
-    safe_str(T(e_disabled), buff, bp);
-#endif
-  }
-
-  if (Chan_Can_Decomp(c, executor)) {
-    safe_str(unparse_boolexp(executor, lock_ptr, 1), buff, bp);
-    return;
-  } else {
-    safe_str(T(e_perm), buff, bp);
-    return;
-  }
-}
-
-#endif				/* CHAT_SYSTEM */
-
-
 /* ARGSUSED */
 FUNCTION(fun_followers)
 {
@@ -1972,7 +2082,7 @@ FUNCTION(fun_followers)
   a = atr_get_noparent(thing, "FOLLOWERS");
   if (!a)
     return;
-  s = uncompress(a->value);
+  s = atr_value(a);
   res = trim_space_sep(s, ' ');
   safe_str(res, buff, bp);
   return;;
@@ -1994,7 +2104,7 @@ FUNCTION(fun_following)
   a = atr_get_noparent(thing, "FOLLOWING");
   if (!a)
     return;
-  s = uncompress(a->value);
+  s = atr_value(a);
   res = trim_space_sep(s, ' ');
   safe_str(res, buff, bp);
   return;;
