@@ -1,3 +1,10 @@
+/**
+ * \file funmisc.c
+ *
+ * \brief Miscellaneous functions for mushcode.
+ *
+ *
+ */
 #include "copyrite.h"
 
 #include "config.h"
@@ -8,9 +15,10 @@
 #include "case.h"
 #include "externs.h"
 #include "version.h"
-#include "extchat.h"
 #include "htab.h"
 #include "flags.h"
+#include "match.h"
+#include "mushdb.h"
 #include "dbdefs.h"
 #include "parse.h"
 #include "function.h"
@@ -23,8 +31,6 @@
 #pragma warning( disable : 4761)	/* NJG: disable warning re conversion */
 #endif
 
-extern time_t start_time, first_start_time;
-extern int reboot_count;
 extern FUN flist[];
 static char *soundex(char *str);
 extern char cf_motd_msg[BUFFER_LEN], cf_wizmotd_msg[BUFFER_LEN],
@@ -35,7 +41,7 @@ extern HASHTAB htab_function;
 FUNCTION(fun_valid)
 {
   /* Checks to see if a given <something> is valid as a parameter of a
-   * given type (such as an object name.
+   * given type (such as an object name.)
    */
 
   if (!args[0] || !*args[0])
@@ -45,7 +51,13 @@ FUNCTION(fun_valid)
   else if (!strcasecmp(args[0], "attrname"))
     safe_boolean(good_atr_name(upcasestr(args[1])), buff, bp);
   else if (!strcasecmp(args[0], "playername"))
-    safe_boolean(ok_player_name(args[1]), buff, bp);
+    safe_boolean(ok_player_name(args[1], executor), buff, bp);
+  else if (!strcasecmp(args[0], "password"))
+    safe_boolean(ok_password(args[1]), buff, bp);
+  else if (!strcasecmp(args[0], "command"))
+    safe_boolean(ok_command_name(upcasestr(args[1])), buff, bp);
+  else if (!strcasecmp(args[0], "function"))
+    safe_boolean(ok_command_name(upcasestr(args[1])), buff, bp);
   else
     safe_str("#-1", buff, bp);
 }
@@ -53,97 +65,90 @@ FUNCTION(fun_valid)
 /* ARGSUSED */
 FUNCTION(fun_pemit)
 {
-  if (!command_check_byname(executor, "@pemit") || fun->flags & FN_NOSIDEFX) {
+  int ns = string_prefix(called_as, "NS");
+  int flags = PEMIT_LIST;
+  if (!command_check_byname(executor, ns ? "@nspemit" : "@pemit") ||
+      fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
   }
   orator = executor;
-  do_pemit_list(executor, args[0], args[1], 1);
+  if (ns)
+    flags |= PEMIT_SPOOF;
+  do_pemit_list(executor, args[0], args[1], flags);
 }
 
-/* ARGSUSED */
-FUNCTION(fun_nspemit)
-{
-  if (!command_check_byname(executor, "@nspemit") || fun->flags & FN_NOSIDEFX) {
-    safe_str(T(e_perm), buff, bp);
-    return;
-  }
-  orator = executor;
-  do_pemit_list(executor, args[0], args[1], 0);
-}
 
 /* ARGSUSED */
 FUNCTION(fun_oemit)
 {
-  if (!command_check_byname(executor, "@oemit") || fun->flags & FN_NOSIDEFX) {
+  int ns = string_prefix(called_as, "NS");
+  int flags = ns ? PEMIT_SPOOF : 0;
+  if (!command_check_byname(executor, ns ? "@nsoemit" : "@oemit") ||
+      fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
   }
   orator = executor;
-  do_oemit_list(executor, args[0], args[1]);
+  do_oemit_list(executor, args[0], args[1], flags);
 }
 
 /* ARGSUSED */
 FUNCTION(fun_emit)
 {
-  if (!command_check_byname(executor, "@emit") || fun->flags & FN_NOSIDEFX) {
+  int ns = string_prefix(called_as, "NS");
+  int flags = ns ? PEMIT_SPOOF : 0;
+  if (!command_check_byname(executor, ns ? "@nsemit" : "@emit") ||
+      fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
   }
   orator = executor;
-  do_emit(executor, args[0]);
+  do_emit(executor, args[0], flags);
 }
 
 /* ARGSUSED */
 FUNCTION(fun_remit)
 {
-  if (!command_check_byname(executor, "@remit") || fun->flags & FN_NOSIDEFX) {
+  int ns = string_prefix(called_as, "NS");
+  int flags = ns ? PEMIT_SPOOF : 0;
+  if (!command_check_byname(executor, ns ? "@nsremit" : "@remit") ||
+      fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
   }
   orator = executor;
-  do_remit(executor, args[0], args[1], 0);
+  do_remit(executor, args[0], args[1], flags);
 }
 
 /* ARGSUSED */
 FUNCTION(fun_lemit)
 {
-  if (!command_check_byname(executor, "@lemit") || fun->flags & FN_NOSIDEFX) {
+  int ns = string_prefix(called_as, "NS");
+  int flags = ns ? PEMIT_SPOOF : 0;
+  if (!command_check_byname(executor, ns ? "@nslemit" : "@lemit") ||
+      fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
   }
   orator = executor;
-  do_lemit(executor, args[0], 0);
+  do_lemit(executor, args[0], flags);
 }
 
 /* ARGSUSED */
 FUNCTION(fun_zemit)
 {
-  if (!command_check_byname(executor, "@zemit") || fun->flags & FN_NOSIDEFX) {
+  int ns = string_prefix(called_as, "NS");
+  int flags = ns ? PEMIT_SPOOF : 0;
+  if (!command_check_byname(executor, ns ? "@nszemit" : "@zemit") ||
+      fun->flags & FN_NOSIDEFX) {
     safe_str(T(e_perm), buff, bp);
     return;
   }
   orator = executor;
-  do_zemit(executor, args[0], args[1]);
+  do_zemit(executor, args[0], args[1], flags);
 }
 
-#ifdef CHAT_SYSTEM
-/* ARGSUSED */
-FUNCTION(fun_cemit)
-{
-  int noisy = 0;
-
-  if (!command_check_byname(executor, "@cemit") || fun->flags & FN_NOSIDEFX) {
-    safe_str(T(e_perm), buff, bp);
-    return;
-  }
-  if (nargs == 3 && parse_boolean(args[2]))
-    noisy = 1;
-  orator = executor;
-  do_cemit(executor, args[0], args[1], noisy);
-}
-#endif
-
 
 extern signed char qreg_indexes[UCHAR_MAX + 1];
 /* ARGSUSED */
@@ -154,8 +159,8 @@ FUNCTION(fun_setq)
 
   if (*args[0] && (*(args[0] + 1) == '\0') &&
       ((qindex = qreg_indexes[(unsigned char) args[0][0]]) != -1)
-      && renv[qindex]) {
-    strcpy(renv[qindex], args[1]);
+      && global_eval_context.renv[qindex]) {
+    strcpy(global_eval_context.renv[qindex], args[1]);
     if (!strcmp(called_as, "SETR"))
       safe_strl(args[1], arglens[1], buff, bp);
   } else
@@ -170,8 +175,8 @@ FUNCTION(fun_r)
 
   if (*args[0] && (*(args[0] + 1) == '\0') &&
       ((qindex = qreg_indexes[(unsigned char) args[0][0]]) != -1)
-      && renv[qindex])
-    safe_str(renv[qindex], buff, bp);
+      && global_eval_context.renv[qindex])
+    safe_str(global_eval_context.renv[qindex], buff, bp);
   else
     safe_str(T("#-1 REGISTER OUT OF RANGE"), buff, bp);
 }
@@ -188,13 +193,29 @@ FUNCTION(fun_rand)
    * Uses Sh'dow's random number generator, found in utils.c.  Better
    * distribution than original, w/ minimal speed losses.
    */
-  int i;
+  int low, high;
   if (!is_integer(args[0])) {
     safe_str(T(e_int), buff, bp);
     return;
   }
-  i = get_random_long(0, parse_integer(args[0]) - 1);
-  safe_integer(i, buff, bp);
+  if (nargs == 1) {
+    low = 0;
+    high = parse_integer(args[0]) - 1;
+  } else {
+    if (!is_integer(args[1])) {
+      safe_str(T(e_ints), buff, bp);
+      return;
+    }
+    low = parse_integer(args[0]);
+    high = parse_integer(args[1]);
+  }
+
+  if (low > high) {
+    safe_str(T(e_range), buff, bp);
+    return;
+  }
+
+  safe_integer(get_random_long(low, high), buff, bp);
 }
 
 /* ARGSUSED */
@@ -306,6 +327,66 @@ FUNCTION(fun_switch)
   }
 }
 
+FUNCTION(fun_reswitch)
+{
+  /* this works a bit like the @switch/regexp command */
+
+  int j, per;
+  char mstr[BUFFER_LEN], pstr[BUFFER_LEN], *dp;
+  char const *sp;
+  char *tbuf1;
+  int first = 1, found = 0, cs = 1;
+
+  if (strstr(called_as, "ALL"))
+    first = 0;
+
+  if (strcmp(called_as, "RESWITCHI") == 0
+      || strcmp(called_as, "RESWITCHALLI") == 0)
+    cs = 0;
+
+  dp = mstr;
+  sp = args[0];
+  process_expression(mstr, &dp, &sp, executor, caller, enactor,
+		     PE_DEFAULT, PT_DEFAULT, pe_info);
+  *dp = '\0';
+
+  /* try matching, return match immediately when found */
+
+  for (j = 1; j < (nargs - 1); j += 2) {
+    dp = pstr;
+    sp = args[j];
+    process_expression(pstr, &dp, &sp, executor, caller, enactor,
+		       PE_DEFAULT, PT_DEFAULT, pe_info);
+    *dp = '\0';
+
+    if (quick_regexp_match(pstr, mstr, cs)) {
+      /* If there's a #$ in a switch's action-part, replace it with
+       * the value of the conditional (mstr) before evaluating it.
+       */
+      tbuf1 = replace_string("#$", mstr, args[j + 1]);
+
+      sp = tbuf1;
+
+      per = process_expression(buff, bp, &sp,
+			       executor, caller, enactor,
+			       PE_DEFAULT, PT_DEFAULT, pe_info);
+      mush_free((Malloc_t) tbuf1, "replace_string.buff");
+      found = 1;
+      if (per || first)
+	return;
+    }
+  }
+
+  if (!(nargs & 1) && !found) {
+    /* Default case */
+    tbuf1 = replace_string("#$", mstr, args[nargs - 1]);
+    sp = tbuf1;
+    process_expression(buff, bp, &sp, executor, caller, enactor,
+		       PE_DEFAULT, PT_DEFAULT, pe_info);
+    mush_free((Malloc_t) tbuf1, "replace_string.buff");
+  }
+}
+
 /* ARGSUSED */
 FUNCTION(fun_if)
 {
@@ -337,35 +418,26 @@ FUNCTION(fun_mudname)
 /* ARGSUSED */
 FUNCTION(fun_version)
 {
-  safe_str(VERSION, buff, bp);
+  safe_format(buff, bp, "PennMUSH version %s patchlevel %s %s",
+	      VERSION, PATCHLEVEL, PATCHDATE);
 }
 
 /* ARGSUSED */
 FUNCTION(fun_starttime)
 {
-  char *s;
-  s = (char *) ctime(&first_start_time);
-  s[strlen(s) - 1] = '\0';
-  if (s[8] == ' ')
-    s[8] = '0';
-  safe_str(s, buff, bp);
+  safe_str(show_time(globals.first_start_time, 0), buff, bp);
 }
 
 /* ARGSUSED */
 FUNCTION(fun_restarttime)
 {
-  char *s;
-  s = (char *) ctime(&start_time);
-  s[strlen(s) - 1] = '\0';
-  if (s[8] == ' ')
-    s[8] = '0';
-  safe_str(s, buff, bp);
+  safe_str(show_time(globals.start_time, 0), buff, bp);
 }
 
 /* ARGSUSED */
 FUNCTION(fun_restarts)
 {
-  safe_integer(reboot_count, buff, bp);
+  safe_integer(globals.reboot_count, buff, bp);
 }
 
 /* Data for soundex functions */
@@ -377,8 +449,7 @@ static char soundex_val[26] = {
 
 /* The actual soundex routine */
 static char *
-soundex(str)
-    char *str;
+soundex(char *str)
 {
   static char tbuf1[BUFFER_LEN];
   char *p, *q;
@@ -457,8 +528,9 @@ FUNCTION(fun_soundlike)
    * I deem the modularity to be more important. So there.
    */
   char tbuf1[5];
-  if (!*args[0] || !*args[1] || !isalpha(*args[0]) || !isalpha(*args[1]) ||
-      strchr(args[0], ' ') || strchr(args[1], ' ')) {
+  if (!*args[0] || !*args[1] || !isalpha((unsigned char) *args[0])
+      || !isalpha((unsigned char) *args[1]) || strchr(args[0], ' ')
+      || strchr(args[1], ' ')) {
     safe_str(T("#-1 FUNCTION (SOUNDLIKE) REQUIRES TWO ONE-WORD ARGUMENTS"),
 	     buff, bp);
     return;
@@ -509,3 +581,99 @@ FUNCTION(fun_list)
     safe_str("#-1", buff, bp);
   return;
 }
+
+/* ARGSUSED */
+FUNCTION(fun_scan)
+{
+  dbref thing;
+  char save_ccom[BUFFER_LEN];
+  char *cmdptr;
+
+  if (nargs == 1) {
+    thing = executor;
+    cmdptr = args[0];
+  } else {
+    thing = match_thing(executor, args[0]);
+    if (!GoodObject(thing)) {
+      safe_str(T(e_notvis), buff, bp);
+      return;
+    }
+    if (!See_All(executor) && !controls(executor, thing)) {
+      notify(executor, T("Permission denied."));
+      safe_str("#-1", buff, bp);
+      return;
+    }
+    cmdptr = args[1];
+  }
+  strcpy(save_ccom, global_eval_context.ccom);
+  strncpy(global_eval_context.ccom, cmdptr, BUFFER_LEN);
+  global_eval_context.ccom[BUFFER_LEN - 1] = '\0';
+  safe_str(scan_list(thing, cmdptr), buff, bp);
+  strcpy(global_eval_context.ccom, save_ccom);
+}
+
+
+enum whichof_t { DO_FIRSTOF, DO_ALLOF };
+static void
+do_whichof(char *args[], int nargs, enum whichof_t flag, char *buff, char **bp,
+	   dbref executor, dbref caller, dbref enactor, PE_Info * pe_info)
+{
+  int j;
+  char tbuf[BUFFER_LEN], *tp;
+  char const *sp;
+  char sep = ' ';
+  int first = 1;
+
+  tbuf[0] = '\0';
+
+  if (flag == DO_ALLOF) {
+    /* The last arg is a delimiter. Parse it in place. */
+    char insep[BUFFER_LEN];
+    char *isep = insep;
+    const char *arglast = args[nargs - 1];
+    process_expression(insep, &isep, &arglast, executor, caller, enactor,
+		       PE_DEFAULT, PT_DEFAULT, pe_info);
+    *isep = '\0';
+    strcpy(args[nargs - 1], insep);
+
+    if (!delim_check(buff, bp, nargs, args, nargs, &sep))
+      return;
+    nargs--;
+  }
+
+  for (j = 0; j < nargs; j++) {
+    tp = tbuf;
+    sp = args[j];
+    process_expression(tbuf, &tp, &sp, executor, caller, enactor,
+		       PE_DEFAULT, PT_DEFAULT, pe_info);
+    *tp = '\0';
+    if (parse_boolean(tbuf)) {
+      if (!first) {
+	safe_chr(sep, buff, bp);
+      } else
+	first = 0;
+
+      safe_str(tbuf, buff, bp);
+
+      if (flag == DO_FIRSTOF)
+	return;
+    }
+  }
+  if (flag == DO_FIRSTOF)
+    safe_str(tbuf, buff, bp);
+}
+
+/* ARGSUSED */
+FUNCTION(fun_firstof)
+{
+  do_whichof(args, nargs, DO_FIRSTOF, buff, bp, executor, caller, enactor,
+	     pe_info);
+}
+
+
+/* ARGSUSED */
+FUNCTION(fun_allof)
+{
+  do_whichof(args, nargs, DO_ALLOF, buff, bp, executor, caller, enactor,
+	     pe_info);
+}
