@@ -1,3 +1,10 @@
+/**
+ * \file funufun.c
+ *
+ * \brief Evaluation and user-function functions for mushcode.
+ * 
+ *
+ */
 
 #include "copyrite.h"
 
@@ -59,64 +66,73 @@ FUNCTION(fun_objeval)
 		     PT_DEFAULT, pe_info);
   *s = '\0';
 
-#ifdef FUNCTION_SIDE_EFFECTS
-  /* The security hole created by function side effects is too great
-   * to allow a see_all player to evaluate functions from someone else's
-   * standpoint. We require control.
-   */
-  if (((obj = match_thing(executor, name)) == NOTHING)
-      || !controls(executor, obj))
-    obj = executor;
-#else
-  /* In order to evaluate from something else's viewpoint, you
-   * must control it, or be able to see_all.
-   */
-  if (((obj = match_thing(executor, name)) == NOTHING)
-      || (!controls(executor, obj) && !See_All(executor)))
-    obj = executor;
-#endif
+  if (FUNCTION_SIDE_EFFECTS) {
+    /* The security hole created by function side effects is too great
+     * to allow a see_all player to evaluate functions from someone else's
+     * standpoint. We require control.
+     */
+    if (((obj = match_thing(executor, name)) == NOTHING)
+	|| !controls(executor, obj))
+      obj = executor;
+  } else {
+    /* In order to evaluate from something else's viewpoint, you
+     * must control it, or be able to see_all.
+     */
+    if (((obj = match_thing(executor, name)) == NOTHING)
+	|| (!controls(executor, obj) && !See_All(executor)))
+      obj = executor;
+  }
 
   p = args[1];
   process_expression(buff, bp, &p, obj, executor, enactor, PE_DEFAULT,
 		     PT_DEFAULT, pe_info);
 }
 
+/** Helper function for ufun and family.
+ * \param buff string to store result of evaluation.
+ * \param bp pointer into end of buff.
+ * \param obj object on which the ufun is stored.
+ * \param attrib pointer to attribute on which the ufun is stored.
+ * \param nargs number of arguments passed to the ufun.
+ * \param args array of arguments passed to the ufun.
+ * \param executor executor.
+ * \param caller caller (unused).
+ * \param enactor enactor.
+ * \param pe_info pointer to structure for process_expression data.
+ */
 void
-do_userfn(buff, bp, obj, attrib, nargs, args, executor, caller, enactor,
-	  pe_info)
-    char *buff, **bp;
-    dbref obj;
-    ATTR *attrib;
-    int nargs;
-    char **args;
-    dbref executor, caller __attribute__ ((__unused__)), enactor;
-    PE_Info *pe_info;
+do_userfn(char *buff, char **bp, dbref obj, ATTR *attrib, int nargs,
+	  char **args, dbref executor, dbref caller
+	  __attribute__ ((__unused__)), dbref enactor, PE_Info * pe_info)
 {
   int j;
   char *tptr[10];
   char *tbuf;
   char const *tp;
+  int pe_flags = PE_DEFAULT;
 
   /* save our stack */
   for (j = 0; j < 10; j++)
-    tptr[j] = wenv[j];
+    tptr[j] = global_eval_context.wenv[j];
 
   /* copy the appropriate args into the stack */
   if (nargs > 10)
     nargs = 10;			/* maximum ten args */
   for (j = 0; j < nargs; j++)
-    wenv[j] = args[j];
+    global_eval_context.wenv[j] = args[j];
   for (; j < 10; j++)
-    wenv[j] = NULL;
+    global_eval_context.wenv[j] = NULL;
 
-  tp = tbuf = safe_uncompress(attrib->value);
-  process_expression(buff, bp, &tp, obj, executor, enactor, PE_DEFAULT,
+  tp = tbuf = safe_atr_value(attrib);
+  if (attrib->flags & AF_DEBUG)
+    pe_flags |= PE_DEBUG;
+  process_expression(buff, bp, &tp, obj, executor, enactor, pe_flags,
 		     PT_DEFAULT, pe_info);
   free(tbuf);
 
   /* restore the stack */
   for (j = 0; j < 10; j++)
-    wenv[j] = tptr[j];
+    global_eval_context.wenv[j] = tptr[j];
 }
 
 /* ARGSUSED */
@@ -132,7 +148,7 @@ FUNCTION(fun_ufun)
     return;
   }
   if (attrib && Can_Read_Attr(executor, obj, attrib)) {
-    if (!CanEval(executor, obj)) {
+    if (!CanEvalAttr(executor, obj, attrib)) {
       safe_str(T(e_perm), buff, bp);
       return;
     }
@@ -163,7 +179,7 @@ FUNCTION(fun_ulocal)
     return;
   }
   if (attrib && Can_Read_Attr(executor, obj, attrib)) {
-    if (!CanEval(executor, obj)) {
+    if (!CanEvalAttr(executor, obj, attrib)) {
       safe_str(T(e_perm), buff, bp);
       return;
     }
@@ -202,7 +218,7 @@ FUNCTION(fun_uldefault)
 		     PE_DEFAULT, PT_DEFAULT, pe_info);
   *dp = '\0';
   parse_attrib(executor, mstr, &thing, &attrib);
-  if (GoodObject(thing) && attrib && CanEval(executor, thing)
+  if (GoodObject(thing) && attrib && CanEvalAttr(executor, thing, attrib)
       && Can_Read_Attr(executor, thing, attrib)) {
     /* Ok, we've got it */
     /* We must now evaluate all the arguments from args[2] on and
@@ -263,7 +279,7 @@ FUNCTION(fun_zfun)
   /* find the user function attribute */
   attrib = atr_get(zone, upcasestr(args[0]));
   if (attrib && Can_Read_Attr(executor, zone, attrib)) {
-    if (!CanEval(executor, zone)) {
+    if (!CanEvalAttr(executor, zone, attrib)) {
       safe_str(T(e_perm), buff, bp);
       return;
     }
