@@ -1,4 +1,10 @@
-/* move.c */
+/**
+ * \file move.c
+ *
+ * \brief Movement commands for PennMUSH.
+ *
+ *
+ */
 
 #include "copyrite.h"
 #include "config.h"
@@ -7,10 +13,10 @@
 #include <string.h>
 
 #include "conf.h"
+#include "externs.h"
 #include "mushdb.h"
 #include "attrib.h"
 #include "match.h"
-#include "externs.h"
 #include "flags.h"
 #include "lock.h"
 #include "dbdefs.h"
@@ -21,8 +27,6 @@
 #include "game.h"
 #include "confmagic.h"
 
-extern char ccom[BUFFER_LEN];
-
 void moveit(dbref what, dbref where, int nomovemsgs);
 static void send_contents(dbref loc, dbref dest);
 static void maybe_dropto(dbref loc, dbref dropto);
@@ -38,20 +42,24 @@ static char *list_following(dbref player);
 static int is_following(dbref follower, dbref leader);
 static void follower_command(dbref leader, dbref loc, const char *com);
 
+/** A convenience wrapper for enter_room().
+ * \param what object to move.
+ * \param where location to move it to.
+ */
 void
-moveto(what, where)
-    dbref what;
-    dbref where;
+moveto(dbref what, dbref where)
 {
   enter_room(what, where, 0);
 }
 
 
+/** Send an object somewhere.
+ * \param what object to move.
+ * \param where location to move it to.
+ * \param nomovemsgs if 1, don't show movement messages.
+ */
 void
-moveit(what, where, nomovemsgs)
-    dbref what;
-    dbref where;
-    int nomovemsgs;
+moveit(dbref what, dbref where, int nomovemsgs)
 {
   dbref loc, old;
   dbref absloc, absold;
@@ -87,26 +95,28 @@ moveit(what, where, nomovemsgs)
     if ((where != NOTHING) && (old != where)) {
       did_it(what, what, NULL, NULL, "OXMOVE", NULL, NULL, old);
       if (Hearer(what)) {
-	did_it(what, old, "LEAVE", NULL, "OLEAVE", T("has left."), "ALEAVE",
-	       old);
+	did_it_interact(what, old, "LEAVE", NULL, "OLEAVE", T("has left."),
+			"ALEAVE", old, NA_INTER_PRESENCE);
 	/* If the player is leaving a zone, do zone messages */
 	/* The tricky bit here is that we only care about the zone of
 	 * the outermost contents */
 	if (GoodObject(absold) && GoodObject(Zone(absold))
 	    && (Zone(absloc) != Zone(absold)))
-	  did_it(what, Zone(absold), "ZLEAVE", NULL, "OZLEAVE", NULL, "AZLEAVE",
-		 old);
+	  did_it_interact(what, Zone(absold), "ZLEAVE", NULL, "OZLEAVE", NULL,
+			  "AZLEAVE", old, NA_INTER_SEE);
 	if (GoodObject(old) && !IsRoom(old))
-	  did_it(what, old, NULL, NULL, "OXLEAVE", NULL, NULL, where);
+	  did_it_interact(what, old, NULL, NULL, "OXLEAVE", NULL, NULL, where,
+			  NA_INTER_SEE);
 	if (!IsRoom(where))
-	  did_it(what, where, NULL, NULL, "OXENTER", NULL, NULL, old);
+	  did_it_interact(what, where, NULL, NULL, "OXENTER", NULL, NULL, old,
+			  NA_INTER_SEE);
 	/* If the player is entering a new zone, do zone messages */
 	if (!GoodObject(absold)
 	    || (GoodObject(Zone(absloc)) && (Zone(absloc) != Zone(absold))))
-	  did_it(what, Zone(absloc), "ZENTER", NULL, "OZENTER", NULL, "AZENTER",
-		 where);
-	did_it(what, where, "ENTER", NULL, "OENTER", T("has arrived."),
-	       "AENTER", where);
+	  did_it_interact(what, Zone(absloc), "ZENTER", NULL, "OZENTER", NULL,
+			  "AZENTER", where, NA_INTER_SEE);
+	did_it_interact(what, where, "ENTER", NULL, "OENTER", T("has arrived."),
+			"AENTER", where, NA_INTER_PRESENCE);
       } else {
 	/* non-listeners only trigger the actions not the messages */
 	did_it(what, old, NULL, NULL, NULL, NULL, "ALEAVE", old);
@@ -120,15 +130,15 @@ moveit(what, where, nomovemsgs)
       }
     }
   if (!nomovemsgs)
-    did_it(what, what, "MOVE", NULL, "OMOVE", NULL, "AMOVE", where);
+    did_it_interact(what, what, "MOVE", NULL, "OMOVE", NULL, "AMOVE", where,
+		    NA_INTER_SEE);
 }
 
+/** A dropper is an object that can hear and has a connected owner */
 #define Dropper(thing) (Hearer(thing) && Connected(Owner(thing)))
 
 static void
-send_contents(loc, dest)
-    dbref loc;
-    dbref dest;
+send_contents(dbref loc, dbref dest)
 {
   dbref first;
   dbref rest;
@@ -154,9 +164,7 @@ send_contents(loc, dest)
 }
 
 static void
-maybe_dropto(loc, dropto)
-    dbref loc;
-    dbref dropto;
+maybe_dropto(dbref loc, dbref dropto)
 {
   dbref thing;
   if (loc == dropto)
@@ -173,11 +181,13 @@ maybe_dropto(loc, dropto)
   send_contents(loc, dropto);
 }
 
+/** Enter a container.
+ * \param player object entering the container.
+ * \param loc container to enter.
+ * \param nomovemsgs if 1, don't give movement messages.
+ */
 void
-enter_room(player, loc, nomovemsgs)
-    dbref player;
-    dbref loc;
-    int nomovemsgs;
+enter_room(dbref player, dbref loc, int nomovemsgs)
 {
   dbref old;
   dbref dropto;
@@ -232,17 +242,18 @@ enter_room(player, loc, nomovemsgs)
 
 
   /* autolook */
-  look_room(player, loc, 2);
+  look_room(player, loc, LOOK_AUTO);
   deep--;
 }
 
 
-/* teleports player to location while removing items they shouldnt take */
+/** Teleport player to location while removing items they shouldn't take.
+ * \param player player to teleport.
+ * \param dest location to teleport player to.
+ * \param nomovemsgs if 1, don't show movement messages
+ */
 void
-safe_tel(player, dest, nomovemsgs)
-    dbref player;
-    dbref dest;
-    int nomovemsgs;
+safe_tel(dbref player, dbref dest, int nomovemsgs)
 {
   dbref first;
   dbref rest;
@@ -280,14 +291,15 @@ safe_tel(player, dest, nomovemsgs)
   enter_room(player, dest, nomovemsgs);
 }
 
-/* This checks to see if there's a go-able direction. It doesn't
+/** Can a player go in a given direction?
+ * This checks to see if there's a go-able direction. It doesn't
  * check whether the GOTO command is restricted. That should be
  * done by the command parser.
+ * \param player dbref of mover.
+ * \param direction name of direction to move.
  */
 int
-can_move(player, direction)
-    dbref player;
-    const char *direction;
+can_move(dbref player, const char *direction)
 {
   if (!strcasecmp(direction, "home") && !Fixed(player))
     return 1;
@@ -298,9 +310,7 @@ can_move(player, direction)
 }
 
 static dbref
-find_var_dest(player, exit_obj)
-    dbref player;
-    dbref exit_obj;
+find_var_dest(dbref player, dbref exit_obj)
 {
   /* This is used to evaluate the u-function DESTINATION on an exit with
    * a VARIABLE (ambiguous) link.
@@ -318,7 +328,7 @@ find_var_dest(player, exit_obj)
     a = atr_get(exit_obj, "EXITTO");
   if (!a)
     return NOTHING;
-  abuf = safe_uncompress(a->value);
+  abuf = safe_atr_value(a);
   if (!abuf)
     return NOTHING;
   if (!*abuf) {
@@ -330,17 +340,19 @@ find_var_dest(player, exit_obj)
   process_expression(buff, &bp, &ap, exit_obj, player, player,
 		     PE_DEFAULT, PT_DEFAULT, NULL);
   *bp = '\0';
-  dest_room = parse_dbref(buff);
+  dest_room = parse_objid(buff);
   free((Malloc_t) abuf);
   return (dest_room);
 }
 
 
+/** The move command.
+ * \param player the enactor.
+ * \param direction name of direction to move.
+ * \param type type of motion to check (global, zone, neither).
+ */
 void
-do_move(player, direction, type)
-    dbref player;
-    const char *direction;
-    int type;			/* type 0 is normal, type 1 is global, type 2 is zone */
+do_move(dbref player, const char *direction, enum move_type type)
 {
   dbref exit_m, loc, var_dest;
   if (!strcasecmp(direction, "home")) {
@@ -355,7 +367,7 @@ do_move(player, direction, type)
     if ((loc = Location(player)) != NOTHING && !Dark(player) && !Dark(loc)) {
       /* tell everybody else */
       notify_except(Contents(loc), player,
-		    tprintf(T("%s goes home."), Name(player)));
+		    tprintf(T("%s goes home."), Name(player)), NA_INTER_SEE);
     }
     /* give the player the messages */
     notify(player, T("There's no place like home..."));
@@ -364,11 +376,11 @@ do_move(player, direction, type)
     safe_tel(player, HOME, 0);
   } else {
     /* find the exit */
-    if (DO_GLOBALS && (type == 1))
+    if (type == MOVE_GLOBAL)
       exit_m =
 	match_result(player, direction, TYPE_EXIT,
 		     MAT_ENGLISH | MAT_EXIT | MAT_GLOBAL | MAT_CHECK_KEYS);
-    else if (DO_GLOBALS && (type == 2))
+    else if (type == MOVE_ZONE)
       exit_m =
 	match_result(player, direction, TYPE_EXIT,
 		     MAT_ENGLISH | MAT_EXIT | MAT_REMOTES | MAT_CHECK_KEYS);
@@ -387,6 +399,11 @@ do_move(player, direction, type)
     default:
       /* we got one */
       /* check to see if we're allowed to pass */
+      if (!eval_lock(player, Location(player), Leave_Lock)) {
+	notify(player, T("You can't go that way."));
+	return;
+      }
+
       if (could_doit(player, exit_m)) {
 	switch (Location(exit_m)) {
 	case HOME:
@@ -425,7 +442,7 @@ do_move(player, direction, type)
 	  enter_room(player, var_dest, 0);
 	  /* Move the followers if the leader is elsewhere */
 	  if (Location(player) != loc)
-	    follower_command(player, loc, tprintf("%s #%d", "move", exit_m));
+	    follower_command(player, loc, tprintf("%s #%d", "goto", exit_m));
 	  break;
 	case TYPE_PLAYER:
 	case TYPE_THING:
@@ -441,24 +458,29 @@ do_move(player, direction, type)
 	  safe_tel(player, var_dest, 0);
 	  /* Move the followers if the leader is elsewhere */
 	  if (Location(player) != loc)
-	    follower_command(player, loc, tprintf("%s #%d", "move", exit_m));
+	    follower_command(player, loc, tprintf("%s #%d", "goto", exit_m));
 	  break;
 	case TYPE_EXIT:
 	  notify(player, T("This feature coming soon."));
 	  break;
 	}
       } else
-	did_it(player, exit_m, "FAILURE", T("You can't go that way."),
-	       "OFAILURE", NULL, "AFAILURE", NOTHING);
+	fail_lock(player, exit_m, Basic_Lock, T("You can't go that way."),
+		  NOTHING);
       break;
     }
   }
 }
 
+/** Move an exit to the first position in the room's exit list.
+ * \verbatim
+ * This implements @firstexit.
+ * \endverbatim
+ * \param player the enactor.
+ * \param what name of exit to promote.
+ */
 void
-do_firstexit(player, what)
-    dbref player;
-    const char *what;
+do_firstexit(dbref player, const char *what)
 {
   dbref thing;
   dbref loc;
@@ -478,18 +500,19 @@ do_firstexit(player, what)
 }
 
 
+/** The get command.
+ * \param player the enactor.
+ * \param what name of object to get.
+ */
 void
-do_get(player, what)
-    dbref player;
-    const char *what;
+do_get(dbref player, const char *what)
 {
   dbref loc = Location(player);
   dbref thing;
   char tbuf1[BUFFER_LEN], tbuf2[BUFFER_LEN], *tp;
   long match_flags =
     MAT_NEIGHBOR | MAT_ABSOLUTE | MAT_CHECK_KEYS | MAT_NEAR | MAT_ENGLISH;
-  char *myenv[10];
-  int i;
+
   if (!IsRoom(loc) && !EnterOk(loc) && !controls(player, loc)) {
     notify(player, T("Permission denied."));
     return;
@@ -498,11 +521,24 @@ do_get(player, what)
     match_flags |= MAT_CONTROL;
   if (match_result(player, what, TYPE_THING, match_flags) == NOTHING) {
     if (POSSESSIVE_GET) {
+      dbref box;
+      const char *boxname = what;
       /* take care of possessive get (stealing) */
-      thing = parse_match_possessive(player, what);
-      if (!GoodObject(thing)) {
+      box = parse_match_possessor(player, &what);
+      if (box == NOTHING) {
 	notify(player, T("I don't see that here."));
 	return;
+      } else if (box == AMBIGUOUS) {
+	notify_format(player, T("I can't tell which %s."), boxname);
+	return;
+      }
+      thing = match_result(box, what, NOTYPE, MAT_POSSESSION);
+      if (thing == NOTHING) {
+	notify(player, T("I don't see that here."));
+	return;
+      } else if (thing == AMBIGUOUS) {
+	notify_format(player, T("I can't tell which %s."), what);
+	return;
       }
       /* to steal something, you have to be able to get it, and the
        * object must be ENTER_OK and not locked against you.
@@ -528,16 +564,11 @@ do_get(player, what)
 	moveto(thing, player);
 	did_it(player, thing, "SUCCESS", tbuf1, "OSUCCESS", tbuf2, "ASUCCESS",
 	       NOTHING);
-	myenv[0] = (char *) mush_malloc(BUFFER_LEN, "dbref");
-	sprintf(myenv[0], "#%d", thing);
-	for (i = 1; i < 10; i++)
-	  myenv[i] = NULL;
-	real_did_it(player, player, "RECEIVE", NULL, "ORECEIVE", NULL,
-		    "ARECEIVE", NOTHING, myenv);
-	mush_free(myenv[0], "dbref");
+	did_it_with(player, player, "RECEIVE", NULL, "ORECEIVE", NULL,
+		    "ARECEIVE", NOTHING, thing, NOTHING, NA_INTER_HEAR);
       } else
-	did_it(player, thing, "FAILURE", T("You can't take that from there."),
-	       "OFAILURE", NULL, "AFAILURE", NOTHING);
+	fail_lock(player, thing, Basic_Lock,
+		  T("You can't take that from there."), NOTHING);
     } else {
       notify(player, T("I don't see that here."));
     }
@@ -575,16 +606,11 @@ do_get(player, what)
 	  *tp = '\0';
 	  did_it(player, thing, "SUCCESS", tbuf1, "OSUCCESS", tbuf2,
 		 "ASUCCESS", NOTHING);
-	  myenv[0] = (char *) mush_malloc(BUFFER_LEN, "dbref");
-	  sprintf(myenv[0], "#%d", thing);
-	  for (i = 1; i < 10; i++)
-	    myenv[i] = NULL;
-	  real_did_it(player, player, "RECEIVE", NULL, "ORECEIVE", NULL,
-		      "ARECEIVE", NOTHING, myenv);
-	  mush_free(myenv[0], "dbref");
+	  did_it_with(player, player, "RECEIVE", NULL, "ORECEIVE", NULL,
+		      "ARECEIVE", NOTHING, thing, NOTHING, NA_INTER_HEAR);
 	} else
-	  did_it(player, thing, "FAILURE", T("You can't pick that up."),
-		 "OFAILURE", NULL, "AFAILURE", NOTHING);
+	  fail_lock(player, thing, Basic_Lock, T("You can't pick that up."),
+		    NOTHING);
 	break;
       case TYPE_EXIT:
 	notify(player, T("You can't pick up exits."));
@@ -598,10 +624,12 @@ do_get(player, what)
 }
 
 
+/** Drop an object.
+ * \param player the enactor.
+ * \param name name of object to drop.
+ */
 void
-do_drop(player, name)
-    dbref player;
-    const char *name;
+do_drop(dbref player, const char *name)
 {
   dbref loc;
   dbref thing;
@@ -655,11 +683,141 @@ do_drop(player, name)
   did_it(player, thing, "DROP", tbuf1, "ODROP", tbuf2, "ADROP", NOTHING);
 }
 
-
+/** The empty command.
+ * This command causes the player to attempt to move everything in
+ * the thing to the location of the thing.
+ * Thing must be in player's inventory or in player's location.
+ * For each item in thing, movement is allowed if one of these is true:
+ * (a) thing is inside player, and player is allowed to get thing's item
+ * (b) thing is next to player, player is allowed to get thing's item,
+ *     and player is allowed to drop item in player's location.
+ * We do not consider the cases of forcing the object to drop the items,
+ * teleporting the items out, or forcing the items to leave; 
+ * 'empty' implies that the items pass through the player's hands.
+ *
+ * There is a choice to be made here with regard to locks - do we 
+ * check locks on the thing (e.g. enter locks) and its location
+ * (e.g. drop locks) once or each time? If we choose once, we break
+ * locks that might make decisions based on the number of items there.
+ * If we choose multiple, we risk running side effects more than once.
+ * We choose multiple, as that's what would happen if the
+ * player did it manually.
+ * \param player the enactor.
+ * \param what the name of the object to empty.
+ */
 void
-do_enter(player, what)
-    dbref player;
-    const char *what;
+do_empty(dbref player, const char *what)
+{
+  dbref player_loc;
+  dbref thing, thing_loc;
+  dbref item;
+  int empty_ok;
+  int count = 0;
+  int next;
+
+  if ((player_loc = Location(player)) == NOTHING)
+    return;
+  thing =
+    noisy_match_result(player, what, TYPE_THING | TYPE_PLAYER,
+		       MAT_NEAR_THINGS | MAT_ENGLISH);
+  if (!GoodObject(thing))
+    return;
+  thing_loc = Location(thing);
+
+  /* Object to empty must be in player's inventory or location */
+  if ((thing_loc != player) && (thing_loc != player_loc)) {
+    notify(player, T("You can't empty that from here."));
+    return;
+  }
+  for (item = first_visible(player, Contents(thing)); GoodObject(item);
+       item = first_visible(player, next)) {
+    next = Next(item);
+    if (IsExit(item))
+      continue;			/* No dropping exits */
+    empty_ok = 0;
+    if (player == thing) {
+      /* empty me: You don't need to get what's in your inventory already */
+      if (eval_lock(player, item, Drop_Lock) &&
+	  (!IsRoom(thing_loc) || eval_lock(player, thing_loc, Drop_Lock)))
+	empty_ok = 1;
+    }
+    /* Check that player can get stuff from thing */
+    else if (controls(player, thing) ||
+	     (EnterOk(thing) && eval_lock(player, thing, Enter_Lock))) {
+      /* Check that player can get item */
+      if (!could_doit(player, item)) {
+	/* Send failure message if set, otherwise be quiet */
+	fail_lock(player, thing, Basic_Lock, NULL, NOTHING);
+	continue;
+      }
+      /* Now check for dropping in the destination */
+      /* Thing is in player's inventory - sufficient */
+      if (thing_loc == player)
+	empty_ok = 1;
+      /* Thing is in player's location - player must also be able to drop */
+      else if (eval_lock(player, item, Drop_Lock) &&
+	       (!IsRoom(thing_loc) || eval_lock(player, thing_loc, Drop_Lock)))
+	empty_ok = 1;
+    }
+    /* Now do the work, if we should. That includes triggering messages */
+    if (empty_ok) {
+      char tbuf1[BUFFER_LEN], tbuf2[BUFFER_LEN], *tp;
+
+      count++;
+      /* Get messages */
+      if (thing != player) {
+	notify_format(thing, T("%s was taken from you."), Name(item));
+	notify_format(item, T("%s took you."), Name(player));
+	tp = tbuf1;
+	safe_format(tbuf1, &tp, T("You take %s from %s."), Name(item),
+		    Name(thing));
+	*tp = '\0';
+	tp = tbuf2;
+	safe_format(tbuf2, &tp, T("takes %s from %s."), Name(item),
+		    Name(thing));
+	*tp = '\0';
+	moveto(item, player);
+	did_it(player, item, "SUCCESS", tbuf1, "OSUCCESS", tbuf2, "ASUCCESS",
+	       NOTHING);
+	did_it_with(player, player, "RECEIVE", NULL, "ORECEIVE", NULL,
+		    "ARECEIVE", NOTHING, item, NOTHING, NA_INTER_HEAR);
+      }
+      /* Drop messages */
+      if (thing_loc != player) {
+	if (Sticky(item) && !Fixed(item)) {
+	  safe_tel(thing, HOME, 0);
+	} else if ((Location(thing_loc) != NOTHING) && IsRoom(thing_loc)
+		   && !Sticky(thing_loc)
+		   && eval_lock(item, thing_loc, Dropto_Lock)) {
+	  /* location has immediate dropto */
+	  notify_format(item, T("%s drops you."), Name(player));
+	  moveto(item, Location(thing_loc));
+	} else {
+	  notify_format(item, T("%s drops you."), Name(player));
+	  moveto(item, thing_loc);
+	}
+	tp = tbuf1;
+	safe_format(tbuf1, &tp, T("You drop %s."), Name(item));
+	*tp = '\0';
+	tp = tbuf2;
+	safe_format(tbuf2, &tp, T("drops %s."), Name(item));
+	*tp = '\0';
+	did_it(player, item, "DROP", tbuf1, "ODROP", tbuf2, "ADROP", NOTHING);
+      }
+    }
+  }
+  notify_format(player, T("You remove %d object%s from %s."),
+		count, (count == 1) ? "" : "s", Name(thing));
+  return;
+}
+
+
+/** The enter command.
+ * \param player the enactor.
+ * \param what name of object to enter.
+ */
+void
+do_enter(dbref player, const char *what)
 {
   dbref thing;
   dbref loc;
@@ -682,11 +840,10 @@ do_enter(player, what)
     }
     /* the object must pass the lock. Also, the thing being entered */
     /* has to be controlled, or must be enter_ok */
-    if (!(((Flags(thing) & ENTER_OK) || controls(player, thing)) &&
+    if (!((EnterOk(thing) || controls(player, thing)) &&
 	  (eval_lock(player, thing, Enter_Lock))
 	)) {
-      did_it(player, thing, "EFAIL", T("Permission denied."), "OEFAIL",
-	     NULL, "AEFAIL", NOTHING);
+      fail_lock(player, thing, Enter_Lock, T("Permission denied."), NOTHING);
       return;
     }
     if (thing == player) {
@@ -702,9 +859,11 @@ do_enter(player, what)
   }
 }
 
+/** The leave command.
+ * \param player the enactor.
+ */
 void
-do_leave(player)
-    dbref player;
+do_leave(dbref player)
 {
   dbref loc;
   loc = Location(player);
@@ -712,8 +871,7 @@ do_leave(player)
       || NoLeave(loc)
       || !eval_lock(player, loc, Leave_Lock)
     ) {
-    did_it(player, loc, "LFAIL", T("You can't leave."), "OLFAIL",
-	   NULL, "ALFAIL", NOTHING);
+    fail_lock(player, loc, Leave_Lock, T("You can't leave."), NOTHING);
     return;
   }
   enter_room(player, Location(loc), 0);
@@ -721,64 +879,70 @@ do_leave(player)
     follower_command(player, loc, "leave");
 }
 
+/** Is direction a global exit?
+ * \param player looker.
+ * \param direction name of exit.
+ * \retval 1 direction is a global exit.
+ * \retval 0 direction is not a global exit.
+ */
 int
-global_exit(player, direction)
-    dbref player;
-    const char *direction;
+global_exit(dbref player, const char *direction)
 {
   return (GoodObject
 	  (match_result(player, direction, TYPE_EXIT, MAT_GLOBAL | MAT_EXIT)));
 }
 
+/** Is direction a remote exit?
+ * \param player looker.
+ * \param direction name of exit.
+ * \retval 1 direction is a remote exit.
+ * \retval 0 direction is not a remote exit.
+ */
 int
-remote_exit(player, direction)
-    dbref player;
-    const char *direction;
+remote_exit(dbref player, const char *direction)
 {
   return (GoodObject
 	  (match_result(player, direction, TYPE_EXIT, MAT_REMOTES | MAT_EXIT)));
 }
 
+/** Wrapper for exit movement.
+ * We check local exit, then zone exit, then global. If nothing is
+ * matched, treat it as local so player will get an error message.
+ * \param player the mover.
+ * \param command direction to move.
+ */
 void
-move_wrapper(player, command)
-    dbref player;
-    const char *command;
+move_wrapper(dbref player, const char *command)
 {
-  /* check local exit, then zone exit, then global. If nothing is
-   * matched, treat it as local so player will get an error message.
-   */
-
   if (!Mobile(player))
     return;
   if (!strcasecmp(command, "home") && Fixed(player)) {
     notify(player, T("You can't do that IC!"));
     return;
   }
-  if (DO_GLOBALS) {
-    if (can_move(player, command))
-      do_move(player, command, 0);
-    else
-      if ((Zone(Location(player)) != NOTHING) && remote_exit(player, command))
-      do_move(player, command, 2);
-    else if ((Location(player) != MASTER_ROOM)
-	     && global_exit(player, command))
-      do_move(player, command, 1);
-    else
-      do_move(player, command, 0);
-  } else {
-    do_move(player, command, 0);
-  }
+  if (can_move(player, command))
+    do_move(player, command, MOVE_NORMAL);
+  else if ((Zone(Location(player)) != NOTHING) && remote_exit(player, command))
+    do_move(player, command, MOVE_ZONE);
+  else if ((Location(player) != MASTER_ROOM)
+	   && global_exit(player, command))
+    do_move(player, command, MOVE_GLOBAL);
+  else
+    do_move(player, command, MOVE_NORMAL);
 }
 
 /* Routines for dealing with the follow commands */
 
-/* follow <arg> tries to start following
+/** The follow command.
+ * \verbatim
+ * follow <arg> tries to start following
  * follow alone lists who you're following
+ * \endverbatim
+ * \param player the enactor.
+ * \param arg name of object to follow.
  */
 void
-do_follow(player, arg)
-    dbref player;
-    const char *arg;
+do_follow(dbref player, const char *arg)
 {
   dbref leader;
   if (arg && *arg) {
@@ -811,8 +975,8 @@ do_follow(player, arg)
     }
     /* Ok, are we allowed to follow them? */
     if (!eval_lock(player, leader, Follow_Lock)) {
-      notify_format(player,
-		    T("You're not allowed to follow %s."), Name(leader));
+      fail_lock(player, leader, Follow_Lock,
+		T("You're not alllowed to follow."), Location(player));
       return;
     }
     /* Ok, looks good */
@@ -824,13 +988,16 @@ do_follow(player, arg)
   }
 }
 
-/* unfollow <arg> removes someone from your following list 
- * unfollow alone removes everyone from your following list. (1.7.4p3)
+/** The unfollow command.
+ * \verbatim
+ * unfollow <arg> removes someone from your following list 
+ * unfollow alone removes everyone from your following list.
+ * \endverbatim
+ * \param player the enactor.
+ * \param arg object to stop following.
  */
 void
-do_unfollow(player, arg)
-    dbref player;
-    const char *arg;
+do_unfollow(dbref player, const char *arg)
 {
   dbref leader;
   if (arg && *arg) {
@@ -859,13 +1026,16 @@ do_unfollow(player, arg)
 }
 
 
-/* dismiss <arg> removes someone from your followers list 
+/** The dismiss command.
+ * \verbatim
+ * dismiss <arg> removes someone from your followers list 
  * dismiss alone removes everyone from your followers list.
+ * \endverbatim
+ * \param player the enactor.
+ * \param arg name of object to dismiss.
  */
 void
-do_dismiss(player, arg)
-    dbref player;
-    const char *arg;
+do_dismiss(dbref player, const char *arg)
 {
   dbref follower;
   if (arg && *arg) {
@@ -889,13 +1059,16 @@ do_dismiss(player, arg)
   }
 }
 
-/* desert <arg> removes someone from your followers and following list 
+/** The desert command.
+ * \verbatim
+ * desert <arg> removes someone from your followers and following list 
  * desert alone removes everyone from both lists
+ * \endverbatim
+ * \param player the enactor.
+ * \param arg name of object to desert.
  */
 void
-do_desert(player, arg)
-    dbref player;
-    const char *arg;
+do_desert(dbref player, const char *arg)
 {
   dbref who;
   if (arg && *arg) {
@@ -925,9 +1098,7 @@ do_desert(player, arg)
 
 /* Add someone to a player's FOLLOWERS attribute */
 static void
-add_follower(leader, follower)
-    dbref leader;
-    dbref follower;
+add_follower(dbref leader, dbref follower)
 {
   ATTR *a;
   char tbuf1[BUFFER_LEN];
@@ -937,7 +1108,7 @@ add_follower(leader, follower)
     (void) atr_add(leader, "FOLLOWERS", unparse_dbref(follower), GOD, NOTHING);
   } else {
     bp = tbuf1;
-    safe_str(uncompress(AL_STR(a)), tbuf1, &bp);
+    safe_str(atr_value(a), tbuf1, &bp);
     safe_chr(' ', tbuf1, &bp);
     safe_dbref(follower, tbuf1, &bp);
     *bp = '\0';
@@ -947,9 +1118,7 @@ add_follower(leader, follower)
 
 /* Add someone to a player's FOLLOWING attribute */
 static void
-add_following(follower, leader)
-    dbref follower;
-    dbref leader;
+add_following(dbref follower, dbref leader)
 {
   ATTR *a;
   char tbuf1[BUFFER_LEN];
@@ -959,7 +1128,7 @@ add_following(follower, leader)
     (void) atr_add(follower, "FOLLOWING", unparse_dbref(leader), GOD, NOTHING);
   } else {
     bp = tbuf1;
-    safe_str(uncompress(AL_STR(a)), tbuf1, &bp);
+    safe_str(atr_value(a), tbuf1, &bp);
     safe_chr(' ', tbuf1, &bp);
     safe_dbref(leader, tbuf1, &bp);
     *bp = '\0';
@@ -968,10 +1137,7 @@ add_following(follower, leader)
 }
 
 static void
-add_follow(leader, follower, noisy)
-    dbref leader;
-    dbref follower;
-    int noisy;
+add_follow(dbref leader, dbref follower, int noisy)
 {
   char msg[BUFFER_LEN];
   add_follower(leader, follower);
@@ -987,9 +1153,7 @@ add_follow(leader, follower, noisy)
 
 /* Delete someone from a player's FOLLOWERS attribute */
 static void
-del_follower(leader, follower)
-    dbref leader;
-    dbref follower;
+del_follower(dbref leader, dbref follower)
 {
   ATTR *a;
   char tbuf1[BUFFER_LEN];
@@ -999,16 +1163,14 @@ del_follower(leader, follower)
     return;			/* No followers, so no deletion */
   /* Let's take it apart and put it back together w/o follower */
   strcpy(flwr, unparse_dbref(follower));
-  strcpy(tbuf1, uncompress(AL_STR(a)));
+  strcpy(tbuf1, atr_value(a));
   (void) atr_add(leader, "FOLLOWERS",
 		 remove_word(tbuf1, flwr, ' '), GOD, NOTHING);
 }
 
 /* Delete someone from a player's FOLLOWING attribute */
 static void
-del_following(follower, leader)
-    dbref follower;
-    dbref leader;
+del_following(dbref follower, dbref leader)
 {
   ATTR *a;
   char tbuf1[BUFFER_LEN];
@@ -1018,16 +1180,13 @@ del_following(follower, leader)
     return;			/* Not following, so no deletion */
   /* Let's take it apart and put it back together w/o leader */
   strcpy(ldr, unparse_dbref(leader));
-  strcpy(tbuf1, uncompress(AL_STR(a)));
+  strcpy(tbuf1, atr_value(a));
   (void) atr_add(follower, "FOLLOWING",
 		 remove_word(tbuf1, ldr, ' '), GOD, NOTHING);
 }
 
 static void
-del_follow(leader, follower, noisy)
-    dbref leader;
-    dbref follower;
-    int noisy;
+del_follow(dbref leader, dbref follower, int noisy)
 {
   char msg[BUFFER_LEN];
   del_follower(leader, follower);
@@ -1042,8 +1201,7 @@ del_follow(leader, follower, noisy)
 
 /* Return a list of names of players who are my followers, comma-separated */
 static char *
-list_followers(player)
-    dbref player;
+list_followers(dbref player)
 {
   ATTR *a;
   char tbuf1[BUFFER_LEN];
@@ -1055,7 +1213,7 @@ list_followers(player)
   a = atr_get_noparent(player, "FOLLOWERS");
   if (!a)
     return (char *) "";
-  strcpy(tbuf1, uncompress(AL_STR(a)));
+  strcpy(tbuf1, atr_value(a));
   bp = buff;
   s = trim_space_sep(tbuf1, ' ');
   while (s) {
@@ -1074,8 +1232,7 @@ list_followers(player)
 
 /* Return a list of names of players who I'm following, comma-separated */
 static char *
-list_following(player)
-    dbref player;
+list_following(dbref player)
 {
   ATTR *a;
   char tbuf1[BUFFER_LEN];
@@ -1087,7 +1244,7 @@ list_following(player)
   a = atr_get_noparent(player, "FOLLOWING");
   if (!a)
     return (char *) "";
-  strcpy(tbuf1, uncompress(AL_STR(a)));
+  strcpy(tbuf1, atr_value(a));
   bp = buff;
   s = trim_space_sep(tbuf1, ' ');
   while (s) {
@@ -1106,9 +1263,7 @@ list_following(player)
 
 /* Is follower following leader? */
 static int
-is_following(follower, leader)
-    dbref follower;
-    dbref leader;
+is_following(dbref follower, dbref leader)
 {
   ATTR *a;
   char *s, *sp;
@@ -1119,7 +1274,7 @@ is_following(follower, leader)
   a = atr_get_noparent(follower, "FOLLOWING");
   if (!a)
     return 0;			/* Following no one */
-  strcpy(tbuf1, uncompress(AL_STR(a)));
+  strcpy(tbuf1, atr_value(a));
   s = trim_space_sep(tbuf1, ' ');
   while (s) {
     sp = split_token(&s, ' ');
@@ -1129,10 +1284,12 @@ is_following(follower, leader)
   return 0;
 }
 
+/** Clear a player's followers list.
+ * \param leader dbref of player whose list is to be cleared.
+ * \param noisy if 1, notify the player.
+ */
 void
-clear_followers(leader, noisy)
-    dbref leader;
-    int noisy;
+clear_followers(dbref leader, int noisy)
 {
   ATTR *a;
   char *s, *sp;
@@ -1141,7 +1298,7 @@ clear_followers(leader, noisy)
   a = atr_get_noparent(leader, "FOLLOWERS");
   if (!a)
     return;			/* No one's following me */
-  strcpy(tbuf1, uncompress(AL_STR(a)));
+  strcpy(tbuf1, atr_value(a));
   s = trim_space_sep(tbuf1, ' ');
   while (s) {
     sp = split_token(&s, ' ');
@@ -1155,10 +1312,12 @@ clear_followers(leader, noisy)
   (void) atr_clr(leader, "FOLLOWERS", GOD);
 }
 
+/** Clear a player's following list.
+ * \param follower dbref of player whose list is to be cleared.
+ * \param noisy if 1, notify the player.
+ */
 void
-clear_following(follower, noisy)
-    dbref follower;
-    int noisy;
+clear_following(dbref follower, int noisy)
 {
   ATTR *a;
   char *s, *sp;
@@ -1167,7 +1326,7 @@ clear_following(follower, noisy)
   a = atr_get_noparent(follower, "FOLLOWING");
   if (!a)
     return;			/* I'm not following anyone */
-  strcpy(tbuf1, uncompress(AL_STR(a)));
+  strcpy(tbuf1, atr_value(a));
   s = trim_space_sep(tbuf1, ' ');
   while (s) {
     sp = split_token(&s, ' ');
@@ -1198,7 +1357,7 @@ follower_command(dbref leader, dbref loc, const char *com)
   a = atr_get_noparent(leader, "FOLLOWERS");
   if (!a)
     return;			/* No followers */
-  strcpy(tbuf1, uncompress(AL_STR(a)));
+  strcpy(tbuf1, atr_value(a));
   s = tbuf1;
   while (s) {
     sp = split_token(&s, ' ');
