@@ -1,3 +1,10 @@
+/**
+ * \file mysocket.c
+ *
+ * \brief Socket routines for PennMUSH.
+ *
+ *
+ */
 #include "copyrite.h"
 
 #include "config.h"
@@ -26,6 +33,10 @@
 #endif
 #endif
 
+#ifdef I_NETINET_TCP
+#include <netinet/tcp.h>
+#endif
+
 #ifdef I_ARPA_INET
 #include <arpa/inet.h>
 #endif
@@ -44,13 +55,11 @@
 #include <nameser.h>
 #endif
 #endif
-#ifndef macintosh
 #ifndef __CYGWIN__
 #include <resolv.h>
 #endif
 #endif
 #endif
-#endif
 
 #ifdef I_NETDB
 #include <netdb.h>
@@ -78,13 +87,11 @@ extern int h_errno;
 #include <sys/time.h>
 #endif
 
-#ifndef INFOSLAVE
 #include "conf.h"
-#endif
+#include "externs.h"
 #include "mymalloc.h"
 #include "mysocket.h"
 #include "ident.h"
-#include "externs.h"
 #include "confmagic.h"
 
 #ifndef __ptr_t
@@ -97,6 +104,13 @@ extern int h_errno;
 
 
 #ifndef INFOSLAVE
+/** Given a sockaddr structure, try to look up and return hostname info.
+ * If we can't get a hostname from DNS (or if we're not using DNS),
+ * we settle for the IP address.
+ * \param host pointer to a sockaddr structure.
+ * \param len length of host structure.
+ * \return static hostname_info structure with hostname and port.
+ */
 struct hostname_info *
 hostname_convert(struct sockaddr *host, int len)
 {
@@ -113,6 +127,11 @@ hostname_convert(struct sockaddr *host, int len)
   return &hi;
 }
 
+/** Given a sockaddr structure, try to look up and return ip address info.
+ * \param host pointer to a sockaddr structure.
+ * \param len length of host structure.
+ * \return static hostname_info structure with ip address and port.
+ */
 struct hostname_info *
 ip_convert(struct sockaddr *host, int len)
 {
@@ -130,8 +149,14 @@ ip_convert(struct sockaddr *host, int len)
 }
 #endif				/* INFOSLAVE */
 
-/* Open a tcp connection to a given host and port. Basically
+/** Open a tcp connection to a given host and port. Basically
  * tcp_connect from UNPv1
+ * \param host hostname or ip address to connect to, as a string.
+ * \param myiterface pointer to sockaddr structure for specific interface.
+ * \param myilen length of myiterface
+ * \param port port to connect to.
+ * \param timeout pointer to timeout for connection.
+ * \return file descriptor for connected socket, or -1 for failure.
  */
 int
 make_socket_conn(const char *host, struct sockaddr *myiterface,
@@ -208,8 +233,13 @@ make_socket_conn(const char *host, struct sockaddr *myiterface,
   return s;
 }
 
-/* Start listening to TCP on a given port. Basically tcp_listen
+/** Start listening to TCP on a given port. Basically tcp_listen
  * from UNPv1
+ * \param port port to listen on.
+ * \param addr pointer to sockaddr to copy address data to.
+ * \param len length of addr.
+ * \param host hostname or address to listen on, as a string.
+ * \return file descriptor of listening socket, or -1 for failure.
  */
 int
 make_socket(Port_t port, union sockaddr_u *addr, socklen_t *len,
@@ -258,13 +288,11 @@ make_socket(Port_t port, union sockaddr_u *addr, socklen_t *len,
     if (s < 0)
       continue;
 
-#ifndef macintosh
     opt = 1;
     if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *) &opt, sizeof(opt)) < 0) {
       perror("setsockopt (Possibly ignorable)");
       continue;
     }
-#endif
 
     if (bind(s, server->ai_addr, server->ai_addrlen) == 0)
       break;			/* Success */
@@ -293,10 +321,12 @@ make_socket(Port_t port, union sockaddr_u *addr, socklen_t *len,
 }
 
 
+/** Make a socket do nonblocking i/o.
+ * \param s file descriptor of socket.
+ */
 void
 make_nonblocking(int s)
 {
-#ifndef macintosh
 #ifdef WIN32
   unsigned long arg = 1;
   if (ioctlsocket(s, FIONBIO, &arg) == -1) {
@@ -306,7 +336,7 @@ make_nonblocking(int s)
   if ((flags = fcntl(s, F_GETFL, 0)) == -1) {
     perror("make_nonblocking: fcntl");
 #ifndef INFOSLAVE
-    panic("F_GETFL fcntl failed");
+    mush_panic("F_GETFL fcntl failed");
 #else
     exit(1);
 #endif
@@ -316,20 +346,55 @@ make_nonblocking(int s)
 #endif				/* WIN32 */
     perror("make_nonblocking: fcntl");
 #ifndef INFOSLAVE
-    panic("O_NDELAY fcntl failed");
+    mush_panic("O_NDELAY fcntl failed");
 #else
     exit(1);
 #endif
   }
-#endif				/* !macintosh */
 }
 
-	/* From UNP, with changes. If nsec > 0, we set the socket
-	   nonblocking and connect with timeout. The socket is STILL
-	   nonblocking after it returns. If nsec == 0, a normal blocking
-	   connect is done. 
 
-	   Returns 0 on success, -1 on failure. */
+
+/** Enable TCP keepalive on the given socket if we can.
+ * \param s socket.
+ */
+/* ARGSUSED */
+void
+set_keepalive(int s __attribute__ ((__unused__)))
+{
+#ifdef CAN_KEEPALIVE
+  int keepalive = 1;
+#ifdef CAN_KEEPIDLE
+  int keepidle = 900;
+#endif
+
+  /* enable TCP keepalive */
+  if (setsockopt(s, SOL_SOCKET, SO_KEEPALIVE,
+		 (void *) &keepalive, sizeof(keepalive)) == -1)
+    fprintf(stderr, "[%d] could not set SO_KEEPALIVE: errno %d", s, errno);
+#ifdef CAN_KEEPIDLE
+  if (setsockopt(s, IPPROTO_TCP, TCP_KEEPIDLE,
+		 (void *) &keepidle, sizeof(keepidle)) == -1)
+    fprintf(stderr, "[%d] could not set TCP_KEEPALIVE: errno %d", s, errno);
+#endif
+#endif
+  return;
+}
+
+
+
+/** Connect a socket, possibly making it nonblocking first.
+ * From UNP, with changes. If nsec > 0, we set the socket
+ * nonblocking and connect with timeout. The socket is STILL
+ * nonblocking after it returns. If nsec == 0, a normal blocking
+ * connect is done. 
+ * \param sockfd file descriptor of socket.
+ * \param saptr pointer to sockaddr structure with connection data.
+ * \param salen length of saptr.
+ * \param timeout pointer to timeout value. If given, nonblocking connect.
+ * \retval 0 success.
+ * \retval -1 failure.
+ */
 int
 connect_nonb(int sockfd, const struct sockaddr *saptr, socklen_t salen,
 	     int *timeout)
@@ -436,8 +501,8 @@ done:
  * sizeof(int) < 4.  sizeof(int) > 4 is fine; all the world's not a VAX.
  */
 
-static const char *inet_ntop4(const u_char * src, char *dst, size_t size);
-static const char *inet_ntop6(const u_char * src, char *dst, size_t size);
+static const char *inet_ntop4(const unsigned char *src, char *dst, size_t size);
+static const char *inet_ntop6(const unsigned char *src, char *dst, size_t size);
 
 /* char *
  * inet_ntop(af, src, dst, size)
@@ -469,12 +534,12 @@ inet_ntop(int af, const void *src, char *dst, size_t size)
  *      `dst' (as a const)
  * notes:
  *      (1) uses no statics
- *      (2) takes a u_char* not an in_addr as input
+ *      (2) takes a unsigned char* not an in_addr as input
  * author:
  *      Paul Vixie, 1996.
  */
 static const char *
-inet_ntop4(const u_char * src, char *dst, size_t size)
+inet_ntop4(const unsigned char *src, char *dst, size_t size)
 {
   static const char fmt[] = "%u.%u.%u.%u";
   char tmp[sizeof "255.255.255.255"];
@@ -495,7 +560,7 @@ inet_ntop4(const u_char * src, char *dst, size_t size)
  *      Paul Vixie, 1996.
  */
 static const char *
-inet_ntop6(const u_char * src, char *dst, size_t size)
+inet_ntop6(const unsigned char *src, char *dst, size_t size)
 {
   /*
    * Note that int32_t and int16_t need only be "at least" large enough
@@ -508,7 +573,7 @@ inet_ntop6(const u_char * src, char *dst, size_t size)
   struct {
     int base, len;
   } best, cur;
-  u_int words[IN6ADDRSZ / INT16SZ];
+  unsigned int words[IN6ADDRSZ / INT16SZ];
   int i;
 
   /*
@@ -608,8 +673,8 @@ inet_ntop6(const u_char * src, char *dst, size_t size)
  * sizeof(int) < 4.  sizeof(int) > 4 is fine; all the world's not a VAX.
  */
 
-static int inet_pton4(const char *src, u_char * dst);
-static int inet_pton6(const char *src, u_char * dst);
+static int inet_pton4(const char *src, unsigned char *dst);
+static int inet_pton6(const char *src, unsigned char *dst);
 
 /* int
  * inet_pton(af, src, dst)
@@ -648,11 +713,11 @@ inet_pton(int af, const char *src, void *dst)
  *      Paul Vixie, 1996.
  */
 static int
-inet_pton4(const char *src, u_char * dst)
+inet_pton4(const char *src, unsigned char *dst)
 {
   static const char digits[] = "0123456789";
   int saw_digit, octets, ch;
-  u_char tmp[INADDRSZ], *tp;
+  unsigned char tmp[INADDRSZ], *tp;
 
   saw_digit = 0;
   octets = 0;
@@ -661,7 +726,7 @@ inet_pton4(const char *src, u_char * dst)
     const char *pch;
 
     if ((pch = strchr(digits, ch)) != NULL) {
-      u_int new = *tp * 10 + (pch - digits);
+      unsigned int new = *tp * 10 + (pch - digits);
 
       if (new > 255)
 	return (0);
@@ -700,14 +765,14 @@ inet_pton4(const char *src, u_char * dst)
  *      Paul Vixie, 1996.
  */
 static int
-inet_pton6(const char *src, u_char * dst)
+inet_pton6(const char *src, unsigned char *dst)
 {
   static const char xdigits_l[] = "0123456789abcdef",
     xdigits_u[] = "0123456789ABCDEF";
-  u_char tmp[IN6ADDRSZ], *tp, *endp, *colonp;
+  unsigned char tmp[IN6ADDRSZ], *tp, *endp, *colonp;
   const char *xdigits, *curtok;
   int ch, saw_xdigit;
-  u_int val;
+  unsigned int val;
 
   memset((tp = tmp), 0, IN6ADDRSZ);
   endp = tp + IN6ADDRSZ;
@@ -742,8 +807,8 @@ inet_pton6(const char *src, u_char * dst)
       }
       if (tp + INT16SZ > endp)
 	return (0);
-      *tp++ = (u_char) (val >> 8) & 0xff;
-      *tp++ = (u_char) val & 0xff;
+      *tp++ = (unsigned char) (val >> 8) & 0xff;
+      *tp++ = (unsigned char) val & 0xff;
       saw_xdigit = 0;
       val = 0;
       continue;
@@ -758,8 +823,8 @@ inet_pton6(const char *src, u_char * dst)
   if (saw_xdigit) {
     if (tp + INT16SZ > endp)
       return (0);
-    *tp++ = (u_char) (val >> 8) & 0xff;
-    *tp++ = (u_char) val & 0xff;
+    *tp++ = (unsigned char) (val >> 8) & 0xff;
+    *tp++ = (unsigned char) val & 0xff;
   }
   if (colonp != NULL) {
     /*
@@ -922,7 +987,7 @@ getaddrinfo(const char *hostname, const char *servname,
   canon = NULL;
 
   if (hintsp == NULL) {
-    bzero(&hints, sizeof(hints));
+    memset(&hints, 0, sizeof(hints));
     hints.ai_family = AF_UNSPEC;
   } else
     hints = *hintsp;		/* struct copy */
