@@ -1,4 +1,11 @@
-/* parse.c */
+/**
+ * \file parse.c
+ *
+ * \brief The PennMUSH function/expression parser
+ *
+ * The most important function in this file is process_expression.
+ *
+ */
 
 #include "copyrite.h"
 
@@ -11,9 +18,9 @@
 #include <errno.h>
 
 #include "conf.h"
+#include "externs.h"
 #include "ansi.h"
 #include "dbdefs.h"
-#include "externs.h"
 #include "function.h"
 #include "case.h"
 #include "match.h"
@@ -23,13 +30,9 @@
 #include "pcre.h"
 #include "flags.h"
 #include "log.h"
-#ifdef MEM_CHECK
-#include "memcheck.h"
-#endif
 #include "mymalloc.h"
 #include "confmagic.h"
 
-extern char ccom[];		/* bsd.c */
 extern char *absp[], *obj[], *poss[], *subj[];	/* fundb.c */
 extern int inum, inum_limit;
 int global_fun_invocations;
@@ -40,14 +43,17 @@ extern char *re_from;
 extern sig_atomic_t cpu_time_limit_hit;
 extern int cpu_limit_warning_sent;
 
+/** Structure for storing DEBUG output in a linked list */
 struct debug_info {
-  char *string;
-  Debug_Info *prev;
-  Debug_Info *next;
+  char *string;		/**< A DEBUG string */
+  Debug_Info *prev;	/**< Previous node in the linked list */
+  Debug_Info *next;	/**< Next node in the linked list */
 };
 
 FUNCTION_PROTO(fun_gfun);
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+
 /* Common error messages */
 char e_int[] = "#-1 ARGUMENT MUST BE INTEGER";
 char e_ints[] = "#-1 ARGUMENTS MUST BE INTEGERS";
@@ -64,6 +70,8 @@ char e_notvis[] = "#-1 NO SUCH OBJECT VISIBLE";
 char e_disabled[] = "#-1 FUNCTION DISABLED";
 char e_range[] = "#-1 OUT OF RANGE";
 
+#endif
+
 #if 0
 static void dummy_errors(void);
 
@@ -91,9 +99,12 @@ dummy_errors()
 
 #endif
 
+/** Given a string, parse out a dbref.
+ * \param str string to parse.
+ * \return dbref contained in the string, or NOTHING if not a valid dbref.
+ */
 dbref
-parse_dbref(str)
-    char const *str;
+parse_dbref(char const *str)
 {
   /* Make sure string is strictly in format "#nnn".
    * Otherwise, possesives will be fouled up.
@@ -114,9 +125,59 @@ parse_dbref(str)
   return num;
 }
 
+/** Version of parse_dbref() that doesn't do GoodObject checks */
+dbref
+qparse_dbref(const char *s)
+{
+
+  if (!s || (*s != NUMBER_TOKEN) || !*(s + 1))
+    return NOTHING;
+  return parse_integer(s + 1);
+}
+
+
+/** Given a string, parse out an object id or dbref.
+ * \param str string to parse.
+ * \return dbref of object referenced by string, or NOTHING if not a valid
+ * string or not an existing dbref.
+ */
+dbref
+parse_objid(char const *str)
+{
+  char *p;
+  if ((p = strchr(str, ':'))) {
+    char tbuf1[BUFFER_LEN];
+    dbref it;
+    /* A unique id, probably */
+    strncpy(tbuf1, str, (p - str));
+    tbuf1[p - str] = '\0';
+    it = parse_dbref(tbuf1);
+    if (GoodObject(it)) {
+      time_t matchtime;
+      p++;
+      if (!is_strict_integer(p))
+	return NOTHING;
+      matchtime = parse_integer(p);
+      return (CreTime(it) == matchtime) ? it : NOTHING;
+    } else
+      return NOTHING;
+  } else
+    return parse_dbref(str);
+}
+
+
+/** Given a string, parse out a boolean value.
+ * The meaning of boolean is fuzzy. To TinyMUSH, any string that begins with
+ * a non-zero number is true, and everything else is false.
+ * To PennMUSH, negative dbrefs are false, non-negative dbrefs are true,
+ * 0 is false, all other numbers are true, empty or blank strings are false,
+ * and all other strings are true. 
+ * \param str string to parse.
+ * \retval 1 str represents a true value.
+ * \retval 0 str represents a false value.
+ */
 int
-parse_boolean(str)
-    char const *str;
+parse_boolean(char const *str)
 {
   if (TINY_BOOLEANS) {
     return (atoi(str) ? 1 : 0);
@@ -126,21 +187,33 @@ parse_boolean(str)
      * Zero is false, all other numbers are true.
      * Empty (or space only) strings are false, all other strings are true.
      */
-    if (!str)
+    /* Null strings are false */
+    if (!str || !*str)
       return 0;
-    if (str[0] == '#' && str[1] == '-')	/* check like this so error msgs */
-      return 0;			/* are false */
+    /* Negative dbrefs are false - actually anything starting #-,
+     * which will also cover our error messages. */
+    if (*str == '#' && *(str + 1) && (*(str + 1) == '-'))
+      return 0;
+    /* Non-zero numbers are true, zero is false */
     if (is_strict_number(str))
       return parse_number(str) != 0;	/* avoid rounding problems */
+    /* Skip blanks */
     while (*str == ' ')
       str++;
+    /* If there's any non-blanks left, it's true */
     return *str != '\0';	/* force to 1 or 0 */
   }
 }
 
+/** Is a string a boolean value?
+ * To TinyMUSH, any integer is a boolean value. To PennMUSH, any
+ * string at all is boolean.
+ * \param str string to check.
+ * \retval 1 string is a valid boolean.
+ * \retval 0 string is not a valid boolean.
+ */
 int
-is_boolean(str)
-    char const *str;
+is_boolean(char const *str)
 {
   if (TINY_BOOLEANS)
     return is_integer(str);
@@ -148,9 +221,15 @@ is_boolean(str)
     return 1;
 }
 
+/** Is a string a dbref?
+ * A dbref is a string starting with a #, optionally followed by a -,
+ * and then followed by at least one digit, and nothing else.
+ * \param str string to check.
+ * \retval 1 string is a dbref.
+ * \retval 0 string is not a dbref.
+ */
 int
-is_dbref(str)
-    char const *str;
+is_dbref(char const *str)
 {
   if (!str || (*str != NUMBER_TOKEN) || !*(str + 1))
     return 0;
@@ -162,9 +241,39 @@ is_dbref(str)
   return !*str;
 }
 
+/** Is a string an objid?
+ * An objid is a string starting with a #, optionally followed by a -,
+ * and then followed by at least one digit, then optionally followed 
+ * by a : and at least one digit, and nothing else.
+ * In regex: ^#-?\d+(:\d+)?$
+ * \param str string to check.
+ * \retval 1 string is a dbref.
+ * \retval 0 string is not a dbref.
+ */
 int
-is_integer(str)
-    char const *str;
+is_objid(char const *str)
+{
+  static pcre *re = NULL;
+  const char *errptr;
+  int erroffset;
+
+  if (!str)
+    return 0;
+  if (!re)
+    re = pcre_compile("^#-?\\d+(?::\\d+)?$", 0, &errptr, &erroffset, NULL);
+  return (pcre_exec(re, NULL, str, strlen(str), 0, 0, NULL, 0) >= 0);
+}
+
+/** Is string an integer?
+ * To TinyMUSH, any string is an integer. To PennMUSH, a string that
+ * passes strtol is an integer, and a blank string is an integer
+ * if NULL_EQ_ZERO is turned on.
+ * \param str string to check.
+ * \retval 1 string is an integer.
+ * \retval 0 string is not an integer.
+ */
+int
+is_integer(char const *str)
 {
   char *end;
 
@@ -184,9 +293,16 @@ is_integer(str)
   return 1;
 }
 
+/** Is string an uinteger?
+ * To TinyMUSH, any string is an uinteger. To PennMUSH, a string that
+ * passes strtoul is an uinteger, and a blank string is an uinteger
+ * if NULL_EQ_ZERO is turned on.
+ * \param str string to check.
+ * \retval 1 string is an uinteger.
+ * \retval 0 string is not an uinteger.
+ */
 int
-is_uinteger(str)
-    char const *str;
+is_uinteger(char const *str)
 {
   char *end;
 
@@ -209,9 +325,14 @@ is_uinteger(str)
   return 1;
 }
 
+/** Is string a number by the strict definition?
+ * A strict number is a non-null string that passes strtod.
+ * \param str string to check.
+ * \retval 1 string is a strict number.
+ * \retval 0 string is not a strict number.
+ */
 int
-is_strict_number(str)
-    char const *str;
+is_strict_number(char const *str)
 {
   char *end;
   NVAL val;
@@ -224,9 +345,14 @@ is_strict_number(str)
   return end > str;
 }
 
+/** Is string an integer by the strict definition?
+ * A strict integer is a non-null string that passes strtol.
+ * \param str string to check.
+ * \retval 1 string is a strict integer.
+ * \retval 0 string is not a strict integer.
+ */
 int
-is_strict_integer(str)
-    const char *str;
+is_strict_integer(char const *str)
 {
   char *end;
   int val;
@@ -239,9 +365,15 @@ is_strict_integer(str)
   return end > str;
 }
 
+/** Is string a number?
+ * To TinyMUSH, any string is a number. To PennMUSH, a strict number is
+ * a number, and a blank string is a number if NULL_EQ_ZERO is turned on.
+ * \param str string to check.
+ * \retval 1 string is a number.
+ * \retval 0 string is not a number.
+ */
 int
-is_number(str)
-    char const *str;
+is_number(char const *str)
 {
   /* If we're emulating Tiny, anything is a number */
   if (TINY_MATH)
@@ -261,19 +393,62 @@ extern signed char qreg_indexes[UCHAR_MAX + 1];
 #ifdef WIN32
 #pragma warning( disable : 4761)	/* NJG: disable warning re conversion */
 #endif
-/* Function and other substitution evaluation. */
+
+
+/** Function and other substitution evaluation.
+ * This is the PennMUSH function/expression parser. Big stuff.
+ *
+ * All results are returned in buff, at the point *bp. bp is likely
+ * not equal to buff, so make no assumptions about writing at the
+ * start of the buffer.  *bp must be updated to point at the next
+ * place to be filled (ala safe_str() and safe_chr()).  Be very
+ * careful about not overflowing buff; use of safe_str() and safe_chr()
+ * for all writes into buff is highly recommended.
+ * 
+ * nargs is the count of the number of arguments passed to the function,
+ * and args is an array of pointers to them.  args will have at least
+ * nargs elements, or 10 elements, whichever is greater.  The first ten
+ * elements are initialized to NULL for ease of porting functions from
+ * the old style, but relying on such is considered bad form.
+ * The argument strings are stored in BUFFER_LEN buffers, but reliance
+ * on that size is also considered bad form.  The argument strings may
+ * be modified, but modifying the pointers to the argument strings will
+ * cause crashes. 
+ *
+ * executor corresponds to %!, the object invoking the function.
+ * caller   corresponds to %@, the last object to do a U() or similar.
+ * enactor  corresponds to %#, the object that started the whole mess.
+ * Note that fun_ufun() and similar must swap around these parameters
+ * in calling process_expression(); no checks are made in the parser
+ * itself to maintain these values.
+ *
+ * called_as contains a pointer to the name of the function called
+ * (taken from the function table).  This may be used to distinguish
+ * multiple functions which use the same C function for implementation.
+ *
+ * pe_info holds context information used by the parser.  It should
+ * be passed untouched to process_expression(), if it is called.
+ * pe_info should be treated as a black box; its structure and contents
+ * may change without notice.
+ *
+ * Normally, p_e() returns 0. It returns 1 upon hitting the CPU time limit.
+ *
+ * \param buff buffer to store returns of parsing.
+ * \param bp pointer to pointer into buff marking insert position.
+ * \param str string to parse.
+ * \param executor dbref of the object invoking the function.
+ * \param caller dbref of  the last object to use u()
+ * \param enactor dbref of the enactor.
+ * \param eflags flags to control what is evaluated.
+ * \param tflags flags to control what terminates an expression.
+ * \param pe_info pointer to parser context data.
+ * \retval 0 success.
+ * \retval 1 CPU time limit exceeded.
+ */
 int
-process_expression(buff, bp, str, executor, caller, enactor,
-		   eflags, tflags, pe_info)
-    char *buff;
-    char **bp;
-    char const **str;
-    dbref executor;
-    dbref caller;
-    dbref enactor;
-    int eflags;
-    int tflags;
-    PE_Info *pe_info;
+process_expression(char *buff, char **bp, char const **str,
+		   dbref executor, dbref caller, dbref enactor,
+		   int eflags, int tflags, PE_Info * pe_info)
 {
   int debugging = 0, made_info = 0;
   char *debugstr = NULL, *sourcestr = NULL;
@@ -329,6 +504,15 @@ process_expression(buff, bp, str, executor, caller, enactor,
     old_iter_limit = -1;
   }
 
+  /* If we've been asked to evaluate, log the expression if:
+   * (a) the last thing we logged wasn't an expression, and
+   * (b) this expression isn't a substring of the last thing we logged
+   */
+  if ((eflags & PE_EVALUATE) &&
+      ((last_activity_type() != LA_PE) || !strstr(last_activity(), *str))) {
+    log_activity(LA_PE, executor, *str);
+  }
+
   if (eflags != PE_NOTHING) {
     if (((*bp) - buff) > (BUFFER_LEN - SBUF_LEN)) {
       realbuff = buff;
@@ -350,7 +534,8 @@ process_expression(buff, bp, str, executor, caller, enactor,
 
 
   if (eflags != PE_NOTHING) {
-    debugging = Debug(executor) && Connected(Owner(executor));
+    debugging = (Debug(executor) || (eflags & PE_DEBUG))
+      && (Connected(Owner(executor)) || atr_get(executor, "DEBUGFORWARDLIST"));
     if (debugging) {
       int j;
       char *debugp;
@@ -538,6 +723,11 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	case '#':		/* enactor dbref */
 	  safe_dbref(enactor, buff, bp);
 	  break;
+	case ':':		/* enactor unique id */
+	  safe_dbref(enactor, buff, bp);
+	  safe_chr(':', buff, bp);
+	  safe_integer(CreTime(enactor), buff, bp);
+	  break;
 	case '?':		/* function limits */
 	  if (pe_info) {
 	    safe_integer(pe_info->fun_invocations, buff, bp);
@@ -547,6 +737,9 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	    safe_str("0 0", buff, bp);
 	  }
 	  break;
+	case '~':		/* enactor accented name */
+	  safe_str(accented_name(enactor), buff, bp);
+	  break;
 	case '0':
 	case '1':
 	case '2':
@@ -557,8 +750,8 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	case '7':
 	case '8':
 	case '9':		/* positional argument */
-	  if (wenv[savec - '0'])
-	    safe_str(wenv[savec - '0'], buff, bp);
+	  if (global_eval_context.wenv[savec - '0'])
+	    safe_str(global_eval_context.wenv[savec - '0'], buff, bp);
 	  break;
 	case 'A':
 	case 'a':		/* enactor absolute possessive pronoun */
@@ -572,7 +765,7 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	  break;
 	case 'C':
 	case 'c':		/* command line */
-	  safe_str(ccom, buff, bp);
+	  safe_str(global_eval_context.ccom, buff, bp);
 	  break;
 	case 'L':
 	case 'l':		/* enactor location dbref */
@@ -604,8 +797,8 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	  (*str)++;
 	  if ((qindex = qreg_indexes[(unsigned char) nextc]) == -1)
 	    break;
-	  if (renv[qindex])
-	    safe_str(renv[qindex], buff, bp);
+	  if (global_eval_context.renv[qindex])
+	    safe_str(global_eval_context.renv[qindex], buff, bp);
 	  break;
 	case 'R':
 	case 'r':		/* newline */
@@ -639,7 +832,7 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	  temp[2] = '\0';
 	  attrib = atr_get(executor, temp);
 	  if (attrib)
-	    safe_str(uncompress(attrib->value), buff, bp);
+	    safe_str(atr_value(attrib), buff, bp);
 	  break;
 	default:		/* just copy */
 	  safe_chr(savec, buff, bp);
@@ -717,11 +910,17 @@ process_expression(buff, bp, str, executor, caller, enactor,
       (*str)++;
       if (!(eflags & PE_EVALUATE) || !(eflags & PE_FUNCTION_CHECK)) {
 	safe_chr('(', buff, bp);
+	if (**str == ' ') {
+	  safe_chr(**str, buff, bp);
+	  (*str)++;
+	}
 	if (process_expression(buff, bp, str,
 			       executor, caller, enactor,
 			       eflags & ~PE_STRIP_BRACES, PT_PAREN, pe_info))
 	  retval = 1;
 	if (**str == ')') {
+	  if (eflags & PE_COMPRESS_SPACES && (*str)[-1] == ' ')
+	    safe_chr(' ', buff, bp);
 	  safe_chr(')', buff, bp);
 	  (*str)++;
 	}
@@ -817,16 +1016,14 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	/* Get the arguments */
 	temp_eflags = (eflags & ~PE_FUNCTION_MANDATORY)
 	  | PE_COMPRESS_SPACES | PE_EVALUATE | PE_FUNCTION_CHECK;
-	if (fp->flags & FN_BUILTIN) {
-	  switch (fp->flags & FN_ARG_MASK) {
-	  case FN_LITERAL:
-	    temp_eflags |= PE_LITERAL;
-	    /* FALL THROUGH */
-	  case FN_NOPARSE:
-	    temp_eflags &= ~(PE_COMPRESS_SPACES | PE_EVALUATE |
-			     PE_FUNCTION_CHECK);
-	    break;
-	  }
+	switch (fp->flags & FN_ARG_MASK) {
+	case FN_LITERAL:
+	  temp_eflags |= PE_LITERAL;
+	  /* FALL THROUGH */
+	case FN_NOPARSE:
+	  temp_eflags &= ~(PE_COMPRESS_SPACES | PE_EVALUATE |
+			   PE_FUNCTION_CHECK);
+	  break;
 	}
 	denied = !check_func(executor, fp);
 	if (denied)
@@ -926,6 +1123,23 @@ process_expression(buff, bp, str, executor, caller, enactor,
 	      pe_info->fun_invocations++;
 	      fp->where.fun(fp, buff, bp, nfargs, fargs, arglens, executor,
 			    caller, enactor, fp->name, pe_info);
+	      if (fp->flags & FN_LOGARGS) {
+		char logstr[BUFFER_LEN];
+		char *logp;
+		int logi;
+		logp = logstr;
+		safe_str(fp->name, logstr, &logp);
+		safe_chr('(', logstr, &logp);
+		for (logi = 0; logi < nfargs; logi++) {
+		  safe_str(fargs[logi], logstr, &logp);
+		  if (logi + 1 < nfargs)
+		    safe_str(fargs[logi], logstr, &logp);
+		}
+		safe_chr(')', logstr, &logp);
+		*logp = '\0';
+		do_log(LT_CMD, executor, caller, "%s", logstr);
+	      } else if (fp->flags & FN_LOGNAME)
+		do_log(LT_CMD, executor, caller, "%s()", fp->name);
 	    } else {
 	      dbref thing;
 	      ATTR *attrib;
@@ -1013,7 +1227,10 @@ exit_sequence:
 	    dbuf[0] = '\0';
 	    safe_format(dbuf, &dbp, "%s :", pe_info->debug_strings->string);
 	    *dbp = '\0';
-	    raw_notify(Owner(executor), dbuf);
+	    if (Connected(Owner(executor)))
+	      raw_notify(Owner(executor), dbuf);
+	    notify_list(executor, executor, "DEBUGFORWARDLIST", dbuf,
+			NA_NOLISTEN | NA_NOPREFIX);
 	    pe_info->debug_strings = pe_info->debug_strings->next;
 	    mush_free((Malloc_t) pe_info->debug_strings->prev,
 		      "process_expression.debug_node");
@@ -1026,7 +1243,10 @@ exit_sequence:
 	dbuf[0] = '\0';
 	safe_format(dbuf, &dbp, "%s => %s", debugstr, startpos);
 	*dbp = '\0';
-	raw_notify(Owner(executor), dbuf);
+	if (Connected(Owner(executor)))
+	  raw_notify(Owner(executor), dbuf);
+	notify_list(executor, executor, "DEBUGFORWARDLIST", dbuf,
+		    NA_NOLISTEN | NA_NOPREFIX);
       } else {
 	Debug_Info *node;
 	node = pe_info->debug_strings;
