@@ -1,9 +1,13 @@
-/* set.c */
+/**
+ * \file set.c
+ *
+ * \brief PennMUSH commands that set parameters.
+ *
+ *
+ */
 
 #include "copyrite.h"
 #include "config.h"
-
-/* commands which set parameters */
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
@@ -18,10 +22,10 @@
 #include <stdlib.h>
 
 #include "conf.h"
+#include "externs.h"
 #include "mushdb.h"
 #include "match.h"
 #include "attrib.h"
-#include "externs.h"
 #include "ansi.h"
 #include "command.h"
 #include "mymalloc.h"
@@ -29,39 +33,40 @@
 #include "dbdefs.h"
 #include "lock.h"
 #include "log.h"
+#include "game.h"
 #include "confmagic.h"
 
-void do_name(dbref player, const char *name, char *newname);
-void do_chown(dbref player, const char *name, const char *newobj, int preserve);
 static int chown_ok(dbref player, dbref thing, dbref newowner);
-int do_chzone(dbref player, const char *name, const char *newobj, int noisy);
 void do_attrib_flags
   (dbref player, const char *obj, const char *atrname, const char *flag);
-int do_set(dbref player, const char *name, char *flag);
-void do_cpattr
-  (dbref player, char *oldpair, char **newpair, int move, int noflagcopy);
-void do_gedit(dbref player, char *it, char **argv);
-void do_trigger(dbref player, char *object, char **argv);
-void do_use(dbref player, const char *what);
-void do_parent(dbref player, char *name, char *parent_name);
-void do_wipe(dbref player, char *name);
-static int af_helper(dbref player, dbref thing, char const *pattern,
-		     ATTR *atr, void *args);
-static int gedit_helper(dbref player, dbref thing, char const *pattern,
-			ATTR *atr, void *args);
-static int wipe_helper(dbref player, dbref thing, char const *pattern,
-		       ATTR *atr, void *args);
+static int af_helper(dbref player, dbref thing, dbref parent,
+		     char const *pattern, ATTR *atr, void *args);
+static int gedit_helper(dbref player, dbref thing, dbref parent,
+			char const *pattern, ATTR *atr, void *args);
+static int wipe_helper(dbref player, dbref thing, dbref parent,
+		       char const *pattern, ATTR *atr, void *args);
 static void copy_attrib_flags(dbref player, dbref target, ATTR *atr, int flags);
 
 
+/** Rename something.
+ * \verbatim
+ * This implements @name.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name current name of object to rename.
+ * \param newname new name for object.
+ */
 void
-do_name(player, name, newname)
-    dbref player;
-    const char *name;
-    char *newname;
+do_name(dbref player, const char *name, char *newname)
 {
   dbref thing;
+  ATTR *a = NULL;
+  char alias[BUFFER_LEN];
   char *password;
+  char *myenv[10];
+  int i;
+
+  newname = trim_space_sep(newname, ' ');
 
   if ((thing = match_controlled(player, name)) != NOTHING) {
     /* check for bad name */
@@ -73,7 +78,8 @@ do_name(player, name, newname)
     if (IsPlayer(thing)) {
       if (PLAYER_NAME_SPACES) {
 	if (*newname == '\"') {
-	  for (; *newname && ((*newname == '\"') || isspace(*newname));
+	  for (; *newname && ((*newname == '\"')
+			      || isspace((unsigned char) *newname));
 	       newname++) ;
 	  password = newname;
 	  while (*password && (*password != '\"')) {
@@ -110,33 +116,27 @@ do_name(player, name, newname)
 	    password++;
 	}
       }
-      /* check for null password */
-      if (!God(player) && !*password) {
-	notify(player,
-	       T("You must specify a password to change a player name."));
-	notify(player, T("E.g.: @name player = newname password"));
-	return;
-      } else if (!God(player) && !password_check(thing, password)) {
-	notify(player, T("Incorrect password."));
-	return;
-      } else if (strcasecmp(newname, Name(thing))
-		 && !ok_player_name(newname)) {
+      a = atr_get_noparent(thing, "ALIAS");
+      if (a)
+	strcpy(alias, atr_value(a));
+      if (strcasecmp(newname, Name(thing))
+	  && !(a && !strcasecmp(newname, alias))	/* Swap alias, name */
+	  &&!ok_player_name(newname, thing)) {
 	/* strcasecmp allows changing foo to Foo, etc. */
 	notify(player, T("You can't give a player that name."));
 	return;
       }
+      if (a && strcasecmp(alias, newname)) {
+	a = NULL;
+      }
       /* everything ok, notify */
       do_log(LT_CONN, 0, 0, T("Name change by %s(#%d) to %s"),
 	     Name(thing), thing, newname);
       if (Suspect(thing))
-	flag_broadcast(WIZARD, 0,
+	flag_broadcast("WIZARD", 0,
 		       T("Broadcast: Suspect %s changed name to %s."),
 		       Name(thing), newname);
-      delete_player(thing, NULL);
-      set_name(thing, newname);
-      add_player(thing, NULL);
-      notify(player, T("Name set."));
-      return;
+      /* everything ok, we can fall through to change the name */
     } else {
       if (!ok_name(newname)) {
 	notify(player, T("That is not a reasonable name."));
@@ -145,17 +145,44 @@ do_name(player, name, newname)
     }
 
     /* everything ok, change the name */
+    myenv[0] = (char *) mush_malloc(BUFFER_LEN, "string");
+    myenv[1] = (char *) mush_malloc(BUFFER_LEN, "string");
+    strncpy(myenv[0], Name(thing), BUFFER_LEN - 1);
+    myenv[0][BUFFER_LEN - 1] = '\0';
+    strcpy(myenv[1], newname);
+    for (i = 2; i < 10; i++)
+      myenv[i] = NULL;
+
+    if (IsPlayer(thing) && !a)
+      delete_player(thing, NULL);
+    else if (a)
+      atr_add(thing, "ALIAS", Name(thing), player, 0);
+
     set_name(thing, newname);
-    notify(player, T("Name set."));
+
+    if (IsPlayer(thing) && !a)
+      add_player(thing, NULL);
+
+    if (!AreQuiet(player, thing))
+      notify(player, T("Name set."));
+    real_did_it(player, thing, NULL, NULL, "ONAME", NULL, "ANAME", NOTHING,
+		myenv, NA_INTER_PRESENCE);
+    mush_free(myenv[0], "string");
+    mush_free(myenv[1], "string");
   }
 }
 
+/** Change an object's owner.
+ * \verbatim
+ * This implements @chown.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name name of object to change owner of.
+ * \param newobj name of new owner for object.
+ * \param preserve if 1, preserve privileges and don't halt the object.
+ */
 void
-do_chown(player, name, newobj, preserve)
-    dbref player;
-    const char *name;
-    const char *newobj;
-    int preserve;
+do_chown(dbref player, const char *name, const char *newobj, int preserve)
 {
   dbref thing;
   dbref newowner = NOTHING;
@@ -216,21 +243,15 @@ do_chown(player, name, newobj, preserve)
     }
     /* Credit the current owner */
     giveto(Owner(thing), Pennies(thing));
-#ifdef QUOTA
     change_quota(Owner(thing), QUOTA_COST);
-#endif
   }
   chown_object(player, thing, newowner, preserve);
   notify(player, T("Owner changed."));
 }
 
 static int
-chown_ok(player, thing, newowner)
-    dbref player;
-    dbref thing;
-    dbref newowner;
+chown_ok(dbref player, dbref thing, dbref newowner)
 {
-
   /* Cant' touch garbage */
   if (IsGarbage(thing))
     return 0;
@@ -278,44 +299,83 @@ chown_ok(player, thing, newowner)
 }
 
 
-/* Actually change the ownership of something, and fix bits */
+/** Actually change the ownership of something, and fix bits.
+ * \param player the enactor.
+ * \param thing object to change ownership of.
+ * \param newowner new owner for thing.
+ * \param preserve if 1, preserve privileges and don't halt.
+ */
 void
-chown_object(player, thing, newowner, preserve)
-    dbref player;
-    dbref thing;
-    dbref newowner;
-    int preserve;
+chown_object(dbref player, dbref thing, dbref newowner, int preserve)
 {
-  undestroy(player, thing);
+  (void) undestroy(player, thing);
   if (God(player)) {
     Owner(thing) = newowner;
   } else {
     Owner(thing) = Owner(newowner);
   }
-  Zone(thing) = Zone(newowner);
-  Flags(thing) &= ~CHOWN_OK;
+  /* Don't allow circular zones */
+  Zone(thing) = NOTHING;
+  if (GoodObject(Zone(newowner))) {
+    dbref tmp;
+    int ok_to_zone = 1;
+    int zone_depth = MAX_ZONES;
+    for (tmp = Zone(Zone(newowner)); GoodObject(tmp); tmp = Zone(tmp)) {
+      if (tmp == thing) {
+	notify(player, T("Circular zone broken."));
+	ok_to_zone = 0;
+	break;
+      }
+      if (tmp == Zone(tmp))	/* Ran into an object zoned to itself */
+	break;
+      zone_depth--;
+      if (!zone_depth) {
+	ok_to_zone = 0;
+	notify(player, T("Overly deep zone chain broken."));
+	break;
+      }
+    }
+    if (ok_to_zone)
+      Zone(thing) = Zone(newowner);
+  }
+  clear_flag_internal(thing, "CHOWN_OK");
   if (!preserve || !Wizard(player)) {
-    Flags(thing) &= ~WIZARD;
-    Flags(thing) &= ~ROYALTY;
-    Flags(thing) &= ~INHERIT;
-    Flags(thing) |= HALT;
-    Powers(thing) = 0;		/* wipe out all powers */
+    clear_flag_internal(thing, "WIZARD");
+    clear_flag_internal(thing, "ROYALTY");
+    clear_flag_internal(thing, "TRUST");
+    set_flag_internal(thing, "HALT");
+    destroy_flag_bitmask(Powers(thing));
+    Powers(thing) = new_flag_bitmask("POWER");
     do_halt(thing, "", thing);
   } else {
     if ((newowner != player) && Wizard(thing) && !God(player)) {
-      notify(player,
-	     T
-	     ("Warning: WIZ flag reset because @CHOWN/PRESERVE is to a third party."));
-      Flags(thing) &= ~WIZARD;
+      notify_format(player,
+		    T
+		    ("Warning: WIZ flag reset on #%d because @CHOWN/PRESERVE is to a third party."),
+		    thing);
+      clear_flag_internal(thing, "WIZARD");
     }
-    if (Powers(thing) || Wizard(thing) || Royalty(thing) || Inherit(thing))
-      notify(player,
-	     T
-	     ("Warning: @CHOWN/PRESERVE on a target with WIZ, ROY, INHERIT, or @power privileges."));
+    if (!null_flagmask("POWER", Powers(thing)) || Wizard(thing) ||
+	Royalty(thing) || Inherit(thing))
+      notify_format(player,
+		    T
+		    ("Warning: @CHOWN/PRESERVE on a object (#%d) with WIZ, ROY, INHERIT, or @power privileges."),
+		    thing);
   }
 }
 
 
+/** Change an object's zone.
+ * \verbatim
+ * This implements @chzone.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name name of the object to change zone of.
+ * \param newobj name of new ZMO.
+ * \param noisy if 1, notify player about success and failure.
+ * \retval 0 failed to change zone.
+ * \retval 1 successfully changed zone.
+ */
 int
 do_chzone(dbref player, char const *name, char const *newobj, int noisy)
 {
@@ -370,6 +430,7 @@ do_chzone(dbref player, char const *name, char const *newobj, int noisy)
   /* Don't allow circular zones */
   if (GoodObject(zone)) {
     dbref tmp;
+    int zone_depth = MAX_ZONES;
     for (tmp = Zone(zone); GoodObject(tmp); tmp = Zone(tmp)) {
       if (tmp == thing) {
 	notify(player, T("You can't make circular zones!"));
@@ -377,36 +438,21 @@ do_chzone(dbref player, char const *name, char const *newobj, int noisy)
       }
       if (tmp == Zone(tmp))	/* Ran into an object zoned to itself */
 	break;
+      zone_depth--;
+      if (!zone_depth) {
+	notify(player, T("Overly deep zone chain."));
+	return 0;
+      }
     }
   }
 
-
   /* Don't allow chzone to objects without elocks! 
    * If no lock is set, set a default lock (warn if zmo are used for control)
    * This checks for many trivial elocks (canuse/1, where &canuse=1)
    */
-  if (zone != NOTHING) {
-    struct boolexp *key = getlock(zone, Zone_Lock);
-    if (key == TRUE_BOOLEXP) {
-      add_lock(GOD, zone, Zone_Lock, parse_boolexp(zone, "$me", Zone_Lock), -1);
-      if (noisy && !ZONE_CONTROL_ZMP)
-	notify(player,
-	       T("Unlocked ZMO - automatically zone-locking ZMO to its owner"));
-    } else if (eval_lock(Location(player), zone, Zone_Lock)) {
-      /* Does #0 and #2 pass it? If so, probably trivial elock */
-      if (eval_lock(PLAYER_START, zone, Zone_Lock) &&
-	  eval_lock(MASTER_ROOM, zone, Zone_Lock)) {
-	if (noisy)
-	  notify(player,
-		 T("ZMO requires a more secure zone-lock before you @chzone!"));
-	return 0;
-      }
-      /* Probably inexact zone lock */
-      notify(player,
-	     T
-	     ("Warning: ZMO may have loose zone lock. Lock ZMOs to =player, not player"));
-    }
-  }
+  if (zone != NOTHING)
+    check_zone_lock(player, zone, noisy);
+
   /* Warn Wiz/Royals when they zone their stuff */
   if ((zone != NOTHING) && Hasprivs(Owner(thing))) {
     if (noisy)
@@ -422,18 +468,17 @@ do_chzone(dbref player, char const *name, char const *newobj, int noisy)
      * inconvenient -- although this may pose a bit of a security
      * risk. Be careful when @chzone'ing wizard or royal players.
      */
-    Flags(thing) &= ~WIZARD;
-    Flags(thing) &= ~ROYALTY;
-    Flags(thing) &= ~INHERIT;
-    Powers(thing) = 0;		/* wipe out all powers */
+    clear_flag_internal(thing, "WIZARD");
+    clear_flag_internal(thing, "ROYALTY");
+    clear_flag_internal(thing, "TRUST");
+    destroy_flag_bitmask(Powers(thing));
+    Powers(thing) = new_flag_bitmask("POWER");
   } else {
-    if (zone != NOTHING) {
+    if (noisy && (zone != NOTHING)) {
       if (Hasprivs(thing))
-	if (noisy)
-	  notify(player, T("Warning: @chzoning a privileged player."));
-      if (Flags(thing) & INHERIT)
-	if (noisy)
-	  notify(player, T("Warning: @chzoning an TRUST player."));
+	notify(player, T("Warning: @chzoning a privileged player."));
+      if (Inherit(thing))
+	notify(player, T("Warning: @chzoning an TRUST player."));
     }
   }
   if (noisy)
@@ -441,18 +486,18 @@ do_chzone(dbref player, char const *name, char const *newobj, int noisy)
   return 1;
 }
 
+/** Structure for af_helper() data. */
 struct af_args {
-  int f;
-  char const *flag;
+  int f;		/**< flag bits */
+  int clear;		/**< True to remove the flag */
+  char *flag;		/**< flag name */
 };
 
 static int
-af_helper(player, thing, pattern, atr, args)
-    dbref player;
-    dbref thing;
-    char const *pattern __attribute__ ((__unused__));
-    ATTR *atr;
-    void *args;
+af_helper(dbref player, dbref thing,
+	  dbref parent __attribute__ ((__unused__)),
+	  char const *pattern
+	  __attribute__ ((__unused__)), ATTR *atr, void *args)
 {
   struct af_args *af = args;
 
@@ -461,21 +506,21 @@ af_helper(player, thing, pattern, atr, args)
    * There is one special case - the resetting of the SAFE flag.
    */
   if (!(Can_Write_Attr(player, thing, AL_ATTR(atr)) ||
-	((*af->flag == NOT_TOKEN) && (af->f == AF_SAFE) &&
+	(af->clear && (af->f & AF_SAFE) &&
 	 Can_Write_Attr_Ignore_Safe(player, thing, AL_ATTR(atr))))) {
     notify_format(player, T("You cannot change that flag on %s/%s"),
 		  Name(thing), AL_NAME(atr));
     return 0;
   }
 
-  if (*af->flag == NOT_TOKEN) {
+  if (af->clear) {
     AL_FLAGS(atr) &= ~af->f;
-    if (!Quiet(player) && !(Quiet(thing) && (Owner(thing) == player)))
+    if (!AreQuiet(player, thing))
       notify_format(player, T("%s/%s - %s reset."), Name(thing), AL_NAME(atr),
-		    (af->flag + 1));
+		    af->flag);
   } else {
     AL_FLAGS(atr) |= af->f;
-    if (!Quiet(player) && !(Quiet(thing) && (Owner(thing) == player)))
+    if (!AreQuiet(player, thing))
       notify_format(player, T("%s/%s - %s set."), Name(thing), AL_NAME(atr),
 		    af->flag);
   }
@@ -484,11 +529,7 @@ af_helper(player, thing, pattern, atr, args)
 }
 
 static void
-copy_attrib_flags(player, target, atr, flags)
-    dbref player;
-    dbref target;
-    ATTR *atr;
-    int flags;
+copy_attrib_flags(dbref player, dbref target, ATTR *atr, int flags)
 {
   if (!atr)
     return;
@@ -501,12 +542,15 @@ copy_attrib_flags(player, target, atr, flags)
   AL_FLAGS(atr) = flags;
 }
 
+/** Set a flag on an attribute.
+ * \param player the enactor.
+ * \param obj the name of the object with the attribute.
+ * \param atrname the name of the attribute.
+ * \param flag the name of the flag to set or clear.
+ */
 void
-do_attrib_flags(player, obj, atrname, flag)
-    dbref player;
-    const char *obj;
-    const char *atrname;
-    const char *flag;
+do_attrib_flags(dbref player, const char *obj, const char *atrname,
+		const char *flag)
 {
   struct af_args af;
   dbref thing;
@@ -519,29 +563,42 @@ do_attrib_flags(player, obj, atrname, flag)
     notify(player, T("What flag do you want to set?"));
     return;
   }
+
+  af.clear = 0;
+
   /* move past NOT token if there is one */
-  for (p = flag; *p && ((*p == NOT_TOKEN) || isspace((unsigned char) *p));
-       p++) ;
+  for (p = flag; *p && ((*p == NOT_TOKEN) || isspace((unsigned char) *p)); p++)
+    if (*p == NOT_TOKEN)
+      af.clear = !af.clear;
 
   if ((af.f = string_to_atrflag(player, p)) < 0) {
     notify(player, T("Unrecognized attribute flag."));
     return;
   }
-  af.flag = flag;
-  if (!atr_iter_get(player, thing, atrname, af_helper, &af))
+  af.flag = mush_strdup(atrflag_to_string(af.f), "af_flag list");
+  if (!atr_iter_get(player, thing, atrname, 0, af_helper, &af))
     notify(player, T("No attribute found to change."));
+  mush_free((Malloc_t) af.flag, "af_flag list");
 }
 
 
+/** Set a flag, attribute flag, or attribute.
+ * \verbatim
+ * This implements @set.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name the first (left) argument to the command.
+ * \param flag the second (right) argument to the command.
+ * \retval 1 successful set.
+ * \retval 0 failure to set.
+ */
 int
-do_set(player, name, flag)
-    dbref player;
-    const char *name;
-    char *flag;
+do_set(dbref player, const char *name, char *flag)
 {
   dbref thing;
-  int her, listener;
-  char *p;
+  int her, listener, negate;
+  char *p, *f;
+  char flagbuff[BUFFER_LEN];
 
   /* check for attribute flag set first */
   if ((p = strchr(name, '/')) != NULL) {
@@ -567,34 +624,40 @@ do_set(player, name, flag)
     }
     return do_set_atr(thing, flag, p, player, 1);
   }
-  /* we haven't set an attribute, so we must be setting a flag */
-
-  /* move p past NOT_TOKEN if present */
-  for (p = (char *) flag;
-       *p && (*p == NOT_TOKEN || isspace((unsigned char) *p)); p++) ;
-
-  /* identify flag */
+  /* we haven't set an attribute, so we must be setting flags */
+  strcpy(flagbuff, flag);
+  p = trim_space_sep(flagbuff, ' ');
   if (*p == '\0') {
     notify(player, T("You must specify a flag to set."));
     return 0;
   }
-  set_flag(player, thing, p, (*flag == NOT_TOKEN) ? 1 : 0, her, listener);
+  do {
+    f = split_token(&p, ' ');
+    negate = 0;
+    if (*f == NOT_TOKEN && *(f + 1)) {
+      negate = 1;
+      f++;
+    }
+    set_flag(player, thing, f, negate, her, listener);
+  } while (p);
   return 1;
 }
 
+/** Copy or move an attribute.
+ * \verbatim
+ * This implements @cpattr and @mvattr.
+ * the command is of the format:
+ * @cpattr oldobj/oldattr = newobj1/newattr1, newobj2/newattr2, etc.
+ * \endverbatim
+ * \param player the enactor.
+ * \param oldpair the obj/attribute pair to copy from.
+ * \param newpair array of obj/attribute pairs to copy to.
+ * \param move if 1, move rather than copy.
+ * \param noflagcopy if 1, don't copy associated flags.
+ */
 void
-do_cpattr(player, oldpair, newpair, move, noflagcopy)
-    dbref player;
-    char *oldpair;
-    char **newpair;
-    int move;
-    int noflagcopy;
+do_cpattr(dbref player, char *oldpair, char **newpair, int move, int noflagcopy)
 {
-  /* the command is of the format:
-   * @cpattr oldobj/oldattr = newobj1/newattr1, newobj2/newattr2, etc.
-   * If move = 1, it's @mvattr, erase original attrib.
-   */
-
   dbref oldobj, newobj;
   char tbuf1[BUFFER_LEN], tbuf2[BUFFER_LEN];
   int i;
@@ -634,7 +697,7 @@ do_cpattr(player, oldpair, newpair, move, noflagcopy)
     return;
   }
   /* we can read it. Copy the value. */
-  text = safe_uncompress(a->value);
+  text = safe_atr_value(a);
 
   /* now we loop through our new object pairs and copy, calling @set. */
   for (i = 1; i < MAX_ARG && (newpair[i] != NULL); i++) {
@@ -674,25 +737,32 @@ do_cpattr(player, oldpair, newpair, move, noflagcopy)
   return;
 }
 
+/** Argument struct for gedit_helper */
+struct gedit_args {
+  enum edit_type target; /**< The type of edit */
+  char *from; /**< What is going to be replaced? */
+  char *to; /**< What it gets replaced with. */
+};
+
 static int
-gedit_helper(player, thing, pattern, a, args)
-    dbref player;
-    dbref thing;
-    char const *pattern __attribute__ ((__unused__));
-    ATTR *a;
-    void *args;
+gedit_helper(dbref player, dbref thing,
+	     dbref parent __attribute__ ((__unused__)),
+	     char const *pattern
+	     __attribute__ ((__unused__)), ATTR *a, void *args)
 {
   int ansi_long_flag = 0;
   const char *r;
   char *s, *val;
   char tbuf1[BUFFER_LEN], tbuf_ansi[BUFFER_LEN];
   char *tbufp, *tbufap;
-  char **argv = (char **) args;
   size_t rlen, vlen;
+  struct gedit_args *gargs;
 
-  val = argv[1];
+  gargs = args;
+
+  val = gargs->from;
   vlen = strlen(val);
-  r = argv[2] ? argv[2] : "";
+  r = gargs->to ? gargs->to : "";
   rlen = strlen(r);
 
   tbufp = tbuf1;
@@ -706,7 +776,7 @@ gedit_helper(player, thing, pattern, a, args)
     notify(player, T("You need to control an attribute to edit it."));
     return 0;
   }
-  s = (char *) uncompress(a->value);	/* warning: pointer to static buffer */
+  s = (char *) atr_value(a);	/* warning: pointer to static buffer */
 
   if (vlen == 1 && *val == '$') {
     /* append */
@@ -734,25 +804,27 @@ gedit_helper(player, thing, pattern, a, args)
 
     /* Add one at the start */
     if (!safe_strl(r, rlen, tbuf1, &tbufp)) {
-      for (last = 0; last < haystack->len; last++) {
-	/* Add the next character */
-	if (safe_ansi_string(haystack, last, 1, tbuf1, &tbufp)) {
-	  too_long = 1;
-	  break;
-	}
-	if (!ansi_long_flag) {
-	  if (safe_ansi_string(haystack, last, 1, tbuf_ansi, &tbufap))
-	    ansi_long_flag = 1;
-	}
-	/* Copy in r */
-	if (safe_strl(r, rlen, tbuf1, &tbufp)) {
-	  too_long = 1;
-	  break;
-	}
-	if (!ansi_long_flag) {
-	  if (safe_format(tbuf_ansi, &tbufap, "%s%s%s", ANSI_HILITE, r,
-			  ANSI_NORMAL))
-	    ansi_long_flag = 1;
+      if (gargs->target != EDIT_FIRST) {
+	for (last = 0; last < haystack->len; last++) {
+	  /* Add the next character */
+	  if (safe_ansi_string(haystack, last, 1, tbuf1, &tbufp)) {
+	    too_long = 1;
+	    break;
+	  }
+	  if (!ansi_long_flag) {
+	    if (safe_ansi_string(haystack, last, 1, tbuf_ansi, &tbufap))
+	      ansi_long_flag = 1;
+	  }
+	  /* Copy in r */
+	  if (safe_strl(r, rlen, tbuf1, &tbufp)) {
+	    too_long = 1;
+	    break;
+	  }
+	  if (!ansi_long_flag) {
+	    if (safe_format(tbuf_ansi, &tbufap, "%s%s%s", ANSI_HILITE, r,
+			    ANSI_NORMAL))
+	      ansi_long_flag = 1;
+	  }
 	}
       }
     }
@@ -790,6 +862,8 @@ gedit_helper(player, thing, pattern, a, args)
 	  ansi_long_flag = 1;
       }
       last = p - haystack->text + vlen;
+      if (gargs->target == EDIT_FIRST)
+	break;
     }
     if (last < haystack->len && !too_long) {
       safe_ansi_string(haystack, last, haystack->len, tbuf1, &tbufp);
@@ -805,7 +879,7 @@ gedit_helper(player, thing, pattern, a, args)
   *tbufap = '\0';
 
   if (do_set_atr(thing, AL_NAME(a), tbuf1, player, 0) &&
-      !Quiet(player) && !Quiet(thing)) {
+      !AreQuiet(player, thing)) {
     if (!ansi_long_flag && ShowAnsi(player))
       notify_format(player, "%s - Set: %s", AL_NAME(a), tbuf_ansi);
     else
@@ -815,15 +889,21 @@ gedit_helper(player, thing, pattern, a, args)
   return 1;
 }
 
+/** Edit an attribute.
+ * \verbatim
+ * This implements @edit obj/attribute = {search}, {replace}
+ * \endverbatim
+ * \param player the enactor.
+ * \param it the object/attribute pair.
+ * \param argv array containing the search and replace strings.
+ */
 void
-do_gedit(player, it, argv)
-    dbref player;
-    char *it;
-    char **argv;
+do_gedit(dbref player, char *it, char **argv, enum edit_type target)
 {
   dbref thing;
   char tbuf1[BUFFER_LEN];
   char *q;
+  struct gedit_args args;
 
   if (!(it && *it)) {
     notify(player, T("I need to know what you want to edit."));
@@ -847,16 +927,23 @@ do_gedit(player, it, argv)
     notify(player, T("Nothing to do."));
     return;
   }
-  if (!atr_iter_get(player, thing, q, gedit_helper, argv))
+  args.from = argv[1];
+  args.to = argv[2];
+  args.target = target;
+  if (!atr_iter_get(player, thing, q, 0, gedit_helper, &args))
     notify(player, T("No matching attributes."));
 }
 
-
+/** Trigger an attribute.
+ * \verbatim
+ * This implements @trigger obj/attribute = list-of-arguments.
+ * \endverbatim
+ * \param player the enactor.
+ * \param object the object/attribute pair.
+ * \param argv array of arguments.
+ */
 void
-do_trigger(player, object, argv)
-    dbref player;
-    char *object;
-    char **argv;
+do_trigger(dbref player, char *object, char **argv)
 {
   dbref thing;
   int a;
@@ -886,10 +973,10 @@ do_trigger(player, object, argv)
   }
   /* trigger modifies the stack */
   for (a = 0; a < 10; a++)
-    wnxt[a] = argv[a + 1];
+    global_eval_context.wnxt[a] = argv[a + 1];
 
   if (charge_action(player, thing, upcasestr(s))) {
-    if (!Quiet(player) && !Quiet(thing))
+    if (!AreQuiet(player, thing))
       notify_format(player, "%s - Triggered.", Name(thing));
   } else {
     notify(player, T("No such attribute."));
@@ -897,12 +984,13 @@ do_trigger(player, object, argv)
 }
 
 
-/* for lack of a better place, the use code is here */
-
+/** The use command.
+ * It's here for lack of a better place.
+ * \param player the enactor.
+ * \param what name of the object to use.
+ */
 void
-do_use(player, what)
-    dbref player;
-    const char *what;
+do_use(dbref player, const char *what)
 {
   dbref thing;
 
@@ -912,19 +1000,23 @@ do_use(player, what)
        noisy_match_result(player, what, TYPE_THING,
 			  MAT_NEAR_THINGS)) != NOTHING) {
     if (!eval_lock(player, thing, Use_Lock)) {
-      did_it(player, thing, "UFAIL", T("Pemission denied."), "OUFAIL", NULL,
-	     "AUFAIL", NOTHING);
+      fail_lock(player, thing, Use_Lock, T("Permission denied."), NOTHING);
       return;
     } else
       did_it(player, thing, "USE", "Used.", "OUSE", NULL, "AUSE", NOTHING);
   }
 }
 
+/** Parent an object to another.
+ * \verbatim
+ * This implements @parent.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name the name of the child object.
+ * \param parent_name the name of the new parent object.
+ */
 void
-do_parent(player, name, parent_name)
-    dbref player;
-    char *name;
-    char *parent_name;
+do_parent(dbref player, char *name, char *parent_name)
 {
   dbref thing;
   dbref parent;
@@ -978,33 +1070,35 @@ do_parent(player, name, parent_name)
   }
   /* everything is okay, do the change */
   Parent(thing) = parent;
-  notify(player, T("Parent changed."));
+  if (!AreQuiet(player, thing))
+    notify(player, T("Parent changed."));
 }
 
 static int
-wipe_helper(player, thing, pattern, atr, args)
-    dbref player;
-    dbref thing;
-    char const *pattern;
-    ATTR *atr;
-    void *args __attribute__ ((__unused__));
+wipe_helper(dbref player, dbref thing,
+	    dbref parent __attribute__ ((__unused__)),
+	    char const *pattern,
+	    ATTR *atr, void *args __attribute__ ((__unused__)))
 {
   /* for added security, only God can modify wiz-only-modifiable
    * attributes using this command and wildcards.  Wiping a specific
    * attr still works, though.
    */
-  if (wildcard(pattern) && (AL_FLAGS(atr) & AF_WIZARD) && !God(player))
+  if (wildcard(pattern) && AF_Wizard(atr) && !God(player))
     return 0;
   return do_set_atr(thing, AL_NAME(atr), NULL, player, 0) == 1;
 }
 
+/** Clear an attribute.
+ * \verbatim
+ * This implements @wipe.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name the object/attribute-pattern to wipe.
+ */
 void
-do_wipe(player, name)
-    dbref player;
-    char *name;
+do_wipe(dbref player, char *name)
 {
-  /* obliterate attributes from an object */
-
   dbref thing;
   char *pattern;
 
@@ -1026,19 +1120,13 @@ do_wipe(player, name)
     notify(player, T("That object is protected."));
     return;
   }
-  if (!atr_iter_get(player, thing, pattern, wipe_helper, NULL))
+
+  we_are_wiping = 1;
+
+  if (!atr_iter_get(player, thing, pattern, 0, wipe_helper, NULL))
     notify(player, T("No matching attributes."));
   else
     notify(player, T("Attributes wiped."));
-}
 
-COMMAND (cmd_with) {
-  dbref what;
-
-  what = noisy_match_result(player, arg_left, NOTYPE, MAT_NEARBY);
-  if (!GoodObject(what))
-    return;
-
-  if (!atr_comm_match(what, player, '$', ':', arg_right, 0, NULL, NULL))
-    notify(player, T("No matching command."));
+  we_are_wiping = 0;
 }
