@@ -1,13 +1,17 @@
+/**
+ * \file speech.c
+ *
+ * \brief Speech-related commands in PennMUSH.
+ *
+ *
+ */
 /* speech.c */
 
 #include "copyrite.h"
 #include "config.h"
-
-/* Commands which involve speaking */
 #include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
-
 #include "conf.h"
 #include "ansi.h"
 #include "externs.h"
@@ -21,23 +25,19 @@
 #include "parse.h"
 #include "game.h"
 #include "pcre.h"
-#include "memcheck.h"
 #include "confmagic.h"
 
-static void oemit_notify_except(dbref loc, dbref exc1, dbref exc2,
-				const char *msg);
-const char *reconstruct_message(char *arg1, char *arg2);
 int okay_pemit(dbref player, dbref target);
 static dbref speech_loc(dbref thing);
 void propagate_sound(dbref thing, const char *msg);
 static void do_audible_stuff(dbref loc, dbref *excs, int numexcs,
 			     const char *msg);
-void do_one_remit(dbref player, const char *target, const char *msg, int noisy);
+static void do_one_remit(dbref player, const char *target, const char *msg,
+			 int flags);
 dbref na_zemit(dbref current, void *data);
-extern int WIN32_CDECL i_comp(const void *s1, const void *s2);
-static const char *spname(dbref thing);
+extern int i_comp(const void *s1, const void *s2);
 
-static const char *
+const char *
 spname(dbref thing)
 {
   /* if FULL_INVIS is defined, dark wizards and dark objects will be
@@ -54,17 +54,27 @@ spname(dbref thing)
   }
 }
 
+/** Can player pemit to target?
+ * You can pemit if you're pemit_all, if you're pemitting to yourself,
+ * if you're pemitting to a non-player, or if you pass target's
+ * pagelock and target isn't HAVEN.
+ * \param player dbref attempting to pemit.
+ * \param target target dbref to pemit to.
+ * \retval 1 player may pemit to target.
+ * \retval 0 player may not pemit to target.
+ */
 int
 okay_pemit(dbref player, dbref target)
 {
   if (Pemit_All(player))
     return 1;
-  if ((player != target) &&
-      (IsPlayer(target) &&
-       (Haven(target) || !eval_lock(player, target, Page_Lock))))
+  if (IsPlayer(target) && Haven(target))
     return 0;
-  else
-    return 1;
+  if (!eval_lock(player, target, Page_Lock)) {
+    fail_lock(player, target, Page_Lock, NULL, NOTHING);
+    return 0;
+  }
+  return 1;
 }
 
 static dbref
@@ -85,6 +95,11 @@ speech_loc(dbref thing)
   }
 }
 
+/** The teach command.
+ * \param player the enactor.
+ * \param cause the object causing the command to run.
+ * \param tbuf1 the command being taught.
+ */
 void
 do_teach(dbref player, dbref cause, const char *tbuf1)
 {
@@ -96,8 +111,8 @@ do_teach(dbref player, dbref cause, const char *tbuf1)
   if (!GoodObject(loc))
     return;
 
-  if (!Hasprivs(player) && !eval_lock(player, loc, Speech_Lock)) {
-    notify(player, T("You may not speak here!"));
+  if (!eval_lock(player, loc, Speech_Lock)) {
+    fail_lock(player, loc, Speech_Lock, T("You may not speak here!"), NOTHING);
     return;
   }
 
@@ -116,13 +131,17 @@ do_teach(dbref player, dbref cause, const char *tbuf1)
   recurse = 1;			/* Protect use from recursive teach */
   notify_except(Contents(loc), NOTHING,
 		tprintf(T("%s types --> %s%s%s"), spname(player),
-			ANSI_HILITE, tbuf1, ANSI_NORMAL));
+			ANSI_HILITE, tbuf1, ANSI_NORMAL), NA_INTER_HEAR);
   command = mush_strdup(tbuf1, "string");	/* process_command is destructive */
   process_command(player, command, cause, 1);
   mush_free(command, "string");
   recurse = 0;			/* Ok, we can be called again safely */
 }
 
+/** The say command.
+ * \param player the enactor.
+ * \param tbuf1 the message to say.
+ */
 void
 do_say(dbref player, const char *tbuf1)
 {
@@ -132,8 +151,8 @@ do_say(dbref player, const char *tbuf1)
   if (!GoodObject(loc))
     return;
 
-  if (!Hasprivs(player) && !eval_lock(player, loc, Speech_Lock)) {
-    notify(player, T("You may not speak here!"));
+  if (!eval_lock(player, loc, Speech_Lock)) {
+    fail_lock(player, loc, Speech_Lock, T("You may not speak here!"), NOTHING);
     return;
   }
 
@@ -143,64 +162,34 @@ do_say(dbref player, const char *tbuf1)
   /* notify everybody */
   notify_format(player, T("You say, \"%s\""), tbuf1);
   notify_except(Contents(loc), player,
-		tprintf(T("%s says, \"%s\""), spname(player), tbuf1));
+		tprintf(T("%s says, \"%s\""), spname(player), tbuf1),
+		NA_INTER_HEAR);
 }
 
+/** The oemit(/list) command.
+ * \verbatim
+ * This implements @oemit and @oemit/list.
+ * \endverbatim
+ * \param player the enactor.
+ * \param list the list of dbrefs to oemit from the emit.
+ * \param message the message to emit.
+ * \param flags PEMIT_* flags.
+ */
 void
-do_oemit(dbref player, const char *arg1, const char *arg2)
-{
-  dbref who;
-  dbref loc;
-  char *temp;
-
-  if ((temp = strchr(arg1, '/')) == NULL) {
-    who = match_result(player, arg1, NOTYPE, MAT_OBJECTS);
-    if (!GoodObject(who)) {
-      notify(player, T("I can't find that object."));
-      return;
-    }
-    loc = Location(who);
-    if (!GoodObject(loc)) {
-      notify(player, T("You can only @oemit to neighboring objects."));
-      return;
-    }
-  } else {
-    *temp++ = '\0';
-
-    /* first find the room */
-    loc = noisy_match_result(player, arg1, NOTYPE, MAT_EVERYTHING);
-    if (!GoodObject(loc)) {
-      notify(player, T("I can't find that room."));
-      return;
-    }
-    /* then find the person to omit within that room */
-    who = match_result(loc, temp, NOTYPE, MAT_POSSESSION | MAT_ABSOLUTE);
-  }
-
-  if (!Hasprivs(player) && !eval_lock(player, loc, Speech_Lock)) {
-    notify(player, T("You may not speak there!"));
-    return;
-  }
-
-  if (!GoodObject(who))
-    oemit_notify_except(loc, NOTHING, loc, arg2);
-  else
-    oemit_notify_except(loc, who, loc, arg2);
-}
-
-void
-do_oemit_list(dbref player, char *list, const char *message)
+do_oemit_list(dbref player, char *list, const char *message, int flags)
 {
   char *temp, *p, *s;
   dbref who;
   dbref pass[12], locs[10];
   int i, oneloc = 0;
+  int na_flags = NA_INTER_HEAR;
 
   /* If no message, further processing is pointless.
    * If no list, they should have used @remit. */
   if (!message || !*message || !list || !*list)
     return;
 
+  orator = player;
   pass[0] = 0;
   /* Find out what room to do this in. If they supplied a db# before
    * the '/', then oemit to anyone in the room who's not on list.
@@ -225,8 +214,9 @@ do_oemit_list(dbref player, char *list, const char *message)
       return;
     }
 
-    if (!Hasprivs(player) && !eval_lock(player, pass[1], Speech_Lock)) {
-      notify(player, T("You may not speak there!"));
+    if (!eval_lock(player, pass[1], Speech_Lock)) {
+      fail_lock(player, pass[1], Speech_Lock, T("You may not speak there!"),
+		NOTHING);
       return;
     }
 
@@ -249,7 +239,7 @@ do_oemit_list(dbref player, char *list, const char *message)
      * locs[0..10] are corresponding dbrefs of locations
      */
     if (GoodObject(who) && GoodObject(Location(who))
-	&& (Hasprivs(player) || eval_lock(player, Location(who), Speech_Lock))
+	&& eval_lock(player, Location(who), Speech_Lock)
       ) {
       if (pass[0] < 10) {
 	locs[pass[0]] = Location(who);
@@ -262,31 +252,29 @@ do_oemit_list(dbref player, char *list, const char *message)
     }
   }
 
-  if (oneloc) {
-    /* Only one room to oemit to. */
-    notify_anything(orator, na_exceptN, pass, ns_esnotify, 0, message);
+  /* Sort the list of rooms to oemit to so we don't oemit to the same           
+   * room twice */
+  qsort((void *) locs, pass[0], sizeof(locs[0]), i_comp);
+
+  if (flags & PEMIT_SPOOF)
+    na_flags |= NA_SPOOF;
+  for (i = 0; i < pass[0]; i++) {
+    if (i != 0 && locs[i] == locs[i - 1])
+      continue;
+    pass[1] = locs[i];
+    notify_anything_loc(orator, na_exceptN, pass, ns_esnotify, na_flags,
+			message, locs[i]);
     do_audible_stuff(pass[1], &pass[2], pass[0], message);
-    return;
-  } else if (pass[0] == 0) {
-    return;			/* none of the dbrefs they gave were valid */
-  } else {
-    /* Multiple rooms to oemit to. Sort 'em so we don't oemit to the
-     * same room twice
-     */
-    qsort((void *) locs, pass[0], sizeof(locs[0]), i_comp);
-    pass[1] = locs[0];
-    notify_anything(orator, na_exceptN, pass, ns_esnotify, 0, message);
-    do_audible_stuff(pass[1], &pass[2], pass[0], message);
-    for (i = 1; i < pass[0]; i++) {
-      if (locs[i] != locs[i - 1]) {
-	pass[1] = locs[i];
-	notify_anything(orator, na_exceptN, pass, ns_esnotify, 0, message);
-	do_audible_stuff(pass[1], &pass[2], pass[0], message);
-      }
-    }
   }
 }
 
+
+/** The whisper command.
+ * \param player the enactor.
+ * \param arg1 name of the object to whisper to.
+ * \param arg2 message to whisper.
+ * \param noisy if 1, others overhear that a whisper has occurred.
+ */
 void
 do_whisper(dbref player, const char *arg1, const char *arg2, int noisy)
 {
@@ -312,7 +300,7 @@ do_whisper(dbref player, const char *arg1, const char *arg2, int noisy)
   }
   tp = tbuf = (char *) mush_malloc(BUFFER_LEN, "string");
   if (!tbuf)
-    panic("Unable to allocate memory in do_whisper_list");
+    mush_panic("Unable to allocate memory in do_whisper_list");
 
   overheard = 0;
   head = arg1;
@@ -337,9 +325,11 @@ do_whisper(dbref player, const char *arg1, const char *arg2, int noisy)
     current = next_in_list(start);
     who = match_result(player, current, TYPE_PLAYER, MAT_NEAR_THINGS |
 		       MAT_CONTAINER);
-    if (!GoodObject(who)) {
+    if (!GoodObject(who) || !can_interact(player, who, INTERACT_HEAR)) {
       safe_chr(' ', tbuf, &tp);
       safe_str_space(current, tbuf, &tp);
+      if (GoodObject(who))
+	notify_format(player, T("%s can't hear you."), Name(who));
     } else {
       /* A good whisper */
       good[gcount++] = who;
@@ -409,26 +399,30 @@ do_whisper(dbref player, const char *arg1, const char *arg2, int noisy)
   mush_free((Malloc_t) tbuf, "string");
 }
 
+/** Send a message to a list of dbrefs. To avoid repeated generation
+ * of the NOSPOOF string, we set it up the first time we encounter
+ * something Nospoof, and then check for it thereafter.
+ * The list is destructively modified.
+ * \param player the enactor.
+ * \param list the list of players to pemit to, destructively modified.
+ * \param message the message to pemit.
+ * \param flags PEMIT_* flags
+ */
 void
-do_pemit_list(dbref player, char *list, const char *message, int nospoof)
+do_pemit_list(dbref player, char *list, const char *message, int flags)
 {
-  /* Send a message to a list of dbrefs. To avoid repeated generation
-   * of the NOSPOOF string, we set it up the first time we encounter
-   * something Nospoof, and then check for it thereafter.
-   * The list is destructively modified.
-   */
-
   char *bp, *p;
   char *nsbuf, *nspbuf;
   char *l;
   dbref who;
+  int nospoof;
 
   /* If no list or no message, further processing is pointless. */
   if (!message || !*message || !list || !*list)
     return;
 
   nspbuf = nsbuf = NULL;
-
+  nospoof = (flags & PEMIT_SPOOF) ? 0 : 1;
   list[BUFFER_LEN - 1] = '\0';
   l = trim_space_sep(list, ' ');
 
@@ -469,11 +463,20 @@ do_pemit_list(dbref player, char *list, const char *message, int nospoof)
 
 }
 
+/** Send a message to an object.
+ * \param player the enactor.
+ * \param arg1 the name of the object to pemit to.
+ * \param arg2 the message to pemit.
+ * \param flags PEMIT_* flags.
+ */
 void
-do_pemit(dbref player, const char *arg1, const char *arg2, int silent,
-	 int nospoof)
+do_pemit(dbref player, const char *arg1, const char *arg2, int flags)
 {
   dbref who;
+  int silent, nospoof;
+
+  silent = (flags & PEMIT_SILENT) ? 1 : 0;
+  nospoof = (flags & PEMIT_SPOOF) ? 0 : 1;
 
   switch (who = match_result(player, arg1, NOTYPE,
 			     MAT_OBJECTS | MAT_HERE | MAT_CONTAINER)) {
@@ -510,6 +513,11 @@ do_pemit(dbref player, const char *arg1, const char *arg2, int silent,
   }
 }
 
+/** The pose and semipose command.
+ * \param player the enactor.
+ * \param tbuf1 the message to pose.
+ * \param space if 1, put a space between name and pose; if 0, don't (semipose)
+ */
 void
 do_pose(dbref player, const char *tbuf1, int space)
 {
@@ -519,89 +527,97 @@ do_pose(dbref player, const char *tbuf1, int space)
   if (!GoodObject(loc))
     return;
 
-  if (!Hasprivs(player) && !eval_lock(player, loc, Speech_Lock)) {
-    notify(player, T("You may not speak here!"));
+  if (!eval_lock(player, loc, Speech_Lock)) {
+    fail_lock(player, loc, Speech_Lock, T("You may not speak here!"), NOTHING);
     return;
   }
 
   /* notify everybody */
   if (!space)
     notify_except(Contents(loc), NOTHING,
-		  tprintf("%s %s", spname(player), tbuf1));
+		  tprintf("%s %s", spname(player), tbuf1), NA_INTER_HEAR);
   else
     notify_except(Contents(loc), NOTHING,
-		  tprintf("%s%s", spname(player), tbuf1));
+		  tprintf("%s%s", spname(player), tbuf1), NA_INTER_HEAR);
 }
 
+/** The *wall commands.
+ * \param player the enactor.
+ * \param message message to broadcast.
+ * \param target type of broadcast (all, royalty, wizard)
+ * \param emit if 1, this is a wallemit.
+ */
 void
-do_wall(dbref player, const char *message, int privs, int key)
+do_wall(dbref player, const char *message, enum wall_type target, int emit)
 {
-  /* privs is 0 for wizard wizwall, 1 for royalty-wizard wizwall,
-   * 2 is for general wall
-   */
-  const char *gap, *prefix;
-  int mask;
+  const char *gap = "", *prefix;
+  const char *mask;
+  int pose = 0;
 
   /* Only @wall is available to those with the announce power.
    * Only @rwall is available to royalty.
    */
   if (!(Wizard(player) ||
-	((privs == 2) && Can_Announce(player)) ||
-	((privs == 1) && Royalty(player)))) {
+	((target == WALL_ALL) && Can_Announce(player)) ||
+	((target == WALL_RW) && Royalty(player)))) {
     notify(player, T("Posing as a wizard could be hazardous to your health."));
     return;
   }
   /* put together the message and figure out what type it is */
-  gap = " ";
-  switch (*message) {
-  case SAY_TOKEN:
-    key = 1;
-    if (CHAT_STRIP_QUOTE)
+  if (!emit) {
+    gap = " ";
+    switch (*message) {
+    case SAY_TOKEN:
+      if (CHAT_STRIP_QUOTE)
+	message++;
+      break;
+    case SEMI_POSE_TOKEN:
+      gap = "";
+    case POSE_TOKEN:
+      pose = 1;
       message++;
-    break;
-  case SEMI_POSE_TOKEN:
-    gap = "";
-  case POSE_TOKEN:
-    key = 2;
-    message++;
-    break;
+      break;
+    }
   }
 
   if (!*message) {
     notify(player, T("What did you want to say?"));
     return;
   }
-  if (privs == 0) {
+  if (target == WALL_WIZ) {
     /* to wizards only */
-    mask = WIZARD;
+    mask = "WIZARD";
     prefix = WIZWALL_PREFIX;
-  } else if (privs == 1) {
+  } else if (target == WALL_RW) {
     /* to wizards and royalty */
-    mask = WIZARD | ROYALTY;
+    mask = "WIZARD ROYALTY";
     prefix = RWALL_PREFIX;
   } else {
     /* to everyone */
-    mask = 0;
+    mask = NULL;
     prefix = WALL_PREFIX;
   }
 
   /* broadcast the message */
-  if (key == 2)
+  if (pose)
     flag_broadcast(mask, 0, "%s %s%s%s", prefix, Name(player), gap, message);
-  else if (key == 3)
+  else if (emit)
     flag_broadcast(mask, 0, "%s [%s]: %s", prefix, Name(player), message);
   else
     flag_broadcast(mask, 0,
 		   "%s %s %s, \"%s\"", prefix, Name(player),
-		   (privs != 2) ? "says" : "shouts", message);
-
-  /* log it if necessary */
-  if ((privs == 2) || (options.log_walls && (privs == 0)))
-    do_log(LT_WIZ, player, NOTHING, "(MSG/%s) %s%s%s",
-	   privs ? "all" : "wiz",
-	   (key == 2) ? Name(player) : "", (key == 2) ? gap : "", message);
+		   target == WALL_ALL ? "shouts" : "says", message);
 }
 
+/** The page command.
+ * \param player the enactor.
+ * \param arg1 the list of players to page.
+ * \param arg2 the message to page.
+ * \param cause the object that caused the command to run.
+ * \param noeval if 1, page/noeval.
+ * \param multipage if 1, a page/list; if 0, a page/blind.
+ * \param override if 1, page/override.
+ */
 void
 do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
 	int noeval, int multipage, int override)
@@ -621,11 +637,13 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
   char *current;
   int i;
   int repage = 0;
+  int fails_lock;
+  int is_haven;
   ATTR *a;
 
   tp2 = tbuf2 = (char *) mush_malloc(BUFFER_LEN, "string");
   if (!tbuf2)
-    panic("Unable to allocate memory in do_page");
+    mush_panic("Unable to allocate memory in do_page");
 
   if (arg2 && *arg2 && *arg1) {
     /* page to=msg. Always evaluate to, maybe evaluate msg */
@@ -645,7 +663,7 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
   }
   if (repage) {
     a = atr_get_noparent(player, "LASTPAGED");
-    if (!a || !*((hp = head = safe_uncompress(a->value)))) {
+    if (!a || !*((hp = head = safe_atr_value(a)))) {
       notify(player, T("You haven't paged anyone since connecting."));
       mush_free((Malloc_t) tbuf2, "string");
       return;
@@ -661,7 +679,7 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
 
   tp = tbuf = (char *) mush_malloc(BUFFER_LEN, "string");
   if (!tbuf)
-    panic("Unable to allocate memory in do_page");
+    mush_panic("Unable to allocate memory in do_page");
 
   if (override && !Pemit_All(player)) {
     notify(player, "Try again after you get the pemit_all power.");
@@ -685,40 +703,44 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
 		    T("I'm not sure who you want to page with: %s"), current);
       safe_chr(' ', tbuf, &tp);
       safe_str_space(current, tbuf, &tp);
-    } else if (!Connected(target) || (Dark(target) && !override &&
-				      (Haven(target)
-				       ||
-				       (!eval_lock
-					(player, target, Page_Lock))))) {
-      /* A player isn't connected if they aren't connected, or if
-       * they're DARK and HAVEN, or DARK and the pagelock fails. */
-      page_return(player, target, "Away", "AWAY",
-		  tprintf(T("%s is not connected."), Name(target)));
-      safe_chr(' ', tbuf, &tp);
-      safe_str_space(current, tbuf, &tp);
-    } else if (Haven(target) && !override) {
-      page_return(player, target, "Haven", "HAVEN",
-		  tprintf(T("%s is not accepting any pages."), Name(target)));
-      safe_chr(' ', tbuf, &tp);
-      safe_str_space(Name(target), tbuf, &tp);
-    } else if (!eval_lock(player, target, Page_Lock) && !override) {
-      page_return(player, target, "Haven", "HAVEN",
-		  tprintf(T("%s is not accepting your pages."), Name(target)));
-      safe_chr(' ', tbuf, &tp);
-      safe_str_space(Name(target), tbuf, &tp);
     } else {
-      /* This is a good page */
-      good[gcount] = target;
-      gcount++;
+      fails_lock = !(override || eval_lock(player, target, Page_Lock));
+      is_haven = !override && Haven(target);
+      if (!Connected(target) || (Dark(target) && (is_haven || fails_lock))) {
+	/* A player isn't connected if they aren't connected, or if
+	 * they're DARK and HAVEN, or DARK and the pagelock fails. */
+	page_return(player, target, "Away", "AWAY",
+		    tprintf(T("%s is not connected."), Name(target)));
+	if (fails_lock)
+	  fail_lock(player, target, Page_Lock, NULL, NOTHING);
+	safe_chr(' ', tbuf, &tp);
+	safe_str_space(current, tbuf, &tp);
+      } else if (is_haven) {
+	page_return(player, target, "Haven", "HAVEN",
+		    tprintf(T("%s is not accepting any pages."), Name(target)));
+	safe_chr(' ', tbuf, &tp);
+	safe_str_space(Name(target), tbuf, &tp);
+      } else if (fails_lock) {
+	page_return(player, target, "Haven", "HAVEN",
+		    tprintf(T("%s is not accepting your pages."),
+			    Name(target)));
+	fail_lock(player, target, Page_Lock, NULL, NOTHING);
+	safe_chr(' ', tbuf, &tp);
+	safe_str_space(Name(target), tbuf, &tp);
+      } else {
+	/* This is a good page */
+	good[gcount] = target;
+	gcount++;
+      }
     }
   }
 
   /* Reset tbuf2 to use later */
   tp2 = tbuf2;
 
-/* We now have an array of good[] dbrefs, a gcount of the good ones,
- * and a tbuf with bad ones.
- */
+  /* We now have an array of good[] dbrefs, a gcount of the good ones,
+   * and a tbuf with bad ones.
+   */
 
   /* We don't know what the heck's going on here, but we're not paging
    * anyone, this looks like a spam attack. */
@@ -767,7 +789,7 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
   else {
     mb = msgbuf = (char *) mush_malloc(BUFFER_LEN, "string");
     if (!msgbuf)
-      panic("Unable to allocate memory in do_page");
+      mush_panic("Unable to allocate memory in do_page");
 
     process_expression(msgbuf, &mb, &message, player, cause, cause,
 		       PE_DEFAULT, PT_DEFAULT, NULL);
@@ -779,7 +801,7 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
     notify(player, T("You are set HAVEN and cannot receive pages."));
 
   /* Figure out what kind of message */
-  wenv[0] = (char *) message;
+  global_eval_context.wenv[0] = (char *) message;
   gap = " ";
   switch (*message) {
   case SEMI_POSE_TOKEN:
@@ -828,7 +850,7 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
 
   /* Figure out the 'name' of the player */
   if (PAGE_ALIASES && (a = atr_get_noparent(player, "ALIAS")))
-    current = tprintf("%s (%s)", Name(player), uncompress(a->value));
+    current = tprintf("%s (%s)", Name(player), atr_value(a));
   else
     current = (char *) Name(player);
 
@@ -877,34 +899,17 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
     free((Malloc_t) hp);
 }
 
-char *
-ns_esnotify(dbref speaker, na_lookup func __attribute__ ((__unused__)),
-	    void *fdata __attribute__ ((__unused__)), int para)
-{
-  char *dest, *bp;
-  bp = dest = mush_malloc(BUFFER_LEN, "string");
-
-  if (!GoodObject(speaker))
-    *dest = '\0';
-  else if (para) {
-    if (speaker == Owner(speaker))
-      safe_format(dest, &bp, "[%s(#%d)] ", Name(speaker), speaker);
-    else
-      safe_format(dest, &bp, "[%s(#%d)'s %s(#%d)] ", Name(Owner(speaker)),
-		  Owner(speaker), Name(speaker), speaker);
-  } else
-    safe_format(dest, &bp, "[%s:] ", spname(speaker));
-  *bp = '\0';
-  return dest;
-}
-
 
+/** Does a message match a filter pattern on an object?
+ * \param thing object with the filter.
+ * \param msg message to match.
+ * \param flag if 0, filter; if 1, infilter.
+ * \retval 1 message matches filter.
+ * \retval 0 message does not match filter.
+ */
 int
 filter_found(dbref thing, const char *msg, int flag)
-				/* 0 for @filter, 1 for @infilter */
 {
-  /* check to see if the message matches the filter pattern on thing */
-
   char *filter;
   ATTR *a;
   char *p, *bp;
@@ -912,7 +917,6 @@ filter_found(dbref thing, const char *msg, int flag)
 				 * by failing to free the storage
 				 * allocated by safe_uncompress
 				 */
-
   int i;
   int matched = 0;
 
@@ -923,7 +927,7 @@ filter_found(dbref thing, const char *msg, int flag)
   if (!a)
     return matched;
 
-  filter = safe_uncompress(a->value);
+  filter = safe_atr_value(a);
   temp = filter;
 
   for (i = 0; (i < MAX_ARG) && !matched; i++) {
@@ -936,17 +940,21 @@ filter_found(dbref thing, const char *msg, int flag)
       break;
     if (*p == '\0')		/* Empty filter */
       continue;
-    if (AL_FLAGS(a) & AF_REGEXP)
-      matched = quick_regexp_match(p, msg, AL_FLAGS(a) & AF_CASE);
+    if (AF_Regexp(a))
+      matched = quick_regexp_match(p, msg, AF_Case(a));
     else
-      matched = local_wild_match_case(p, msg, AL_FLAGS(a) & AF_CASE);
+      matched = local_wild_match_case(p, msg, AF_Case(a));
   }
 
   free((Malloc_t) temp);
   return matched;
 }
 
-/* Copies thing/PREFIX + msg into tbuf1 */
+/** Copy a message into a buffer, adding an object's PREFIX attribute.
+ * \param thing object with prefix attribute.
+ * \param msg message.
+ * \param tbuf1 destination buffer.
+ */
 void
 make_prefixstr(dbref thing, const char *msg, char *tbuf1)
 {
@@ -966,19 +974,19 @@ make_prefixstr(dbref thing, const char *msg, char *tbuf1)
     safe_str(", ", tbuf1, &bp);
   } else {
     for (j = 0; j < 10; j++) {
-      wsave[j] = wenv[j];
-      wenv[j] = NULL;
+      wsave[j] = global_eval_context.wenv[j];
+      global_eval_context.wenv[j] = NULL;
     }
-    wenv[0] = (char *) msg;
+    global_eval_context.wenv[0] = (char *) msg;
     save_global_regs("prefix_save", preserve);
-    asave = safe_uncompress(a->value);
+    asave = safe_atr_value(a);
     ap = asave;
     process_expression(tbuf1, &bp, &ap, thing, orator, orator,
 		       PE_DEFAULT, PT_DEFAULT, NULL);
     free((Malloc_t) asave);
     restore_global_regs("prefix_save", preserve);
     for (j = 0; j < 10; j++)
-      wenv[j] = wsave[j];
+      global_eval_context.wenv[j] = wsave[j];
     if (bp != tbuf1)
       safe_chr(' ', tbuf1, &bp);
   }
@@ -987,12 +995,14 @@ make_prefixstr(dbref thing, const char *msg, char *tbuf1)
   return;
 }
 
+/** pass a message on, for AUDIBLE, prepending a prefix, unless the
+ * message matches a filter pattern.
+ * \param thing object to check for filter and prefix.
+ * \param msg message to pass.
+ */
 void
 propagate_sound(dbref thing, const char *msg)
 {
-  /* pass a message on, for AUDIBLE, prepending a prefix, unless the
-   * message matches a filter pattern.
-   */
   char tbuf1[BUFFER_LEN];
   dbref loc = Location(thing);
   dbref pass[2];
@@ -1013,11 +1023,12 @@ propagate_sound(dbref thing, const char *msg)
    */
 
   if (IsExit(thing)) {
-    notify_anything(orator, na_next, &Contents(loc), NULL, 0, tbuf1);
+    notify_anything(orator, na_next, &Contents(loc), NULL, NA_INTER_HEAR,
+		    tbuf1);
   } else {
     pass[0] = Contents(loc);
     pass[1] = thing;
-    notify_anything(orator, na_nextbut, pass, NULL, 0, tbuf1);
+    notify_anything(orator, na_nextbut, pass, NULL, NA_INTER_HEAR, tbuf1);
   }
 }
 
@@ -1045,8 +1056,15 @@ do_audible_stuff(dbref loc, dbref *excs, int numexcs, const char *msg)
   }
 }
 
+/** notify_anthing() wrapper to notify everyone in a location except one
+ * object.
+ * \param first object in location to notify.
+ * \param exception dbref of object not to notify, or NOTHING.
+ * \param msg message to send.
+ * \param flags flags to pass to notify_anything().
+ */
 void
-notify_except(dbref first, dbref exception, const char *msg)
+notify_except(dbref first, dbref exception, const char *msg, int flags)
 {
   dbref loc;
   dbref pass[2];
@@ -1062,13 +1080,22 @@ notify_except(dbref first, dbref exception, const char *msg)
 
   pass[0] = loc;
   pass[1] = exception;
-  notify_anything(orator, na_except, pass, &ns_esnotify, 0, msg);
+
+  notify_anything(orator, na_except, pass, ns_esnotify, flags, msg);
 
   do_audible_stuff(loc, &pass[1], 1, msg);
 }
 
+/** notify_anthing() wrapper to notify everyone in a location except two
+ * objects.
+ * \param first object in location to notify.
+ * \param exc1 dbref of one object not to notify, or NOTHING.
+ * \param exc2 dbref of another object not to notify, or NOTHING.
+ * \param msg message to send.
+ * \param flags interaction flags to control type of interaction.
+ */
 void
-notify_except2(dbref first, dbref exc1, dbref exc2, const char *msg)
+notify_except2(dbref first, dbref exc1, dbref exc2, const char *msg, int flags)
 {
   dbref loc;
   dbref pass[3];
@@ -1088,66 +1115,64 @@ notify_except2(dbref first, dbref exc1, dbref exc2, const char *msg)
   pass[1] = exc1;
   pass[2] = exc2;
 
-  notify_anything(orator, na_except2, pass, NULL, 0, msg);
-
-  do_audible_stuff(loc, &pass[1], 2, msg);
-}
-
-static void
-oemit_notify_except(dbref loc, dbref exc1, dbref exc2, const char *msg)
-{
-  dbref pass[3];
-
-  if (!GoodObject(loc))
-    return;
-
-  if (exc1 == NOTHING)
-    exc1 = AMBIGUOUS;
-  if (exc2 == NOTHING)
-    exc2 = AMBIGUOUS;
-
-  pass[0] = loc;
-  pass[1] = exc1;
-  pass[2] = exc2;
-
-  notify_anything(orator, na_except2, pass, ns_esnotify, 0, msg);
+  notify_anything(orator, na_except2, pass, ns_esnotify, flags, msg);
 
   do_audible_stuff(loc, &pass[1], 2, msg);
 }
 
+/** The think command.
+ * \param player the enactor.
+ * \param message the message to think.
+ */
 void
 do_think(dbref player, const char *message)
 {
-  /* privately tell yourself a message */
-
   notify(player, message);
 }
 
 
+/** The emit command.
+ * \verbatim
+ * This implements @emit.
+ * \endverbatim
+ * \param player the enactor.
+ * \param tbuf1 the message to emit.
+ * \param flags bitmask of notification flags.
+ */
 void
-do_emit(dbref player, const char *tbuf1)
+do_emit(dbref player, const char *tbuf1, int flags)
 {
   dbref loc;
+  int na_flags = NA_INTER_HEAR;
 
   loc = speech_loc(player);
   if (!GoodObject(loc))
     return;
 
-  if (!Hasprivs(player) && !eval_lock(player, loc, Speech_Lock)) {
-    notify(player, T("You may not speak here!"));
+  if (!eval_lock(player, loc, Speech_Lock)) {
+    fail_lock(player, loc, Speech_Lock, T("You may not speak here!"), NOTHING);
     return;
   }
 
   /* notify everybody */
-  notify_anything(player, na_loc, &loc, ns_esnotify, 0, tbuf1);
+  if (flags & PEMIT_SPOOF)
+    na_flags |= NA_SPOOF;
+  notify_anything(player, na_loc, &loc, ns_esnotify, na_flags, tbuf1);
 
   do_audible_stuff(loc, NULL, 0, tbuf1);
 }
 
-void
-do_one_remit(dbref player, const char *target, const char *msg, int noisy)
+/** Remit a message to a single room.
+ * \param player the enactor.
+ * \param target string containing dbref of room to remit in.
+ * \param msg message to emit.
+ * \param flags PEMIT_* flags
+ */
+static void
+do_one_remit(dbref player, const char *target, const char *msg, int flags)
 {
   dbref room;
+  int na_flags = NA_INTER_HEAR;
   room = match_result(player, target, NOTYPE, MAT_EVERYTHING);
   if (!GoodObject(room)) {
     notify(player, T("I can't find that."));
@@ -1158,41 +1183,60 @@ do_one_remit(dbref player, const char *target, const char *msg, int noisy)
       notify_format(player,
 		    T("I'm sorry, but %s wishes to be left alone now."),
 		    Name(room));
-    } else if (!Hasprivs(player) && !eval_lock(player, room, Speech_Lock)) {
-      notify(player, T("You may not speak there!"));
+    } else if (!eval_lock(player, room, Speech_Lock)) {
+      fail_lock(player, room, Speech_Lock, T("You may not speak there!"),
+		NOTHING);
     } else {
-      if (noisy && (Location(player) != room)) {
+      if (!(flags & PEMIT_SILENT) && (Location(player) != room)) {
 	const char *rmno;
 	rmno = unparse_object(player, room);
 	notify_format(player, T("You remit, \"%s\" in %s"), msg, rmno);
       }
-      notify_anything(player, na_loc, &room, ns_esnotify, 0, msg);
+      if (flags & PEMIT_SPOOF)
+	na_flags |= NA_SPOOF;
+      notify_anything_loc(player, na_loc, &room, ns_esnotify, na_flags,
+			  msg, room);
       do_audible_stuff(room, NULL, 0, msg);
     }
   }
 }
 
+/** Remit a message
+ * \verbatim
+ * This implements @remit.
+ * \endverbatim
+ * \param player the enactor.
+ * \param arg1 string containing dbref(s) of rooms to remit it.
+ * \param arg2 message to emit.
+ * \param flags for remit.
+ */
 void
 do_remit(dbref player, char *arg1, const char *arg2, int flags)
 {
-  if (flags & 0x2) {
+  if (flags & PEMIT_LIST) {
     /* @remit/list */
     char *current;
     arg1 = trim_space_sep(arg1, ' ');
     while ((current = split_token(&arg1, ' ')) != NULL)
-      do_one_remit(player, current, arg2, flags & 0x1);
+      do_one_remit(player, current, arg2, flags);
   } else {
-    do_one_remit(player, arg1, arg2, flags & 0x1);
+    do_one_remit(player, arg1, arg2, flags);
   }
 }
 
+/** Emit a message to the absolute location of enactor.
+ * \param player the enactor.
+ * \param tbuf1 message to emit.
+ * \param flags bitmask of notification flags.
+ */
 void
-do_lemit(dbref player, const char *tbuf1, int silent)
+do_lemit(dbref player, const char *tbuf1, int flags)
 {
   /* give a message to the "absolute" location of an object */
-
   dbref room;
   int rec = 0;
+  int na_flags = NA_INTER_HEAR;
+  int silent = (flags & PEMIT_SILENT) ? 1 : 0;
 
   /* only players and things may use this command */
   if (!Mobile(player))
@@ -1212,15 +1256,24 @@ do_lemit(dbref player, const char *tbuf1, int silent)
   if (rec > 15) {
     notify(player, T("Too many containers."));
     return;
-  } else if (!Hasprivs(player) && !eval_lock(player, room, Speech_Lock)) {
-    notify(player, T("You may not speak there!"));
+  } else if (!eval_lock(player, room, Speech_Lock)) {
+    fail_lock(player, room, Speech_Lock, T("You may not speak there!"),
+	      NOTHING);
+    return;
   } else {
     if (!silent && (Location(player) != room))
       notify_format(player, T("You lemit: \"%s\""), tbuf1);
-    notify_anything(player, na_loc, &room, ns_esnotify, 0, tbuf1);
+    if (flags & PEMIT_SPOOF)
+      na_flags |= NA_SPOOF;
+    notify_anything(player, na_loc, &room, ns_esnotify, na_flags, tbuf1);
   }
 }
 
+/** notify_anything() function for zone emits.
+ * \param current unused.
+ * \param data array of notify data.
+ * \return last object in zone, or NOTHING.
+ */
 dbref
 na_zemit(dbref current __attribute__ ((__unused__)), void *data)
 {
@@ -1232,8 +1285,7 @@ na_zemit(dbref current __attribute__ ((__unused__)), void *data)
     if (this == NOTHING) {
       for (room = dbrefs[1]; room < db_top; room++) {
 	if (IsRoom(room) && (Zone(room) == dbrefs[2])
-	    && (Hasprivs(dbrefs[3])
-		|| eval_lock(dbrefs[3], room, Speech_Lock))
+	    && eval_lock(dbrefs[3], room, Speech_Lock)
 	  )
 	  break;
       }
@@ -1251,12 +1303,22 @@ na_zemit(dbref current __attribute__ ((__unused__)), void *data)
   return this;
 }
 
+/** The zemit command.
+ * \verbatim
+ * This implements @zemit and @nszemit.
+ * \endverbatim
+ * \param player the enactor.
+ * \param arg1 string containing dbref of ZMO.
+ * \param arg2 message to emit.
+ * \param flags bitmask of notificati flags.
+ */
 void
-do_zemit(dbref player, const char *arg1, const char *arg2)
+do_zemit(dbref player, const char *arg1, const char *arg2, int flags)
 {
   const char *where;
   dbref zone;
   dbref pass[5];
+  int na_flags = NA_INTER_HEAR;
 
   zone = match_result(player, arg1, NOTYPE, MAT_ABSOLUTE);
   if (!GoodObject(zone)) {
@@ -1280,5 +1342,7 @@ do_zemit(dbref player, const char *arg1, const char *arg2)
   pass[2] = zone;
   pass[3] = player;
   pass[4] = player;
-  notify_anything(player, na_zemit, pass, ns_esnotify, 0, arg2);
+  if (flags & PEMIT_SPOOF)
+    na_flags |= NA_SPOOF;
+  notify_anything(player, na_zemit, pass, ns_esnotify, na_flags, arg2);
 }
