@@ -1,7 +1,11 @@
+/**
+ * \file strutil.c
+ *
+ * \brief String utilities for PennMUSH.
+ *
+ *
+ */
 
-/* strutil.c */
-
-/* String utilities */
 #include "config.h"
 
 #include <ctype.h>
@@ -16,21 +20,18 @@
 #include "pueblo.h"
 #include "parse.h"
 #include "externs.h"
-#ifdef MEM_CHECK
-#include "memcheck.h"
-#endif
 #include "mymalloc.h"
+#include "log.h"
 #include "confmagic.h"
 
-#include "log.h"
-
 char *next_token(char *str, char sep);
-static int format_long(long val, char *buff, char **bp, int maxlen);
-char *mush_strndup(const char *src, size_t len, const char *check);
+int format_long(long val, char *buff, char **bp, int maxlen, int base);
+static char *
+mush_strndup(const char *src, size_t len, const char *check)
+  __attribute_malloc__;
 
 /* Duplicate the first len characters of s */
-char *
-mush_strndup(const char *src, size_t len, const char *check)
+    static char *mush_strndup(const char *src, size_t len, const char *check)
 {
   char *copy;
   size_t rlen = strlen(src);
@@ -47,6 +48,13 @@ mush_strndup(const char *src, size_t len, const char *check)
   return copy;
 }
 
+/** Our version of strdup, with memory leak checking.
+ * This should be used in preference to strdup, and in assocation
+ * with mush_free().
+ * \param s string to duplicate.
+ * \param check label for memory checking.
+ * \return newly allocated copy of s.
+ */
 char *
 mush_strdup(const char *s, const char *check __attribute__ ((__unused__)))
 {
@@ -54,13 +62,11 @@ mush_strdup(const char *s, const char *check __attribute__ ((__unused__)))
 
 #ifdef HAS_STRDUP
   x = strdup(s);
-#ifdef MEM_CHECK
   if (x)
     add_check(check);
-#endif
 #else
 
-  Size_t len = strlen(s) + 1;
+  size_t len = strlen(s) + 1;
   x = mush_malloc(len, check);
   if (x)
     memcpy(x, s, len);
@@ -68,7 +74,11 @@ mush_strdup(const char *s, const char *check __attribute__ ((__unused__)))
   return x;
 }
 
-/* Return the string chopped at lim characters */
+/** Return the string chopped at lim characters.
+ * \param str string to chop.
+ * \param lim character at which to chop the string.
+ * \return statically allocated buffer with chopped string.
+ */
 char *
 chopstr(const char *str, size_t lim)
 {
@@ -83,6 +93,13 @@ chopstr(const char *str, size_t lim)
 
 #ifndef HAS_STRCASECMP
 #ifndef WIN32
+/** strcasecmp for systems without it.
+ * \param s1 one string to compare.
+ * \param s2 another string to compare.
+ * \retval -1 s1 is less than s2.
+ * \retval 0 s1 equals s2
+ * \retval 1 s1 is greater than s2.
+ */
 int
 strcasecmp(const char *s1, const char *s2)
 {
@@ -92,8 +109,16 @@ strcasecmp(const char *s1, const char *s2)
   return (DOWNCASE(*s1) - DOWNCASE(*s2));
 }
 
+/** strncasecmp for systems without it.
+ * \param s1 one string to compare.
+ * \param s2 another string to compare.
+ * \param n number of characters to compare.
+ * \retval -1 s1 is less than s2.
+ * \retval 0 s1 equals s2
+ * \retval 1 s1 is greater than s2.
+ */
 int
-strncasecmp(const char *s1, const char *s2, Size_t n)
+strncasecmp(const char *s1, const char *s2, size_t n)
 {
   for (; 0 < n; ++s1, ++s2, --n)
     if (DOWNCASE(*s1) != DOWNCASE(*s2))
@@ -106,8 +131,15 @@ strncasecmp(const char *s1, const char *s2, Size_t n)
 #endif				/* !WIN32 */
 #endif				/* !HAS_STRCASECMP */
 
+/** Does string begin with prefix? 
+ * This comparison is case-insensitive.
+ * \param string to check.
+ * \param prefix to check against.
+ * \retval 1 string begins with prefix.
+ * \retval 0 string does not begin with prefix.
+ */
 int
-string_prefix(const char *string, const char *prefix)
+string_prefix(const char *RESTRICT string, const char *RESTRICT prefix)
 {
   if (!string || !prefix)
     return 0;
@@ -116,7 +148,11 @@ string_prefix(const char *string, const char *prefix)
   return *prefix == '\0';
 }
 
-/* accepts only nonempty matches starting at the beginning of a word */
+/** Match a substring at the start of a word in a string, case-insensitively.
+ * \param src a string of words to match against.
+ * \param sub a prefix to match against the start of a word in string.
+ * \return pointer into src at the matched word, or NULL.
+ */
 const char *
 string_match(const char *src, const char *sub)
 {
@@ -136,9 +172,34 @@ string_match(const char *src, const char *sub)
 	src++;
     }
   }
-  return 0;
+  return NULL;
 }
 
+/** Return an initial-cased version of a string in a static buffer.
+ * \param s string to initial-case.
+ * \return pointer to a static buffer containing the initial-cased version.
+ */
+char *
+strinitial(const char *s)
+{
+  static char buf1[BUFFER_LEN];
+  char *p;
+
+  if (!s || !*s) {
+    buf1[0] = '\0';
+    return buf1;
+  }
+  strcpy(buf1, s);
+  for (p = buf1; *p; p++)
+    *p = DOWNCASE(*p);
+  buf1[0] = UPCASE(buf1[0]);
+  return buf1;
+}
+
+/** Return an uppercased version of a string in a static buffer.
+ * \param s string to uppercase.
+ * \return pointer to a static buffer containing the uppercased version.
+ */
 char *
 strupper(const char *s)
 {
@@ -155,6 +216,10 @@ strupper(const char *s)
   return buf1;
 }
 
+/** Return a lowercased version of a string in a static buffer.
+ * \param s string to lowercase.
+ * \return pointer to a static buffer containing the lowercased version.
+ */
 char *
 strlower(const char *s)
 {
@@ -171,19 +236,31 @@ strlower(const char *s)
   return buf1;
 }
 
+/** Modify a string in-place to uppercase.
+ * \param s string to uppercase.
+ * \return s, now modified to be all uppercase.
+ */
 char *
 upcasestr(char *s)
 {
-  /* modifies a string in-place to be upper-case */
   char *p;
   for (p = s; p && *p; p++)
     *p = UPCASE(*p);
   return s;
 }
 
+/** Safely add an accented string to a buffer.
+ * \param base base string to which accents are applied.
+ * \param tmplate accent template string.
+ * \param len length of base (and tmplate).
+ * \param buff pointer to buffer to store accented string.
+ * \param bp pointer to pointer to insertion point in buff.
+ * \retval 1 failed to store entire string.
+ * \retval 0 success.
+ */
 int
-safe_accent(const char *base, const char *tmplate, size_t len, char *buff,
-	    char **bp)
+safe_accent(const char *RESTRICT base, const char *RESTRICT tmplate, size_t len,
+	    char *buff, char **bp)
 {
   /* base and tmplate must be the same length */
   size_t n;
@@ -211,6 +288,10 @@ safe_accent(const char *base, const char *tmplate, size_t len, char *buff,
       case 'o':
 	c = 197;
 	break;
+      case 'e':
+      case 'E':
+	c = 198;
+	break;
       default:
 	c = 'A';
       }
@@ -235,6 +316,10 @@ safe_accent(const char *base, const char *tmplate, size_t len, char *buff,
       case 'o':
 	c = 229;
 	break;
+      case 'e':
+      case 'E':
+	c = 230;
+	break;
       default:
 	c = 'a';
       }
@@ -493,10 +578,11 @@ safe_accent(const char *base, const char *tmplate, size_t len, char *buff,
 }
 
 
-/* Macros used by the safe_XXX functions to append a
- *       string c to the end of buff, starting at *bp.
-*/
+/** Define the args used in APPEND_TO_BUF */
 #define APPEND_ARGS int len, blen, clen
+/** Append a string c to the end of buff, starting at *bp.
+ * This macro is used by the safe_XXX functions.
+ */
 #define APPEND_TO_BUF \
   /* Trivial cases */  \
   if (c[0] == '\0') \
@@ -517,8 +603,15 @@ safe_accent(const char *base, const char *tmplate, size_t len, char *buff,
   return len - clen
 
 
+/** Safely store a formatted string into a buffer.
+ * This is a better way to do safe_str(tprintf(fmt,...),buff,bp)
+ * \param buff buffer to store formatted string.
+ * \param bp pointer to pointer to insertion point in buff.
+ * \param fmt format string.
+ * \return number of characters left over, or 0 for success.
+ */
 int
-safe_format(char *buff, char **bp, const char *fmt, ...)
+safe_format(char *buff, char **bp, const char *RESTRICT fmt, ...)
 {
   APPEND_ARGS;
 #ifdef HAS_VSNPRINTF
@@ -541,38 +634,59 @@ safe_format(char *buff, char **bp, const char *fmt, ...)
   APPEND_TO_BUF;
 }
 
+/** Safely store an integer into a buffer.
+ * \param i integer to store.
+ * \param buff buffer to store into.
+ * \param bp pointer to pointer to insertion point in buff.
+ * \return 0 on success, non-zero on failure.
+ */
 int
 safe_integer(int i, char *buff, char **bp)
 {
-  /* Adds a int to a string, being careful not to overflow buffer */
-  return format_long(i, buff, bp, BUFFER_LEN);
+  return format_long(i, buff, bp, BUFFER_LEN, 10);
 }
 
+/** Safely store an unsigned integer into a buffer.
+ * \param i integer to store.
+ * \param buff buffer to store into.
+ * \param bp pointer to pointer to insertion point in buff.
+ * \return 0 on success, non-zero on failure.
+ */
 int
 safe_uinteger(unsigned int i, char *buff, char **bp)
 {
-  /* Adds an unsigned int to a string */
   return safe_str(unparse_uinteger(i), buff, bp);
 }
 
+/** Safely store an unsigned integer into a short buffer.
+ * \param i integer to store.
+ * \param buff buffer to store into.
+ * \param bp pointer to pointer to insertion point in buff.
+ * \return 0 on success, non-zero on failure.
+ */
 int
 safe_integer_sbuf(int i, char *buff, char **bp)
 {
-  /* Adds a int to a string, being careful not to overflow buffer */
-  return format_long(i, buff, bp, SBUF_LEN);
+  return format_long(i, buff, bp, SBUF_LEN, 10);
 }
 
+/** Safely store a dbref into a buffer.
+ * Don't store partial dbrefs.
+ * \param d dbref to store.
+ * \param buff buffer to store into.
+ * \param bp pointer to pointer to insertion point in buff.
+ * \retval 0 success.
+ * \retval 1 failure.
+ */
 int
 safe_dbref(dbref d, char *buff, char **bp)
 {
-  /* Adds a dbref to a string, being careful not to overflow buffer and 
-     avoiding writing partial dbrefs */
   char *saved = *bp;
   if (safe_chr('#', buff, bp)) {
     *bp = saved;
     return 1;
   }
-  if (format_long(d, buff, bp, BUFFER_LEN)) {
+  if (format_long(d, buff, bp, BUFFER_LEN, 10)) {
     *bp = saved;
     return 1;
   }
@@ -580,35 +694,48 @@ safe_dbref(dbref d, char *buff, char **bp)
 }
 
 
+/** Safely store a number into a buffer.
+ * \param n number to store.
+ * \param buff buffer to store into.
+ * \param bp pointer to pointer to insertion point in buff.
+ * \retval 0 success.
+ * \retval 1 failure.
+ */
 int
 safe_number(NVAL n, char *buff, char **bp)
 {
-  /* Adds a number to a string, being careful not to overflow buffer */
   const char *c;
   APPEND_ARGS;
-
   c = unparse_number(n);
   APPEND_TO_BUF;
 }
 
+/** Safely store a string into a buffer.
+ * \param c string to store.
+ * \param buff buffer to store into.
+ * \param bp pointer to pointer to insertion point in buff.
+ * \retval 0 success.
+ * \retval 1 failure.
+ */
 int
 safe_str(const char *c, char *buff, char **bp)
 {
-  /* copies a string into a buffer, making sure there's no overflow. */
   APPEND_ARGS;
-
   if (!c || !*c)
     return 0;
-
   APPEND_TO_BUF;
-
 }
 
+/** Safely store a string into a buffer, quoting it if it contains a space.
+ * \param c string to store.
+ * \param buff buffer to store into.
+ * \param bp pointer to pointer to insertion point in buff.
+ * \retval 0 success.
+ * \retval 1 failure.
+ */
 int
 safe_str_space(const char *c, char *buff, char **bp)
 {
-  /* copies a string into a buffer, making sure there's no overflow.
-   * puts it in quotes if there's space in it. */
   APPEND_ARGS;
   char *saved = *bp;
 
@@ -628,6 +755,15 @@ safe_str_space(const char *c, char *buff, char **bp)
 }
 
 
+/** Safely store a string of known length into a buffer
+ * This is an optimization of safe_str for when we know the string's length.
+ * \param s string to store.
+ * \param len length of s.
+ * \param buff buffer to store into.
+ * \param bp pointer to pointer to insertion point in buff.
+ * \retval 0 success.
+ * \retval 1 failure.
+ */
 int
 safe_strl(const char *s, int len, char *buff, char **bp)
 {
@@ -650,6 +786,14 @@ safe_strl(const char *s, int len, char *buff, char **bp)
   return len - clen;
 }
 
+/** Safely fill a string with a given character a given number of times.
+ * \param x character to fill with.
+ * \param n number of copies of character to fill in.
+ * \param buff buffer to store into.
+ * \param bp pointer to pointer to insertion point in buff.
+ * \retval 0 success.
+ * \retval 1 failure (filled to end of buffer, but more was requested).
+ */
 int
 safe_fill(char x, size_t n, char *buff, char **bp)
 {
@@ -664,7 +808,7 @@ safe_fill(char x, size_t n, char *buff, char **bp)
   if (n > BUFFER_LEN - 1)
     n = BUFFER_LEN - 1;
 
-  blen = BUFFER_LEN - (*bp - buff);
+  blen = BUFFER_LEN - (*bp - buff) - 1;
 
   if (blen < n) {
     n = blen;
@@ -681,53 +825,73 @@ safe_fill(char x, size_t n, char *buff, char **bp)
 
 /* skip_space and seek_char are essentially right out of the 2.0 code */
 
+/** Return a pointer to the next non-space character in a string, or NULL.
+ * We return NULL if given a null string or a string with only spaces.
+ * \param s string to search for non-spaces.
+ * \return pointer to next non-space character in s.
+ */
 char *
 skip_space(const char *s)
 {
-  /* returns pointer to the next non-space char in s, or NULL if s == NULL
-   * or *s == NULL or s has only spaces.
-   */
-
   char *c = (char *) s;
   while (c && *c && isspace((unsigned char) *c))
     c++;
   return c;
 }
 
+/** Return a pointer to next char in s which matches c, or to the terminating
+ * null at the end of s.
+ * \param s string to search.
+ * \param c character to search for.
+ * \return pointer to next occurence of c or to the end of s.
+ */
 char *
 seek_char(const char *s, char c)
 {
-  /* similar to strchr(). returns a pointer to the next char in s which
-   * matches c, or a pointer to the terminating null at the end of s.
-   */
-
   char *p = (char *) s;
   while (p && *p && (*p != c))
     p++;
   return p;
 }
 
+/** Unsigned char version of strlen.
+ * \param s string.
+ * \return length of s.
+ */
 int
 u_strlen(const unsigned char *s)
 {
   return strlen((const char *) s);
 }
 
+/** Unsigned char version of strcpy. Equally dangerous.
+ * \param target destination for copy.
+ * \param source string to copy.
+ * \return pointer to copy.
+ */
 unsigned char *
 u_strcpy(unsigned char *target, const unsigned char *source)
 {
   return (unsigned char *) strcpy((char *) target, (const char *) source);
 }
 
+/** Search for all copies of old in string, and replace each with newbit.
+ * The replaced string is returned, newly allocated.
+ * \param old string to find.
+ * \param newbit string to replace old with.
+ * \param string string to search for old in.
+ * \return allocated string with replacements performed.
+ */
 char *
-replace_string(const char *old, const char *newbit, const char *string)
+replace_string(const char *RESTRICT old, const char *RESTRICT newbit,
+	       const char *RESTRICT string)
 {
   char *result, *r;
-  Size_t len, newlen;
+  size_t len, newlen;
 
   r = result = mush_malloc(BUFFER_LEN, "replace_string.buff");
   if (!result)
-    panic(T("Couldn't allocate memory in replace_string!"));
+    mush_panic(T("Couldn't allocate memory in replace_string!"));
 
   len = strlen(old);
   newlen = strlen(newbit);
@@ -747,24 +911,34 @@ replace_string(const char *old, const char *newbit, const char *string)
   return result;
 }
 
+/** Standard replacer tokens for text and position */
 const char *standard_tokens[2] = { "##", "#@" };
 
 /* Replace two tokens in a string at once. All-around better than calling
  * replace_string() twice
  */
+/** Search for all copies of two old strings, and replace each with a 
+ * corresponding newbit.
+ * The replaced string is returned, newly allocated.
+ * \param old array of two strings to find.
+ * \param newbits array of two strings to replace old with.
+ * \param string string to search for old.
+ * \return allocated string with replacements performed.
+ */
 char *
-replace_string2(const char *old[2], const char *newbits[2], const char *string)
+replace_string2(const char *old[2], const char *newbits[2],
+		const char *RESTRICT string)
 {
   char *result, *rp;
   char firsts[3] = { '\0', '\0', '\0' };
-  Size_t oldlens[2], newlens[2];
+  size_t oldlens[2], newlens[2];
 
   if (!string)
     return NULL;
 
   rp = result = mush_malloc(BUFFER_LEN, "replace_string.buff");
   if (!result)
-    panic(T("Couldn't allocate memory in replace_string2!"));
+    mush_panic(T("Couldn't allocate memory in replace_string2!"));
 
   firsts[0] = old[0][0];
   firsts[1] = old[1][0];
@@ -775,20 +949,22 @@ replace_string2(const char *old[2], const char *newbits[2], const char *string)
   newlens[1] = strlen(newbits[1]);
 
   while (*string) {
-    Size_t skip = strcspn(string, firsts);
+    size_t skip = strcspn(string, firsts);
     if (skip) {
       safe_strl(string, skip, result, &rp);
       string += skip;
     }
-    if (strncmp(string, old[0], oldlens[0]) == 0) {	/* Copy the first */
-      safe_strl(newbits[0], newlens[0], result, &rp);
-      string += oldlens[0];
-    } else if (strncmp(string, old[1], oldlens[1]) == 0) {	/* The second */
-      safe_strl(newbits[1], newlens[1], result, &rp);
-      string += oldlens[1];
-    } else {
-      safe_chr(*string, result, &rp);
-      string++;
+    if (*string) {
+      if (strncmp(string, old[0], oldlens[0]) == 0) {	/* Copy the first */
+	safe_strl(newbits[0], newlens[0], result, &rp);
+	string += oldlens[0];
+      } else if (strncmp(string, old[1], oldlens[1]) == 0) {	/* The second */
+	safe_strl(newbits[1], newlens[1], result, &rp);
+	string += oldlens[1];
+      } else {
+	safe_chr(*string, result, &rp);
+	string++;
+      }
     }
   }
 
@@ -797,6 +973,12 @@ replace_string2(const char *old[2], const char *newbits[2], const char *string)
 
 }
 
+/** Given a string and a separator, trim leading and trailing spaces
+ * if the separator is a space. This destructively modifies the string.
+ * \param str string to trim.
+ * \param sep separator character.
+ * \return pointer to (trimmed) string.
+ */
 char *
 trim_space_sep(char *str, char sep)
 {
@@ -816,6 +998,12 @@ trim_space_sep(char *str, char sep)
   return str;
 }
 
+/** Find the start of the next token in a string.
+ * If the separator is a space, we magically skip multiple spaces.
+ * \param str the string.
+ * \param sep the token separator character.
+ * \return pointer to start of next token in string.
+ */
 char *
 next_token(char *str, char sep)
 {
@@ -833,14 +1021,17 @@ next_token(char *str, char sep)
   return str;
 }
 
+/** Split out the next token from a string, destructively modifying it.
+ * As usually, if the separator is a space, we skip multiple spaces.
+ * The string's address is update to be past the token, and the token
+ * is returned. This code from TinyMUSH 2.0.
+ * \param sp pointer to string to split from.
+ * \param sep token separator.
+ * \return pointer to token, now null-terminated.
+ */
 char *
 split_token(char **sp, char sep)
 {
-  /* Get next token from string as a null-terminated string, depending
-   * on the separator character. This destructively modifies the string.
-   * Code from 2.0.
-   */
-
   char *str, *save;
 
   save = str = *sp;
@@ -863,10 +1054,14 @@ split_token(char **sp, char sep)
   return save;
 }
 
+/** Count the number of tokens in a string.
+ * \param str string to count.
+ * \param sep token separator.
+ * \return number of tokens in str.
+ */
 int
 do_wordcount(char *str, char sep)
 {
-  /* count the number of words in a string */
   int n;
 
   if (!*str)
@@ -875,26 +1070,10 @@ do_wordcount(char *str, char sep)
   return n;
 }
 
-/* Pretty much straight from Tiny 2.2, we see if str and target
- * match in the first min chars
+/** A version of strlen that ignores ansi and HTML sequences.
+ * \param p string to get length of.
+ * \return length of string p, not including ansi/html sequences.
  */
-int
-minmatch(const char *str, const char *target, int min)
-{
-  while (*str && *target && (DOWNCASE(*str) == DOWNCASE(*target))) {
-    str++;
-    target++;
-    min--;
-  }
-  if (*str)
-    return 0;
-  if (!*target)
-    return 1;
-  return ((min <= 0) ? 1 : 0);
-}
-
-/* Strlen that ignores ansi and HTML sequences */
-
 int
 ansi_strlen(const char *p)
 {
@@ -903,9 +1082,6 @@ ansi_strlen(const char *p)
   if (!p)
     return 0;
 
-  if (!ANSI_JUSTIFY)
-    return strlen(p);
-
   while (*p) {
     if (*p == ESC_CHAR) {
       while ((*p) && (*p != 'm'))
@@ -921,7 +1097,12 @@ ansi_strlen(const char *p)
   return i;
 }
 
-/* Returns true length of string up to numchars visible characters. 
+/** Returns the apparent length of a string, up to numchars visible 
+ * characters. The apparent length skips over nonprinting ansi and
+ * tags.
+ * \param p string.
+ * \param numchars maximum size to report.
+ * \return apparent length of string.
  */
 int
 ansi_strnlen(const char *p, size_t numchars)
@@ -949,8 +1130,13 @@ ansi_strnlen(const char *p, size_t numchars)
   return i;
 }
 
-/* Given a string, a word, and a separator, remove first occurence
+/** Given a string, a word, and a separator, remove first occurence
  * of the word from the string. Destructive.
+ * \param list a string containing a separated list.
+ * \param word a word to remove from the list.
+ * \param sep the separator between list items.
+ * \return pointer to static buffer containing list without first occurence
+ * of word.
  */
 char *
 remove_word(char *list, char *word, char sep)
@@ -980,6 +1166,12 @@ remove_word(char *list, char *word, char sep)
   return buff;
 }
 
+/** Return the next name in a list. A name may be a single word, or
+ * a quoted string. This is used by things like page/list. The list's
+ * pointer is advanced to the next name in the list.
+ * \param head pointer to pointer to string of names.
+ * \return pointer to static buffer containing next name.
+ */
 char *
 next_in_list(const char **head)
 {
@@ -1011,7 +1203,14 @@ next_in_list(const char **head)
 
 }
 
-/* Strip all ansi and html markup from a string */
+/** Strip all ansi and html markup from a string. As a side effect,
+ * stores the length of the stripped string in a provided address.
+ * NOTE! Length returned is length *including* the terminating NULL,
+ * because we usually memcpy the result.
+ * \param orig string to strip.
+ * \param s_len address to store length of stripped string, if provided.
+ * \return pointer to static buffer containing stripped string.
+ */
 char *
 remove_markup(const char *orig, size_t * s_len)
 {
@@ -1020,8 +1219,11 @@ remove_markup(const char *orig, size_t * s_len)
   const char *q;
   size_t len = 0;
 
-  if (!orig)
+  if (!orig) {
+    if (s_len)
+      *s_len = 0;
     return NULL;
+  }
 
   for (q = orig; *q;) {
     switch (*q) {
@@ -1045,16 +1247,20 @@ remove_markup(const char *orig, size_t * s_len)
 }
 
 
-/* Append an int to the string. Returns a true value on failure.
+/** Safely append an int to a string. Returns a true value on failure.
  * This will someday take extra arguments for use with our version 
  * of snprintf. Please try not to use it.
+ * maxlen = total length of string.
+ * buf[maxlen - 1] = place where \0 will go.
+ * buf[maxlen - 2] = last visible character.
+ * \param val value to append.
+ * \param buff string to append to.
+ * \param bp pointer to pointer to insertion point in buff.
+ * \param maxlen total length of string. 
+ * \param base the base to render the number in.
  */
-/* maxlen = total length of string.
-   buf[maxlen - 1] = place where \0 will go.
-   buf[maxlen - 2] = last visible character.
-*/
-static int
-format_long(long val, char *buff, char **bp, int maxlen)
+int
+format_long(long val, char *buff, char **bp, int maxlen, int base)
 {
   char stack[128];		/* Even a negative 64 bit number will only be 21
 				   digits or so max. This should be plenty of
@@ -1062,6 +1268,7 @@ format_long(long val, char *buff, char **bp, int maxlen)
   char *current;
   int size = 0, neg = 0;
   ldiv_t r;
+  const char *digits = "0123456789abcdefghijklmnopqrstuvwxyz";
 
   /* Sanity checks */
   if (!bp || !buff || !*bp)
@@ -1069,14 +1276,28 @@ format_long(long val, char *buff, char **bp, int maxlen)
   if (*bp - buff >= maxlen - 1)
     return 1;
 
+  if (base < 2)
+    base = 2;
+  else if (base > 36)
+    base = 36;
+
   if (val < 0) {
     neg = 1;
     val = -val;
     if (val < 0) {
       /* -LONG_MIN == LONG_MIN on 2's complement systems. Take the
          easy way out since this value is rarely encountered. */
-      sprintf(stack, "%ld", val);
-      return safe_str(stack, buff, bp);
+      switch (base) {
+      case 10:
+	return safe_format(buff, bp, "%ld", val);
+      case 16:
+	return safe_format(buff, bp, "%lx", val);
+      case 8:
+	return safe_format(buff, bp, "%lo", val);
+      default:
+	/* Weird base /and/ LONG_MIN. Fix someday. */
+	return 0;
+      }
     }
 
   }
@@ -1090,8 +1311,8 @@ format_long(long val, char *buff, char **bp, int maxlen)
     /* ldiv(x, y) does x/y and x%y at the same time (both of
      * which we need).
      */
-    r = ldiv(r.quot, 10);
-    *(--current) = (unsigned char) r.rem + '0';
+    r = ldiv(r.quot, base);
+    *(--current) = digits[r.rem];
   } while (r.quot);
 
   /* Add the negative sign if needed. */
@@ -1140,6 +1361,14 @@ format_long(long val, char *buff, char **bp, int maxlen)
 }
 
 #if defined(HAS_STRXFRM) && !defined(WIN32)
+/** A locale-sensitive strncmp.
+ * \param s1 first string to compare.
+ * \param s2 second string to compare.
+ * \param t number of characters to compare.
+ * \retval -1 s1 collates before s2.
+ * \retval 0 s1 collates the same as s2.
+ * \retval 1 s1 collates after s2.
+ */
 int
 strncoll(const char *s1, const char *s2, size_t t)
 {
@@ -1161,11 +1390,20 @@ strncoll(const char *s1, const char *s2, size_t t)
   (void) strxfrm(d1, ns1, s1_len);
   (void) strxfrm(d2, ns2, s2_len);
   result = strcmp(d1, d2);
+  mush_free(ns1, "string");
+  mush_free(ns2, "string");
   mush_free(d1, "string");
   mush_free(d2, "string");
   return result;
 }
 
+/** A locale-sensitive strcasecmp.
+ * \param s1 first string to compare.
+ * \param s2 second string to compare.
+ * \retval -1 s1 collates before s2.
+ * \retval 0 s1 collates the same as s2.
+ * \retval 1 s1 collates after s2.
+ */
 int
 strcasecoll(const char *s1, const char *s2)
 {
@@ -1186,6 +1424,14 @@ strcasecoll(const char *s1, const char *s2)
   return result;
 }
 
+/** A locale-sensitive strncasecmp.
+ * \param s1 first string to compare.
+ * \param s2 second string to compare.
+ * \param t number of characters to compare.
+ * \retval -1 s1 collates before s2.
+ * \retval 0 s1 collates the same as s2.
+ * \retval 1 s1 collates after s2.
+ */
 int
 strncasecoll(const char *s1, const char *s2, size_t t)
 {
@@ -1207,12 +1453,18 @@ strncasecoll(const char *s1, const char *s2, size_t t)
   (void) strxfrm(d1, strupper(ns1), s1_len);
   (void) strxfrm(d2, strupper(ns2), s2_len);
   result = strcmp(d1, d2);
+  mush_free(ns1, "string");
+  mush_free(ns2, "string");
   mush_free(d1, "string");
   mush_free(d2, "string");
   return result;
 }
 #endif				/* HAS_STRXFRM && !WIN32 */
 
+/** Return a string pointer past any ansi/html markup at the start.
+ * \param p a string.
+ * \return pointer to string after any initial ansi/html markup.
+ */
 char *
 skip_leading_ansi(const char *p)
 {
@@ -1233,12 +1485,19 @@ skip_leading_ansi(const char *p)
 
 }
 
+/** Convert a string into an ansi_string.
+ * This takes a string that may contain ansi/html markup codes and
+ * converts it to an ansi_string structure that separately stores
+ * the plain string and the markup codes for each character.
+ * \param src string to parse.
+ * \return pointer to an ansi_string structure representing the src string.
+ */
 ansi_string *
 parse_ansi_string(const char *src)
 {
   ansi_string *data;
   char *y, *current = NULL;
-  Size_t p = 0;
+  size_t p = 0;
 
   if (!src)
     return NULL;
@@ -1280,8 +1539,12 @@ parse_ansi_string(const char *src)
 }
 
 
+/** Fill up an ansi_string with codes so that when a code starts it
+ * applies to all the following characters until there's a new code.
+ * \param as pointer to an ansi_string to populate codes in.
+ */
 void
-populate_codes(ansi_string * as)
+populate_codes(ansi_string *as)
 {
   size_t p;
   char *current = NULL;
@@ -1303,8 +1566,12 @@ populate_codes(ansi_string * as)
     mush_free(current, "markup_codes");
 }
 
+/** Strip out codes from an ansi_string, leaving in only the codes where
+ * they change.
+ * \param as pointer to an ansi_string.
+ */
 void
-depopulate_codes(ansi_string * as)
+depopulate_codes(ansi_string *as)
 {
   size_t p, m;
   int normal = 1;
@@ -1341,6 +1608,7 @@ depopulate_codes(ansi_string * as)
 static int is_ansi_code(const char *s);
 static int is_start_html_code(const char *s) __attribute__ ((__unused__));
 static int is_end_html_code(const char *s);
+/** Is s a string that signifies the end of ANSI codes? */
 #define is_end_ansi_code(s) (!strcmp((s),ANSI_NORMAL))
 
 
@@ -1362,8 +1630,11 @@ is_end_html_code(const char *s)
   return s && *s == TAG_START && *(s + 1) == '/';
 }
 
+/** Free an ansi_string.
+ * \param as pointer to ansi_string to free.
+ */
 void
-free_ansi_string(ansi_string * as)
+free_ansi_string(ansi_string *as)
 {
   int p;
 
@@ -1376,8 +1647,17 @@ free_ansi_string(ansi_string * as)
   mush_free(as, "ansi_string");
 }
 
+/** Safely append an ansi_string into a buffer as a real string.
+ * \param as pointer to ansi_string to append.
+ * \param start position in as to start copying from.
+ * \param len length in characters to copy from as.
+ * \param buff buffer to insert into.
+ * \param bp pointer to pointer to insertion point of buff.
+ * \retval 0 success.
+ * \retval 1 failure.
+ */
 int
-safe_ansi_string(ansi_string * as, size_t start, size_t len, char *buff,
+safe_ansi_string(ansi_string *as, size_t start, size_t len, char *buff,
 		 char **bp)
 {
   int p, q;
@@ -1412,7 +1692,6 @@ safe_ansi_string(ansi_string * as, size_t start, size_t len, char *buff,
 	in_html++;
       q--;
     }
-    p = q + 1;
     /* p is now pointing to the first starting code, and we know if we're
      * in ansi, html, or both. We also know how many html tags have been
      * opened.
@@ -1462,10 +1741,122 @@ safe_ansi_string(ansi_string * as, size_t start, size_t len, char *buff,
   return 0;
 }
 
-/* Given the current item number in a list, whether it's the last item
+/** Safely append an ansi_string into a buffer as a real string,
+ * with extra copying of starting tags (for wrap()/align()).
+ * \param as pointer to ansi_string to append.
+ * \param start position in as to start copying from.
+ * \param len length in characters to copy from as.
+ * \param buff buffer to insert into.
+ * \param bp pointer to pointer to insertion point of buff.
+ * \retval 0 success.
+ * \retval 1 failure.
+ */
+int
+safe_ansi_string2(ansi_string *as, size_t start, size_t len, char *buff,
+		  char **bp)
+{
+  int p, q;
+  int in_ansi = 0;
+  int in_html = 0;
+
+  if (!as)
+    return 1;
+
+  depopulate_codes(as);
+
+  if (start > as->len || len == 0 || as->len == 0)
+    return safe_str("", buff, bp);
+
+  /* Find the starting codes by working our way backward until we
+   * reach some opening codes, and then working our way back from there
+   * until we hit a non-opening code or non-code 
+   */
+  p = start;
+  while ((p >= 0) && (as->codes[p] == NULL))
+    p--;
+  /* p is now either <0 or pointing to a code */
+  if ((p >= 0) && !is_end_html_code(as->codes[p]) &&
+      !is_end_ansi_code(as->codes[p])) {
+    /* p is now pointing to a starting code */
+    q = p;
+    while ((q >= 0) && as->codes[q] && !is_end_html_code(as->codes[q]) &&
+	   !is_end_ansi_code(as->codes[q])) {
+      if (is_ansi_code(as->codes[q]))
+	in_ansi = 1;
+      else if (is_start_html_code(as->codes[q]))
+	in_html++;
+      q--;
+    }
+    /* p is now pointing to the first starting code, and we know if we're
+     * in ansi, html, or both. We also know how many html tags have been
+     * opened.
+     */
+    /* Except there's this one problem - Now we know it, we weren't
+     * doing anything with it.
+     */
+    for (q = q + 1; q <= p; q++) {
+      if (safe_str(as->codes[q], buff, bp))
+	return 1;
+    }
+  }
+
+  /* Copy the text. The right thing to do now would be to have a stack
+   * of open html tags and clear in_html once all of the tags have
+   * been closed. We don't quite do that, alas.
+   */
+  for (p = (int) start; p < (int) (start + len) && p < (int) as->len; p++) {
+    if (as->codes[p]) {
+      if (safe_str(as->codes[p], buff, bp))
+	return 1;
+      if (is_end_ansi_code(as->codes[p]))
+	in_ansi = 0;
+      else if (is_ansi_code(as->codes[p]))
+	in_ansi = 1;
+      if (is_end_html_code(as->codes[p]))
+	in_html--;
+      else if (is_start_html_code(as->codes[p]))
+	in_html++;
+    }
+    if (safe_chr(as->text[p], buff, bp))
+      return 1;
+  }
+
+  /* Output (only) closing codes if needed. */
+  while (p <= (int) as->len) {
+    if (!in_ansi && !in_html)
+      break;
+    if (as->codes[p]) {
+      if (is_end_ansi_code(as->codes[p])) {
+	in_ansi = 0;
+	if (safe_str(as->codes[p], buff, bp))
+	  return 1;
+      } else if (is_end_html_code(as->codes[p])) {
+	in_html--;
+	if (safe_str(as->codes[p], buff, bp))
+	  return 1;
+      }
+    }
+    p++;
+  }
+  if (in_ansi)
+    safe_str(ANSI_NORMAL, buff, bp);
+  return 0;
+}
+
+
+/** Safely append a list item to a buffer, possibly with punctuation
+ * and conjunctions.
+ * Given the current item number in a list, whether it's the last item
  * in the list, the list's output separator, a conjunction,
  * and a punctuation mark to use between items, store the appropriate
  * inter-item stuff into the given buffer safely.
+ * \param cur_num current item number of the item to append.
+ * \param done 1 if this is the final item.
+ * \param delim string to insert after most items (comma).
+ * \param conjoin string to insert before last time ("and").
+ * \param space output delimiter.
+ * \param buff buffer to append to.
+ * \param bp pointer to pointer to insertion point in buff.
  */
 void
 safe_itemizer(int cur_num, int done, const char *delim, const char *conjoin,
@@ -1489,3 +1880,48 @@ safe_itemizer(int cur_num, int done, const char *delim, const char *conjoin,
   safe_str(space, buff, bp);
 
 }
+
+/** Return a stringified time in a static buffer
+ * Just like ctime() except without the trailing newlines.
+ * \param t the time to format.
+ * \param utc true if the time should be displayed in UTC, 0 for local time zone.
+ * \return a pointer to a static buffer with the stringified time.
+ */
+char *
+show_time(time_t t, int utc)
+{
+  struct tm *when;
+
+  if (utc)
+    when = gmtime(&t);
+  else
+    when = localtime(&t);
+
+  return show_tm(when);
+}
+
+/** Return a stringified time in a static buffer
+ * Just like asctime() except without the trailing newlines.
+ * \param when the time to format.
+ * \return a pointer to a static buffer with the stringified time.
+ */
+char *
+show_tm(struct tm *when)
+{
+  static char buffer[BUFFER_LEN];
+  int p;
+
+  if (!when)
+    return NULL;
+
+  strcpy(buffer, asctime(when));
+
+  p = strlen(buffer) - 1;
+  if (buffer[p] == '\n')
+    buffer[p] = '\0';
+
+  if (buffer[8] == ' ')
+    buffer[8] = '0';
+
+  return buffer;
+}
