@@ -1,18 +1,17 @@
-/* This file defines the @version command. It's all by itself because
+/**
+ * \file version.c
+ *
+ * \brief Version.
+ *
+ * \verbatim
+ * This file defines the @version command. It's all by itself because
  * we want to rebuild this file at every compilation, so that the
  * BUILDDATE is correct
+ * \endverbatim
  */
 #include "config.h"
 #include "copyrite.h"
-
-#ifdef I_STRING
-#include <string.h>
-#else
-#include <strings.h>
-#endif
-#include <time.h>
 #include "conf.h"
-
 #include "externs.h"
 #include "version.h"
 #include "patches.h"
@@ -21,22 +20,20 @@
 #endif
 #include "confmagic.h"
 
-extern time_t start_time;	/* from bsd.c */
 void do_version(dbref player);
 
+/** The version command.
+ * \param player the enactor.
+ */
 void
-do_version(player)
-    dbref player;
+do_version(dbref player)
 {
-  char buff[BUFFER_LEN];
 
   notify_format(player, T("You are connected to %s"), MUDNAME);
-
-  strcpy(buff, ctime(&start_time));
-  buff[strlen(buff) - 1] = '\0';	/* eat the newline */
-  notify_format(player, T("Last restarted: %s"), buff);
-
-  notify_format(player, "%s", VERSION);
+  notify_format(player, T("Last restarted: %s"),
+		show_time(globals.start_time, 0));
+  notify_format(player, "PennMUSH version %s patchlevel %s %s", VERSION,
+		PATCHLEVEL, PATCHDATE);
 #ifdef PATCHES
   notify_format(player, "Patches: %s", PATCHES);
 #endif
