@@ -1,6 +1,10 @@
-
-/* warnings.c - check to make sure rooms and exits are the way they
- * should be. */
+/**
+ * \file warnings.c
+ *
+ * \brief Check topology and messages on PennMUSH objects and give warnings
+ *
+ *
+ */
 
 #include "config.h"
 
@@ -10,57 +14,60 @@
 
 #include "copyrite.h"
 #include "conf.h"
+#include "externs.h"
 #include "mushdb.h"
 #include "lock.h"
 #include "flags.h"
 #include "dbdefs.h"
-#include "externs.h"
 #include "match.h"
 #include "attrib.h"
 #include "confmagic.h"
 
-#ifdef USE_WARNINGS
 
-#define W_UNLOCKED      0x1
-#define W_LOCKED        0x2
+/* We might check for both locked and unlocked warnings if we can't
+ * figure out a lock.
+ */
+#define W_UNLOCKED      0x1	/**< Check for unlocked-object warnings */
+#define W_LOCKED        0x2	/**< Check for locked-object warnings */
 
-#define W_EXIT_ONEWAY   0x1
-#define W_EXIT_MULTIPLE 0x2
-#define W_EXIT_MSGS     0x4
-#define W_EXIT_DESC     0x8
-#define W_EXIT_UNLINKED 0x10
+#define W_EXIT_ONEWAY   0x1	/**< Find one-way exits */
+#define W_EXIT_MULTIPLE 0x2	/**< Find multiple exits to same place */
+#define W_EXIT_MSGS     0x4	/**< Find exits without messages */
+#define W_EXIT_DESC     0x8	/**< Find exits without descs */
+#define W_EXIT_UNLINKED 0x10	/**< Find unlinked exits */
 /* Space for more exit stuff */
-#define W_THING_MSGS    0x100
-#define W_THING_DESC    0x200
+#define W_THING_MSGS    0x100	/**< Find things without messages */
+#define W_THING_DESC    0x200	/**< Find things without descs */
 /* Space for more thing stuff */
-#define W_ROOM_DESC     0x1000
+#define W_ROOM_DESC     0x1000	/**< Find rooms without descs */
 /* Space for more room stuff */
-#define W_PLAYER_DESC   0x10000
+#define W_PLAYER_DESC   0x10000	/**< Find players without descs */
 
-#define W_LOCK_PROBS 0x100000
+#define W_LOCK_PROBS    0x100000	/**< Find bad locks */
 
 /* Groups of warnings */
-#define W_NONE          0
+#define W_NONE          0      /**< No warnings */
+/** Serious warnings only */
 #define W_SERIOUS       (W_EXIT_UNLINKED|W_THING_DESC|W_ROOM_DESC|W_PLAYER_DESC|W_LOCK_PROBS)
+/** Standard warnings: serious warnings plus others */
 #define W_NORMAL        (W_SERIOUS|W_EXIT_ONEWAY|W_EXIT_MULTIPLE|W_EXIT_MSGS)
+/** Extra warnings: standard warnings plus others */
 #define W_EXTRA         (W_NORMAL|W_THING_MSGS)
+/** All warnings */
 #define W_ALL           (W_EXTRA|W_EXIT_DESC)
 
-typedef long int warn_type;
 
-typedef struct a_tcheck tcheck;
-struct a_tcheck {
-  const char *name;
-  warn_type flag;
-};
+/** A structure representing a topology warning check. */
+typedef struct a_tcheck {
+  const char *name;	/**< Name of warning. */
+  warn_type flag;	/**< Bitmask of warning. */
+} tcheck;
 
 
-static int warning_lock_type(const struct boolexp *l);
-static void complain
-  (dbref player, dbref i, const char *name, const char *desc, ...)
+extern int warning_lock_type(const boolexp l);
+void complain(dbref player, dbref i, const char *name, const char *desc, ...)
   __attribute__ ((__format__(__printf__, 4, 5)));
-static void check_lock(dbref player, dbref i, const char *name,
-		       struct boolexp *be);
+extern void check_lock(dbref player, dbref i, const char *name, boolexp be);
 static void ct_generic(dbref player, dbref i, warn_type flags);
 static void ct_room(dbref player, dbref i, warn_type flags);
 static void ct_exit(dbref player, dbref i, warn_type flags);
@@ -89,24 +96,13 @@ static tcheck checklist[] = {
   {NULL, 0}
 };
 
-/* This is really simple-minded for efficiency. Basically, if it's
- * unlocked, it's unlocked. If it's locked to something starting with
- * a specific db#, it's locked. Anything else, and we don't know.
+/** Issue a warning about an object.
+ * \param player player to receive the warning notification.
+ * \param i object the warning is about.
+ * \param name name of the warnings.
+ * \param desc a formatting string for the warning message.
  */
-static int
-warning_lock_type(const struct boolexp *l)
-     /* 0== unlocked. 1== locked, 2== sometimes */
-{
-  if (l == TRUE_BOOLEXP)
-    return W_UNLOCKED;
-  if (l->type == BOOLEXP_CONST ||
-      l->type == BOOLEXP_CARRY ||
-      l->type == BOOLEXP_IS || l->type == BOOLEXP_OWNER)
-    return W_LOCKED;
-  return (W_LOCKED | W_UNLOCKED);
-}
-
-static void
+void
 complain(dbref player, dbref i, const char *name, const char *desc, ...)
 {
 #ifdef HAS_VSNPRINTF
@@ -132,52 +128,6 @@ complain(dbref player, dbref i, const char *name, const char *desc, ...)
   notify(player, buff);
 }
 
-static void
-check_lock(dbref player, dbref i, const char *name, struct boolexp *be)
-{
-  switch (be->type) {
-  case BOOLEXP_CONST:
-  case BOOLEXP_CARRY:
-  case BOOLEXP_IS:
-  case BOOLEXP_OWNER:
-    if (!GoodObject(be->thing) || IsGarbage(be->thing))
-      complain(player, i, "lock-checks",
-	       T("%s lock refers to garbage object"), name);
-    break;
-  case BOOLEXP_NOT:
-    check_lock(player, i, name, be->data.n);
-    break;
-  case BOOLEXP_OR:
-  case BOOLEXP_AND:
-    check_lock(player, i, name, be->data.sub.a);
-    check_lock(player, i, name, be->data.sub.b);
-    break;
-  case BOOLEXP_EVAL:
-    {
-      ATTR *a;
-      a = atr_get(i, be->data.atr_lock->name);
-      if (!a || !Can_Read_Attr(i, i, a))
-	complain(player, i, "lock-checks",
-		 T
-		 ("%s lock has eval-lock that uses a nonexistant attribute '%s'."),
-		 name, be->data.atr_lock->name);
-    }
-    break;
-  case BOOLEXP_IND:
-    if (!GoodObject(be->thing) || IsGarbage(be->thing))
-      complain(player, i, "lock-checks",
-	       T("%s lock refers to garbage object"), name);
-    else if (!(Can_Read_Lock(i, be->thing, be->data.ind_lock) &&
-	       getlock(be->thing, be->data.ind_lock) != TRUE_BOOLEXP))
-      complain(player, i, "lock-checks",
-	       T("%s lock has indirect lock to %s/%s that it can't read"),
-	       name, unparse_object(player, be->thing), be->data.ind_lock);
-    break;
-  default:
-    break;
-  }
-}
-
 static void
 ct_generic(dbref player, dbref i, warn_type flags)
 {
@@ -226,7 +176,7 @@ ct_exit(dbref player, dbref i, warn_type flags)
       complain(player, i, "exit-unlinked",
 	       T("Variable exit has no %s attribute"), var);
     else {
-      const char *x = uncompress(AL_STR(a));
+      const char *x = atr_value(a);
       if (!x || !*x)
 	complain(player, i, "exit-unlinked",
 		 T("Variable exit has empty %s attribute"), var);
@@ -308,6 +258,9 @@ ct_thing(dbref player, dbref i, warn_type flags)
   }
 }
 
+/** Set up the default warnings on an object.
+ * \param player object to set warnings on.
+ */
 void
 set_initial_warnings(dbref player)
 {
@@ -315,15 +268,18 @@ set_initial_warnings(dbref player)
   return;
 }
 
+/** Set warnings on an object.
+ * \verbatim
+ * This implements @warnings obj=warning list
+ * \endverbatim
+ * \param player the enactor.
+ * \param name name of object to set warnings on.
+ * \param warns list of warnings to set, space-separated.
+ */
 void
 do_warnings(dbref player, const char *name, const char *warns)
 {
   dbref thing;
-  int found = 0;
-  warn_type flags, negate_flags;
-  char tbuf1[BUFFER_LEN];
-  char *w, *s;
-  tcheck *c;
 
   switch (thing = match_result(player, name, NOTYPE, MAT_EVERYTHING)) {
   case NOTHING:
@@ -344,10 +300,29 @@ do_warnings(dbref player, const char *name, const char *warns)
     break;
   }
 
+  Warnings(thing) = parse_warnings(player, warns);
+  notify_format(player, T("@warnings set to %s"),
+		unparse_warnings(Warnings(thing)));
+}
+
+/** Given a list of warnings, return the bitmask that represents it.
+ * \param player dbref to report errors to, or NOTHING.
+ * \param warnings the string of warning names
+ * \return a warning bitmask
+ */
+warn_type
+parse_warnings(dbref player, const char *warnings)
+{
+  int found = 0;
+  warn_type flags, negate_flags;
+  char tbuf1[BUFFER_LEN];
+  char *w, *s;
+  tcheck *c;
+
   flags = W_NONE;
   negate_flags = W_NONE;
-  if (warns && *warns) {
-    strcpy(tbuf1, warns);
+  if (warnings && *warnings) {
+    strcpy(tbuf1, warnings);
     /* Loop through whatever's listed and add on those warnings */
     s = trim_space_sep(tbuf1, ' ');
     w = split_token(&s, ' ');
@@ -371,34 +346,25 @@ do_warnings(dbref player, const char *name, const char *warns)
 	}
       }
       /* At this point, we haven't matched any warnings. */
-      if (!found) {
+      if (!found && player != NOTHING) {
 	notify_format(player, T("Unknown warning: %s"), w);
       }
       w = split_token(&s, ' ');
     }
-    if (flags || !negate_flags) {
-      Warnings(thing) = (flags & ~negate_flags);
-    } else {
-      Warnings(thing) &= ~negate_flags;
-    }
-    notify_format(player, T("@warnings set to %s"), unparse_warnings(thing));
-    return;
-  }
+    return flags & ~negate_flags;
+  } else
+    return 0;
 }
 
-
-/* Given an object, return a string of warnings on it */
+/** Given a warning bitmask, return a string of warnings on it.
+ * \param warns the warnings.
+ * \return pointer to statically allocated string listing warnings.
+ */
 const char *
-unparse_warnings(dbref thing)
+unparse_warnings(warn_type warns)
 {
   static char tbuf1[BUFFER_LEN];
-  int warns, listsize, indexx;
-  warn_type the_flag;
-
-  warns = Warnings(thing);
-
-  if (!warns)
-    return "none";
+  int listsize, indexx;
 
   tbuf1[0] = '\0';
 
@@ -407,7 +373,7 @@ unparse_warnings(dbref thing)
 
   /* Point c at last non-null in checklist, and go backwards */
   for (indexx = listsize - 2; warns && (indexx >= 0); indexx--) {
-    the_flag = checklist[indexx].flag;
+    warn_type the_flag = checklist[indexx].flag;
     if (!(the_flag & ~warns)) {
       /* Which is to say:
        * if the bits set on the_flag is a subset of the bits set on warns
@@ -464,9 +430,9 @@ check_topology_on(dbref player, dbref i)
 }
 
 
-/* Loop through all objects and check their topology */
+/** Loop through all objects and check their topology.  */
 void
-run_topology()
+run_topology(void)
 {
   int ndone;
   for (ndone = 0; ndone < db_top; ndone++) {
@@ -476,6 +442,12 @@ run_topology()
   }
 }
 
+/** Wizard command to check all objects.
+ * \verbatim
+ * This implements @wcheck/all.
+ * \endverbatim
+ * \param player the enactor.
+ */
 void
 do_wcheck_all(dbref player)
 {
@@ -488,7 +460,9 @@ do_wcheck_all(dbref player)
   notify(player, T("Warning checks complete."));
 }
 
-/* wcheck on a specific player, by that player */
+/** Check warnings on a specific player by themselves.
+ * \param player player checking warnings on their objects.
+ */
 void
 do_wcheck_me(dbref player)
 {
@@ -504,8 +478,10 @@ do_wcheck_me(dbref player)
 }
 
 
-/* Called when a player wants to do a check on something. We check for
- * ownership or hasprivs before allowing it
+/** Check warnings on a specific object.
+ * We check for ownership or hasprivs before allowing this.
+ * \param player the enactor.
+ * \param name name of object to check.
  */
 void
 do_wcheck(dbref player, const char *name)
@@ -535,6 +511,3 @@ do_wcheck(dbref player, const char *name)
   notify(player, T("@wcheck complete."));
   return;
 }
-
-
-#endif				/* USE_WARNINGS */
