@@ -1,7 +1,10 @@
-/* wild.c - wildcard routines
+/**
+ * \file wild.c
+ *
+ * \brief Wildcard matching routings for PennMUSH
  *
  * Written by T. Alexander Popiel, 24 June 1993
- * Last modified by T. Alexander Popiel, 28 July 2000
+ * Last modified by Javelin, 2002-2003
  *
  * Thanks go to Andrew Molitor for debugging
  * Thanks also go to Rich $alz for code to benchmark against
@@ -17,50 +20,54 @@
  * then please feel free to contact the author at
  * popiel@wolfskeep.com to work out agreeable terms.
  */
+
 #include "config.h"
 #include <ctype.h>
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
-#ifdef I_STDLIB
 #include <stdlib.h>
-#endif
 
 #include "copyrite.h"
 #include "conf.h"
 #include "case.h"
 #include "externs.h"
-#include "memcheck.h"
 #include "mymalloc.h"
 #include "parse.h"
 #include "pcre.h"
 #include "confmagic.h"
 
+/** Force a char to be lowercase */
 #define FIXCASE(a) (DOWNCASE(a))
+/** Check for equality of characters, maybe case-sensitive */
 #define EQUAL(cs,a,b) ((cs) ? (a == b) : (FIXCASE(a) == FIXCASE(b)))
+/** Check for inequality of characters, maybe case-sensitive */
 #define NOTEQUAL(cs,a,b) ((cs) ? (a != b) : (FIXCASE(a) != FIXCASE(b)))
+/** Maximum number of wildcarded arguments */
 #define NUMARGS (10)
 
-const unsigned char *tables = NULL;
+const unsigned char *tables = NULL;  /** Pointer to character tables */
 
 static char wspace[3 * BUFFER_LEN + NUMARGS];	/* argument return buffer */
 						/* big to match tprintf */
 
 static int wild1
-  (const char *tstr, const char *dstr, int arg, char *wbuf, int cs);
-static int wild(const char *s, const char *d, int p, int cs);
-static int check_literals(const char *tstr, const char *dstr, int cs);
+  (const char *RESTRICT tstr, const char *RESTRICT dstr, int arg,
+   char *RESTRICT wbuf, int cs);
+static int wild(const char *RESTRICT s, const char *RESTRICT d, int p, int cs);
+static int check_literals(const char *RESTRICT tstr, const char *RESTRICT dstr,
+			  int cs);
 static char *strip_backslashes(const char *str);
 
-/* ---------------------------------------------------------------------------
- * quick_wild: do a wildcard match, without remembering the wild data.
+/** Do a wildcard match, without remembering the wild data.
  *
  * This routine will cause crashes if fed NULLs instead of strings.
+ *
+ * \param tstr pattern to match against.
+ * \param dstr string to check.
+ * \retval 1 dstr matches the tstr pattern.
+ * \retval 0 dstr does not match the tstr pattern.
  */
 int
-quick_wild(const char *tstr, const char *dstr)
+quick_wild(const char *RESTRICT tstr, const char *RESTRICT dstr)
 {
   /* quick_wild_new does the real work, but before we call it, 
    * we do some sanity checking. 
@@ -70,9 +77,18 @@ quick_wild(const char *tstr, const char *dstr)
   return quick_wild_new(tstr, dstr, 0);
 }
 
-
+/** Do a wildcard match, possibly case-sensitive, without memory.
+ *
+ * This probably crashes if fed NULLs instead of strings, too.
+ *
+ * \param tstr pattern to match against.
+ * \param dstr string to check.
+ * \param cs if 1, case-sensitive; if 0, case-insensitive.
+ * \retval 1 dstr matches the tstr pattern.
+ * \retval 0 dstr does not match the tstr pattern.
+ */
 int
-quick_wild_new(const char *tstr, const char *dstr, int cs)
+quick_wild_new(const char *RESTRICT tstr, const char *RESTRICT dstr, int cs)
 {
   while (*tstr != '*') {
     switch (*tstr) {
@@ -136,6 +152,114 @@ quick_wild_new(const char *tstr, const char *dstr, int cs)
   return 0;
 }
 
+/** Do an attribute name wildcard match.
+ *
+ * This probably crashes if fed NULLs instead of strings, too.
+ * The special thing about this one is that ` doesn't match normal
+ * wildcards; you have to use ** to match embedded `.  Also, patterns
+ * ending in ` are treated as patterns ending in `*, and empty patterns
+ * are treated as *.
+ *
+ * \param tstr pattern to match against.
+ * \param dstr string to check.
+ * \retval 1 dstr matches the tstr pattern.
+ * \retval 0 dstr does not match the tstr pattern.
+ */
+int
+atr_wild(const char *RESTRICT tstr, const char *RESTRICT dstr)
+{
+  int starcount;
+
+  if (!*tstr)
+    return !strchr(dstr, '`');
+
+  while (*tstr != '*') {
+    switch (*tstr) {
+    case '?':
+      /* Single character match.  Return false if at
+       * end of data.
+       */
+      if (!*dstr || *dstr == '`')
+	return 0;
+      break;
+    case '`':
+      /* Delimiter match.  Special handling if at end of pattern. */
+      if (*dstr != '`')
+	return 0;
+      if (!tstr[1])
+	return !strchr(dstr + 1, '`');
+      break;
+    case '\\':
+      /* Escape character.  Move up, and force literal
+       * match of next character.
+       */
+      tstr++;
+      /* FALL THROUGH */
+    default:
+      /* Literal character.  Check for a match.
+       * If matching end of data, return true.
+       */
+      if (NOTEQUAL(0, *dstr, *tstr))
+	return 0;
+      if (!*dstr)
+	return 1;
+    }
+    tstr++;
+    dstr++;
+  }
+
+  /* Skip over '*'. */
+  tstr++;
+  starcount = 1;
+
+  /* Skip over wildcards. */
+  while (starcount < 2 && ((*tstr == '?') || (*tstr == '*'))) {
+    if (*tstr == '?') {
+      if (!*dstr || *dstr == '`')
+	return 0;
+      dstr++;
+      starcount = 0;
+    } else
+      starcount++;
+    tstr++;
+  }
+
+  /* Skip over long strings of '*'. */
+  while (*tstr == '*')
+    tstr++;
+
+  /* Return true on trailing '**'. */
+  if (!*tstr)
+    return starcount == 2 || !strchr(dstr, '`');
+
+  if (*tstr == '?') {
+    /* Scan for possible matches. */
+    while (*dstr) {
+      if (*dstr != '`' && atr_wild(tstr + 1, dstr + 1))
+	return 1;
+      dstr++;
+    }
+  } else {
+    /* Skip over a backslash in the pattern string if it is there. */
+    if (*tstr == '\\')
+      tstr++;
+
+    /* Scan for possible matches. */
+    while (*dstr) {
+      if (EQUAL(0, *dstr, *tstr)) {
+	if (!*(tstr + 1) && *(dstr + 1))
+	  return 0;		/* No more in pattern string, but more in target */
+	if (atr_wild(tstr + 1, dstr + 1))
+	  return 1;
+      }
+      if (starcount < 2 && *dstr == '`')
+	return 0;
+      dstr++;
+    }
+  }
+  return 0;
+}
+
 /* ---------------------------------------------------------------------------
  * wild1: INTERNAL: do a wildcard match, remembering the wild data.
  *
@@ -146,12 +270,8 @@ quick_wild_new(const char *tstr, const char *dstr, int cs)
  * and what it points to.
  */
 static int
-wild1(tstr, dstr, arg, wbuf, cs)
-    const char *tstr;
-    const char *dstr;
-    int arg;
-    char *wbuf;
-    int cs;			/* Case sensitive? */
+wild1(const char *RESTRICT tstr, const char *RESTRICT dstr, int arg,
+      char *RESTRICT wbuf, int cs)
 {
   const char *datapos;
   char *wnext;
@@ -166,7 +286,7 @@ wild1(tstr, dstr, arg, wbuf, cs)
       if (!*dstr)
 	return 0;
 
-      wnxt[arg++] = wbuf;
+      global_eval_context.wnxt[arg++] = wbuf;
       *wbuf++ = *dstr;
       *wbuf++ = '\0';
 
@@ -196,7 +316,7 @@ wild1(tstr, dstr, arg, wbuf, cs)
 
   /* If at end of pattern, slurp the rest, and leave. */
   if (!tstr[1]) {
-    wnxt[arg] = wbuf;
+    global_eval_context.wnxt[arg] = wbuf;
     strcpy(wbuf, dstr);
     return 1;
   }
@@ -210,7 +330,7 @@ wild1(tstr, dstr, arg, wbuf, cs)
       /* Fill in arguments if someone put another '*'
        * before a fixed string.
        */
-      wnxt[argpos++] = wbuf;
+      global_eval_context.wnxt[argpos++] = wbuf;
       *wbuf++ = '\0';
 
       /* Jump to the fast routine if we can. */
@@ -219,7 +339,7 @@ wild1(tstr, dstr, arg, wbuf, cs)
 
       /* Fill in any intervening '?'s */
       while (argpos < arg) {
-	wnxt[argpos++] = wbuf;
+	global_eval_context.wnxt[argpos++] = wbuf;
 	*wbuf++ = *datapos++;
 	*wbuf++ = '\0';
 
@@ -272,7 +392,7 @@ wild1(tstr, dstr, arg, wbuf, cs)
   /* Found a match!  Fill in all remaining arguments.
    * First do the '*'...
    */
-  wnxt[argpos++] = wbuf;
+  global_eval_context.wnxt[argpos++] = wbuf;
   strncpy(wbuf, datapos, (dstr - datapos) - numextra);
   wbuf += (dstr - datapos) - numextra;
   *wbuf++ = '\0';
@@ -282,7 +402,7 @@ wild1(tstr, dstr, arg, wbuf, cs)
   while (numextra) {
     if (argpos >= NUMARGS)
       return 1;
-    wnxt[argpos++] = wbuf;
+    global_eval_context.wnxt[argpos++] = wbuf;
     *wbuf++ = *datapos++;
     *wbuf++ = '\0';
     numextra--;
@@ -302,11 +422,7 @@ wild1(tstr, dstr, arg, wbuf, cs)
  * Side Effect: this routine modifies the 'wnxt' global variable.
  */
 static int
-wild(s, d, p, cs)
-    const char *s;
-    const char *d;
-    int p;
-    int cs;			/* Case sensitive? */
+wild(const char *RESTRICT s, const char *RESTRICT d, int p, int cs)
 {
   /* Do fast match. */
   while ((*s != '*') && (*s != '?')) {
@@ -328,35 +444,40 @@ wild(s, d, p, cs)
   return wild1(s, d, p, wspace, cs);
 }
 
-/* ---------------------------------------------------------------------------
- * wild_match: do a wildcard
- * match, remembering the wild data, if wildcard match is done.
+/** Wildcard match, possibly case-sensitive, and remember the wild data.
  *
  * This routine will cause crashes if fed NULLs instead of strings.
+ *
+ * \param s pattern to match against.
+ * \param d string to check.
+ * \param cs if 1, case-sensitive; if 0, case-insensitive.
+ * \retval 1 d matches s.
+ * \retval 0 d doesn't match s.
  */
 int
-wild_match_case(s, d, cs)
-    const char *s;
-    const char *d;
-    int cs;			/* Case sensitive match? */
+wild_match_case(const char *RESTRICT s, const char *RESTRICT d, int cs)
 {
   int j;
   /* Clear %0-%9 and r(0) - r(9) */
   for (j = 0; j < NUMARGS; j++)
-    wnxt[j] = (char *) NULL;
+    global_eval_context.wnxt[j] = (char *) NULL;
   for (j = 0; j < NUMQ; j++)
-    rnxt[j] = (char *) NULL;
+    global_eval_context.rnxt[j] = (char *) NULL;
   return wild(s, d, 0, cs);
 }
 
-/* ---------------------------------------------------------------------------
- * regexp_match: do a regexp match, remembering the matched subexpressions
+/** Regexp match, possibly case-sensitive, and remember matched subexpressions.
+ *
+ * This routine will cause crashes if fed NULLs instead of strings.
+ *
+ * \param s regexp to match against.
+ * \param d string to check.
+ * \param cs if 1, case-sensitive; if 0, case-insensitive.
+ * \retval 1 d matches s.
+ * \retval 0 d doesn't match s.
  */
 int
-regexp_match_case(s, d, cs)
-    const char *s;
-    const char *d;
-    int cs;			/* Case sensitive? */
+regexp_match_case(const char *RESTRICT s, const char *RESTRICT d, int cs)
 {
   int j;
   pcre *re;
@@ -367,11 +488,6 @@ regexp_match_case(s, d, cs)
   int offsets[99];
   int subpatterns;
 
-  if (!tables) {
-    /* Initialize char tables so they match current locale. */
-    tables = pcre_maketables();
-  }
-
   if ((re = pcre_compile(s, (cs ? 0 : PCRE_CASELESS), &errptr, &erroffset,
 			 tables)) == NULL) {
     /*
@@ -381,9 +497,7 @@ regexp_match_case(s, d, cs)
      */
     return 0;
   }
-#ifdef MEM_CHECK
   add_check("pcre");
-#endif
   /* 
    * Now we try to match the pattern. The relevant fields will
    * automatically be filled in by this.
@@ -409,14 +523,14 @@ regexp_match_case(s, d, cs)
   /* Clear %0-%9 and r(0) - r(9) */
   for (j = 0; j < NUMARGS; j++) {
     wtmp[j][0] = '\0';
-    wnxt[j] = (char *) NULL;
+    global_eval_context.wnxt[j] = (char *) NULL;
   }
   for (j = 0; j < NUMQ; j++)
-    rnxt[j] = (char *) NULL;
+    global_eval_context.rnxt[j] = (char *) NULL;
 
   for (i = 0; (i < 10) && (i < NUMARGS); i++) {
     pcre_copy_substring(d, offsets, subpatterns, i, wtmp[i], BUFFER_LEN);
-    wnxt[i] = wtmp[i];
+    global_eval_context.wnxt[i] = wtmp[i];
   }
 
   mush_free(re, "pcre");
@@ -424,29 +538,33 @@ regexp_match_case(s, d, cs)
 }
 
 
-/* ---------------------------------------------------------------------------
- * quick_regexp_match: do a regexp match, not caring about saving matched
- * subexpressions.
+/** Regexp match, possibly case-sensitive, and with no memory.
+ *
+ * This routine will cause crashes if fed NULLs instead of strings.
+ *
+ * \param s regexp to match against.
+ * \param d string to check.
+ * \param cs if 1, case-sensitive; if 0, case-insensitive.
+ * \retval 1 d matches s.
+ * \retval 0 d doesn't match s.
  */
 int
-quick_regexp_match(s, d, cs)
-    const char *s;
-    const char *d;
-    int cs;			/* Case sensitive? */
+quick_regexp_match(const char *RESTRICT s, const char *RESTRICT d, int cs)
 {
   pcre *re;
   const char *errptr;
   int erroffset;
   int offsets[99];
   int r;
+  int flags = 0;		/* There's a PCRE_NO_AUTO_CAPTURE flag to turn all raw
+				   ()'s into (?:)'s, which would be nice to use,
+				   except that people might use backreferences in
+				   their patterns. Argh. */
 
-  if (!tables) {
-    /* Initialize char tables so they match current locale. */
-    tables = pcre_maketables();
-  }
+  if (!cs)
+    flags |= PCRE_CASELESS;
 
-  if ((re = pcre_compile(s, (cs ? 0 : PCRE_CASELESS), &errptr, &erroffset,
-			 tables)) == NULL) {
+  if ((re = pcre_compile(s, flags, &errptr, &erroffset, tables)) == NULL) {
     /*
      * This is a matching error. We have an error message in
      * errptr that we can ignore, since we're doing
@@ -454,9 +572,7 @@ quick_regexp_match(s, d, cs)
      */
     return 0;
   }
-#ifdef MEM_CHECK
   add_check("pcre");
-#endif
   /* 
    * Now we try to match the pattern. The relevant fields will
    * automatically be filled in by this.
@@ -469,17 +585,18 @@ quick_regexp_match(s, d, cs)
 }
 
 
-/* ---------------------------------------------------------------------------
- * local_wild_match: do either an order comparison or a wildcard match.
- * Ignore match correspondences.
+/** Either an order comparison or a wildcard match with no memory.
  *
  * This routine will cause crashes if fed NULLs instead of strings.
+ *
+ * \param s pattern to match against.
+ * \param d string to check.
+ * \param cs if 1, case-sensitive; if 0, case-insensitive.
+ * \retval 1 d matches s.
+ * \retval 0 d doesn't match s.
  */
 int
-local_wild_match_case(s, d, cs)
-    const char *s;
-    const char *d;
-    int cs;
+local_wild_match_case(const char *RESTRICT s, const char *RESTRICT d, int cs)
 {
   switch (*s) {
   case '>':
@@ -499,14 +616,14 @@ local_wild_match_case(s, d, cs)
   return quick_wild_new(s, d, cs);
 }
 
-/* ---------------------------------------------------------------------------
- * wildcard - return 1 if the string has a wildcard character (* or ?)
- * in it, and 0 otherwise. Not used by the wild matching routines, but
- * suitable for outside use.
+/** Does a string contain a wildcard character (* or ?)?
+ * Not used by the wild matching routines, but suitable for outside use.
+ * \param s string to check.
+ * \retval 1 s contains a * or ?
+ * \retval 0 s does not contain a * or ?
  */
 int
-wildcard(s)
-    const char *s;
+wildcard(const char *s)
 {
   if (strchr(s, '*') || strchr(s, '?'))
     return 1;
@@ -514,7 +631,7 @@ wildcard(s)
 }
 
 static int
-check_literals(const char *tstr, const char *dstr, int cs)
+check_literals(const char *RESTRICT tstr, const char *RESTRICT dstr, int cs)
 {
   /* Every literal string in tstr must appear, in order, in dstr,
    * or no match can happen. That is, tstr is the pattern and dstr
